@@ -1,9 +1,9 @@
 /*
- * "$Id: lpd.c,v 1.65 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: lpd.c 5886 2006-08-24 19:53:17Z mike $"
  *
  *   Line Printer Daemon backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -30,7 +30,7 @@
  *   lpd_queue()       - Queue a file using the Line Printer Daemon protocol.
  *   lpd_timeout()     - Handle timeout alarms...
  *   lpd_write()       - Write a buffer of data to an LPD server.
- *   rresvport()       - A simple implementation of rresvport().
+ *   rresvport_af()    - A simple implementation of rresvport_af().
  *   sigterm_handler() - Handle 'terminate' signals that stop the backend.
  */
 
@@ -38,12 +38,13 @@
  * Include necessary headers.
  */
 
+#include <cups/backend.h>
+#include <cups/http-private.h>
 #include <cups/cups.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <ctype.h>
-#include <cups/http-private.h>
 #include <cups/string.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -58,6 +59,10 @@
 #  include <arpa/inet.h>
 #  include <netdb.h>
 #endif /* WIN32 */
+#ifdef __APPLE__
+#  include <CoreFoundation/CFNumber.h>
+#  include <CoreFoundation/CFPreferences.h>
+#endif /* __APPLE__ */
 
 
 /*
@@ -65,6 +70,7 @@
  */
 
 static char	tmpfilename[1024] = "";	/* Temporary spool file name */
+static int	abort_job = 0;		/* Non-zero if we get SIGTERM */
 
 
 /*
@@ -84,13 +90,6 @@ static char	tmpfilename[1024] = "";	/* Temporary spool file name */
 #define RESERVE_ANY		2	/* Reserve port 1-1023 */
 
 
-/*
- * It appears that rresvport() is never declared on most systems...
- */
-
-extern int	rresvport(int *port);
-
-
 /*
  * Local functions...
  */
@@ -100,9 +99,12 @@ static int	lpd_queue(const char *hostname, int port, const char *printer,
 		          const char *filename,
 		          const char *user, const char *title, int copies,
 			  int banner, int format, int order, int reserve,
-			  int manual_copies, int timeout);
+			  int manual_copies, int timeout, int contimeout);
 static void	lpd_timeout(int sig);
 static int	lpd_write(int lpd_fd, char *buffer, int length);
+#ifndef HAVE_RRESVPORT_AF
+static int	rresvport_af(int *port, int family);
+#endif /* !HAVE_RRESVPORT_AF */
 static void	sigterm_handler(int sig);
 
 
@@ -137,6 +139,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   int			sanitize_title;	/* Sanitize title string? */
   int			manual_copies,	/* Do manual copies? */
 			timeout,	/* Timeout */
+			contimeout,	/* Connection timeout */
 			copies;		/* Number of copies */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction	action;		/* Actions for POSIX signals */
@@ -177,13 +180,13 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     puts("network lpd \"Unknown\" \"LPD/LPR Host or Printer\"");
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fprintf(stderr, "Usage: %s job-id user title copies options [file]\n",
             argv[0]);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
@@ -206,7 +209,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     if ((fd = cupsTempFd(tmpfilename, sizeof(tmpfilename))) < 0)
     {
       perror("ERROR: unable to create temporary file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     while ((bytes = fread(buffer, 1, sizeof(buffer), stdin)) > 0)
@@ -215,7 +218,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
         perror("ERROR: unable to write to temporary file");
 	close(fd);
 	unlink(tmpfilename);
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
 
     close(fd);
@@ -228,7 +231,10 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   * Extract the hostname and printer name from the URI...
   */
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
+  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
+                  method, sizeof(method), username, sizeof(username),
+		  hostname, sizeof(hostname), &port,
+		  resource, sizeof(resource));
 
   if (!username[0])
   {
@@ -243,18 +249,42 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   * See if there are any options...
   */
 
-  banner         = 0;
-  format         = 'l';
-  order          = ORDER_CONTROL_DATA;
-  reserve        = RESERVE_ANY;
-  manual_copies  = 1;
-  timeout        = 300;
-  sanitize_title = 1;
+  banner        = 0;
+  format        = 'l';
+  order         = ORDER_CONTROL_DATA;
+  reserve       = RESERVE_ANY;
+  manual_copies = 1;
+  timeout       = 300;
+  contimeout    = 7 * 24 * 60 * 60;
 
-#if defined(__APPLE__)
+#ifdef __APPLE__
   /* We want to pass utf-8 characters, not re-map them (3071945) */
-  sanitize_title= 0;
-#endif
+  sanitize_title = 0;
+
+  /* Get the default timeout from a system preference... */
+  {
+    CFPropertyListRef	pvalue;		/* Preference value */
+    SInt32		toval;		/* Timeout value */
+
+
+    pvalue = CFPreferencesCopyValue(CFSTR("timeout"),
+                                    CFSTR("com.apple.print.backends"),
+				    kCFPreferencesAnyUser,
+				    kCFPreferencesCurrentHost);
+    if (pvalue)
+    {
+      if (CFGetTypeID(pvalue) == CFNumberGetTypeID())
+      {
+	CFNumberGetValue(pvalue, kCFNumberSInt32Type, &toval);
+	contimeout = (int)toval;
+      }
+
+      CFRelease(pvalue);
+    }
+  }
+#else
+  sanitize_title = 1;
+#endif /* __APPLE__ */
 
   if ((options = strchr(resource, '?')) != NULL)
   {
@@ -288,12 +318,12 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
         options ++;
 
-	for (ptr = value; *options && *options != '+';)
+	for (ptr = value; *options && *options != '+' && *options != '&';)
           if (ptr < (value + sizeof(value) - 1))
             *ptr++ = *options++;
 	*ptr = '\0';
 
-	if (*options == '+')
+	if (*options == '+' || *options == '&')
 	  options ++;
       }
       else
@@ -303,51 +333,47 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       * Process the option...
       */
 
-      if (strcasecmp(name, "banner") == 0)
+      if (!strcasecmp(name, "banner"))
       {
        /*
         * Set the banner...
 	*/
 
-        banner = !value[0] ||
-	         strcasecmp(value, "on") == 0 ||
-		 strcasecmp(value, "yes") == 0 ||
-		 strcasecmp(value, "true") == 0;
+        banner = !value[0] || !strcasecmp(value, "on") ||
+		 !strcasecmp(value, "yes") || !strcasecmp(value, "true");
       }
-      else if (strcasecmp(name, "format") == 0 && value[0])
+      else if (!strcasecmp(name, "format") && value[0])
       {
        /*
         * Set output format...
 	*/
 
-        if (strchr("cdfglnoprtv", value[0]) != NULL)
+        if (strchr("cdfglnoprtv", value[0]))
 	  format = value[0];
 	else
 	  fprintf(stderr, "ERROR: Unknown format character \"%c\"\n", value[0]);
       }
-      else if (strcasecmp(name, "order") == 0 && value[0])
+      else if (!strcasecmp(name, "order") && value[0])
       {
        /*
         * Set control/data order...
 	*/
 
-        if (strcasecmp(value, "control,data") == 0)
+        if (!strcasecmp(value, "control,data"))
 	  order = ORDER_CONTROL_DATA;
-	else if (strcasecmp(value, "data,control") == 0)
+	else if (!strcasecmp(value, "data,control"))
 	  order = ORDER_DATA_CONTROL;
 	else
 	  fprintf(stderr, "ERROR: Unknown file order \"%s\"\n", value);
       }
-      else if (strcasecmp(name, "reserve") == 0)
+      else if (!strcasecmp(name, "reserve"))
       {
        /*
         * Set port reservation mode...
 	*/
 
-        if (!value[0] ||
-	    !strcasecmp(value, "on") ||
-	    !strcasecmp(value, "yes") ||
-	    !strcasecmp(value, "true") ||
+        if (!value[0] || !strcasecmp(value, "on") ||
+	    !strcasecmp(value, "yes") || !strcasecmp(value, "true") ||
 	    !strcasecmp(value, "rfc1179"))
 	  reserve = RESERVE_RFC1179;
 	else if (!strcasecmp(value, "any"))
@@ -355,29 +381,25 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 	else
 	  reserve = RESERVE_NONE;
       }
-      else if (strcasecmp(name, "manual_copies") == 0)
+      else if (!strcasecmp(name, "manual_copies"))
       {
        /*
         * Set manual copies...
 	*/
 
-        manual_copies = !value[0] ||
-	        	strcasecmp(value, "on") == 0 ||
-	 		strcasecmp(value, "yes") == 0 ||
-	 		strcasecmp(value, "true") == 0;
+        manual_copies = !value[0] || !strcasecmp(value, "on") ||
+	 		!strcasecmp(value, "yes") || !strcasecmp(value, "true");
       }
-      else if (strcasecmp(name, "sanitize_title") == 0)
+      else if (!strcasecmp(name, "sanitize_title"))
       {
        /*
         * Set sanitize title...
 	*/
 
-        sanitize_title = !value[0] ||
-	        	strcasecmp(value, "on") == 0 ||
-	 		strcasecmp(value, "yes") == 0 ||
-	 		strcasecmp(value, "true") == 0;
+        sanitize_title = !value[0] || !strcasecmp(value, "on") ||
+	 		!strcasecmp(value, "yes") || !strcasecmp(value, "true");
       }
-      else if (strcasecmp(name, "timeout") == 0)
+      else if (!strcasecmp(name, "timeout"))
       {
        /*
         * Set the timeout...
@@ -386,6 +408,15 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 	if (atoi(value) > 0)
 	  timeout = atoi(value);
       }
+      else if (!strcasecmp(name, "contimeout"))
+      {
+       /*
+        * Set the timeout...
+	*/
+
+	if (atoi(value) > 0)
+	  contimeout = atoi(value);
+      }
     }
   }
 
@@ -426,7 +457,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
     status = lpd_queue(hostname, port, resource + 1, filename,
                        username, title, copies,
-		       banner, format, order, reserve, manual_copies, timeout);
+		       banner, format, order, reserve, manual_copies,
+		       timeout, contimeout);
 
     if (!status)
       fprintf(stderr, "PAGE: 1 %d\n", atoi(argv[4]));
@@ -434,7 +466,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   else
     status = lpd_queue(hostname, port, resource + 1, filename,
                        username, title, 1,
-		       banner, format, order, reserve, 1, timeout);
+		       banner, format, order, reserve, 1,
+		       timeout, contimeout);
 
  /*
   * Remove the temporary file if necessary...
@@ -467,6 +500,13 @@ lpd_command(int  fd,		/* I - Socket connection to LPD host */
   char		status;		/* Status from command */
 
 
+ /*
+  * Don't try to send commands if the job has been cancelled...
+  */
+
+  if (abort_job)
+    return (-1);
+
  /*
   * Format the string...
   */
@@ -529,7 +569,8 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
           int        order,		/* I - Order of data/control files */
 	  int        reserve,		/* I - Reserve ports? */
 	  int        manual_copies,	/* I - Do copies by hand... */
-	  int        timeout)		/* I - Timeout... */
+	  int        timeout,		/* I - Timeout... */
+	  int        contimeout)	/* I - Connection timeout */
 {
   FILE			*fp;		/* Job file */
   char			localhost[255];	/* Local host name */
@@ -540,12 +581,17 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
   char			control[10240],	/* LPD control 'file' */
 			*cptr;		/* Pointer into control file string */
   char			status;		/* Status byte from command */
-  struct sockaddr_in	addr;		/* Socket address */
-  struct hostent	*hostaddr;	/* Host address */
+  char			portname[255];	/* Port name */
+  http_addrlist_t	*addrlist,	/* Address list */
+			*addr;		/* Socket address */
   int			copy;		/* Copies written */
-  size_t		nbytes,		/* Number of bytes written */
-			tbytes;		/* Total bytes written */
-  char			buffer[8192];	/* Output buffer */
+  time_t		start_time;	/* Time of first connect */
+#ifdef __APPLE__
+  int			recoverable;	/* Recoverable error shown? */
+#endif /* __APPLE__ */
+  size_t		nbytes;		/* Number of bytes written */
+  off_t			tbytes;		/* Total bytes written */
+  char			buffer[32768];	/* Output buffer */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction	action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
@@ -567,37 +613,63 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
   signal(SIGALRM, lpd_timeout);
 #endif /* HAVE_SIGSET */
 
+ /*
+  * Find the printer...
+  */
+
+  sprintf(portname, "%d", port);
+
+  if ((addrlist = httpAddrGetList(hostname, AF_UNSPEC, portname)) == NULL)
+  {
+    fprintf(stderr, "ERROR: Unable to locate printer \'%s\'!\n",
+            hostname);
+    return (CUPS_BACKEND_STOP);
+  }
+
+ /*
+  * Remember when we starting trying to connect to the printer...
+  */
+
+#ifdef __APPLE__
+  recoverable = 0;
+#endif /* __APPLE__ */
+  start_time  = time(NULL);
+
  /*
   * Loop forever trying to print the file...
   */
 
-  for (;;) /* FOREVER */
+  while (!abort_job)
   {
    /*
     * First try to reserve a port for this connection...
     */
 
-    if ((hostaddr = httpGetHostByName(hostname)) == NULL)
-    {
-      fprintf(stderr, "ERROR: Unable to locate printer \'%s\' - %s\n",
-              hostname, hstrerror(h_errno));
-      return (1);
-    }
-
+    fputs("STATE: +connecting-to-device\n", stderr);
     fprintf(stderr, "INFO: Attempting to connect to host %s for printer %s\n",
             hostname, printer);
 
-    memset(&addr, 0, sizeof(addr));
-    memcpy(&(addr.sin_addr), hostaddr->h_addr, hostaddr->h_length);
-    addr.sin_family = hostaddr->h_addrtype;
-    addr.sin_port   = htons(port);
-
-    for (lport = reserve == RESERVE_RFC1179 ? 732 : 1024;;)
+    for (lport = reserve == RESERVE_RFC1179 ? 732 : 1024, addr = addrlist;;
+         addr = addr->next)
     {
+     /*
+      * Stop if this job has been cancelled...
+      */
+
+      if (abort_job)
+      {
+        httpAddrFreeList(addrlist);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
      /*
       * Choose the next priviledged port...
       */
 
+      if (!addr)
+        addr = addrlist;
+
       lport --;
 
       if (lport < 721 && reserve == RESERVE_RFC1179)
@@ -615,10 +687,12 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	* Just create a regular socket...
 	*/
 
-	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+	if ((fd = socket(addr->addr.addr.sa_family, SOCK_STREAM, 0)) < 0)
 	{
           perror("ERROR: Unable to create socket");
-          return (1);
+	  sleep(1);
+
+          continue;
 	}
 
         lport = 0;
@@ -630,7 +704,7 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	* priviledged lport between 721 and 731...
 	*/
 
-	if ((fd = rresvport(&lport)) < 0)
+	if ((fd = rresvport_af(&lport, addr->addr.addr.sa_family)) < 0)
 	{
 	  perror("ERROR: Unable to reserve port");
 	  sleep(1);
@@ -639,37 +713,112 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	}
       }
 
-      if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
+     /*
+      * Connect to the printer or server...
+      */
+
+      if (abort_job)
       {
-	error = errno;
+        httpAddrFreeList(addrlist);
+
 	close(fd);
-	fd = -1;
 
-	if (error == ECONNREFUSED || error == EHOSTDOWN ||
-            error == EHOSTUNREACH)
-	{
-	  fprintf(stderr, "WARNING: Network host \'%s\' is busy, down, or unreachable; will retry in 30 seconds...\n",
-                  hostname);
-	  sleep(30);
-	}
-	else if (error == EADDRINUSE)
-	{
-	 /*
-	  * Try on another port...
-	  */
+	return (CUPS_BACKEND_FAILED);
+      }
 
-	  sleep(1);
-	}
-	else
+      if (!connect(fd, &(addr->addr.addr), httpAddrLength(&(addr->addr))))
+	break;
+
+      error = errno;
+      close(fd);
+      fd = -1;
+
+      if (addr->next)
+        continue;
+
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        fprintf(stderr, "INFO: Unable to connect to %s, queuing on next printer in class...\n",
+		hostname);
+
+        httpAddrFreeList(addrlist);
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
+      if (error == ECONNREFUSED || error == EHOSTDOWN ||
+          error == EHOSTUNREACH)
+      {
+        if (contimeout && (time(NULL) - start_time) > contimeout)
 	{
-	  perror("ERROR: Unable to connect to printer; will retry in 30 seconds...");
-          sleep(30);
+	  fputs("ERROR: Printer not responding!\n", stderr);
+	  return (CUPS_BACKEND_FAILED);
 	}
+
+#ifdef __APPLE__
+        recoverable = 1;
+	fprintf(stderr, "WARNING: recoverable: "
+	                "Network host \'%s\' is busy, down, or "
+	                "unreachable; will retry in 30 seconds...\n",
+                hostname);
+#else
+	fprintf(stderr, "WARNING: "
+	                "Network host \'%s\' is busy, down, or "
+	                "unreachable; will retry in 30 seconds...\n",
+                hostname);
+#endif /* __APPLE__ */
+	sleep(30);
+      }
+      else if (error == EADDRINUSE)
+      {
+       /*
+	* Try on another port...
+	*/
+
+	sleep(1);
       }
       else
-	break;
+      {
+#ifdef __APPLE__
+        recoverable = 1;
+	perror("ERROR: recoverable: "
+	       "Unable to connect to printer; will retry in 30 seconds...");
+#else
+	perror("ERROR: "
+	       "Unable to connect to printer; will retry in 30 seconds...");
+#endif /* __APPLE__ */
+        sleep(30);
+      }
     }
 
+#ifdef __APPLE__
+    if (recoverable)
+    {
+     /*
+      * If we've shown a recoverable error make sure the printer proxies
+      * have a chance to see the recovered message. Not pretty but
+      * necessary for now...
+      */
+
+      fputs("INFO: recovered: \n", stderr);
+      sleep(5);
+    }
+#endif /* __APPLE__ */
+
+    fputs("STATE: -connecting-to-device\n", stderr);
     fprintf(stderr, "INFO: Connected to %s...\n", hostname);
     fprintf(stderr, "DEBUG: Connected on ports %d (local %d)...\n", port,
             lport);
@@ -680,16 +829,22 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     if (stat(filename, &filestats))
     {
+      httpAddrFreeList(addrlist);
+      close(fd);
+
       perror("ERROR: unable to stat print file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     filestats.st_size *= manual_copies;
 
     if ((fp = fopen(filename, "rb")) == NULL)
     {
+      httpAddrFreeList(addrlist);
+      close(fd);
+
       perror("ERROR: unable to open print file for reading");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
    /*
@@ -699,44 +854,58 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     if (lpd_command(fd, timeout, "\002%s\n",
                     printer))		/* Receive print job(s) */
-      return (1);
+    {
+      httpAddrFreeList(addrlist);
+      close(fd);
+      return (CUPS_BACKEND_FAILED);
+    }
 
-    gethostname(localhost, sizeof(localhost));
-    localhost[31] = '\0'; /* RFC 1179, Section 7.2 - host name < 32 chars */
+    httpGetHostname(NULL, localhost, sizeof(localhost));
 
-    snprintf(control, sizeof(control), "H%s\nP%s\nJ%s\n", localhost, user,
-             title);
+    snprintf(control, sizeof(control),
+             "H%.31s\n"		/* RFC 1179, Section 7.2 - host name <= 31 chars */
+	     "P%.31s\n"		/* RFC 1179, Section 7.2 - user name <= 31 chars */
+	     "J%.99s\n",	/* RFC 1179, Section 7.2 - job name <= 99 chars */
+	     localhost, user, title);
     cptr = control + strlen(control);
 
     if (banner)
     {
-      snprintf(cptr, sizeof(control) - (cptr - control), "C%s\nL%s\n",
+      snprintf(cptr, sizeof(control) - (cptr - control),
+               "C%.31s\n"	/* RFC 1179, Section 7.2 - class name <= 31 chars */
+	       "L%s\n",
                localhost, user);
       cptr   += strlen(cptr);
     }
 
     while (copies > 0)
     {
-      snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%.15s\n", format,
-               getpid() % 1000, localhost);
+      snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%.15s\n",
+               format, (int)getpid() % 1000, localhost);
       cptr   += strlen(cptr);
       copies --;
     }
 
     snprintf(cptr, sizeof(control) - (cptr - control),
-             "UdfA%03d%.15s\nN%s\n",
-             getpid() % 1000, localhost, title);
+             "UdfA%03d%.15s\n"
+	     "N%.131s\n",	/* RFC 1179, Section 7.2 - sourcefile name <= 131 chars */
+             (int)getpid() % 1000, localhost, title);
 
     fprintf(stderr, "DEBUG: Control file is:\n%s", control);
 
     if (order == ORDER_CONTROL_DATA)
     {
       if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
-                      getpid() % 1000, localhost))
-        return (1);
+                      (int)getpid() % 1000, localhost))
+      {
+        httpAddrFreeList(addrlist);
+	close(fd);
 
-      fprintf(stderr, "INFO: Sending control file (%lu bytes)\n",
-              (unsigned long)strlen(control));
+        return (CUPS_BACKEND_FAILED);
+      }
+
+      fprintf(stderr, "INFO: Sending control file (%u bytes)\n",
+              (unsigned)strlen(control));
 
       if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
       {
@@ -772,13 +941,18 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       * Send the print file...
       */
 
-      if (lpd_command(fd, timeout, "\003%u dfA%03.3d%.15s\n",
-                      (unsigned)filestats.st_size, getpid() % 1000,
+      if (lpd_command(fd, timeout, "\003" CUPS_LLFMT " dfA%03.3d%.15s\n",
+                      CUPS_LLCAST filestats.st_size, (int)getpid() % 1000,
 		      localhost))
-        return (1);
+      {
+        httpAddrFreeList(addrlist);
+	close(fd);
 
-      fprintf(stderr, "INFO: Sending data file (%u bytes)\n",
-              (unsigned)filestats.st_size);
+        return (CUPS_BACKEND_FAILED);
+      }
+
+      fprintf(stderr, "INFO: Sending data file (" CUPS_LLFMT " bytes)\n",
+              CUPS_LLCAST filestats.st_size);
 
       tbytes = 0;
       for (copy = 0; copy < manual_copies; copy ++)
@@ -787,8 +961,8 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
 	while ((nbytes = fread(buffer, 1, sizeof(buffer), fp)) > 0)
 	{
-	  fprintf(stderr, "INFO: Spooling LPR job, %u%% complete...\n",
-        	  (unsigned)(100.0f * tbytes / filestats.st_size));
+	  fprintf(stderr, "INFO: Spooling LPR job, %.0f%% complete...\n",
+        	  100.0 * tbytes / filestats.st_size);
 
 	  if (lpd_write(fd, buffer, nbytes) < nbytes)
 	  {
@@ -838,8 +1012,13 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
     if (status == 0 && order == ORDER_DATA_CONTROL)
     {
       if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
-                      getpid() % 1000, localhost))
-        return (1);
+                      (int)getpid() % 1000, localhost))
+      {
+        httpAddrFreeList(addrlist);
+	close(fd);
+
+        return (CUPS_BACKEND_FAILED);
+      }
 
       fprintf(stderr, "INFO: Sending control file (%lu bytes)\n",
               (unsigned long)strlen(control));
@@ -878,7 +1057,11 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
     fclose(fp);
 
     if (status == 0)
-      return (0);
+    {
+      httpAddrFreeList(addrlist);
+
+      return (CUPS_BACKEND_OK);
+    }
 
    /*
     * Waiting for a retry...
@@ -886,6 +1069,14 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     sleep(30);
   }
+
+  httpAddrFreeList(addrlist);
+
+ /*
+  * If we get here, then the job has been cancelled...
+  */
+
+  return (CUPS_BACKEND_FAILED);
 }
 
 
@@ -917,6 +1108,9 @@ lpd_write(int  lpd_fd,			/* I - LPD socket */
 	total;				/* Total number of bytes written */
 
 
+  if (abort_job)
+    return (-1);
+
   total = 0;
   while ((bytes = send(lpd_fd, buffer, length - total, 0)) >= 0)
   {
@@ -934,23 +1128,24 @@ lpd_write(int  lpd_fd,			/* I - LPD socket */
 }
 
 
-#ifndef HAVE_RRESVPORT
+#ifndef HAVE_RRESVPORT_AF
 /*
- * 'rresvport()' - A simple implementation of rresvport().
+ * 'rresvport_af()' - A simple implementation of rresvport_af().
  */
 
-int					/* O  - Socket or -1 on error */
-rresvport(int *port)			/* IO - Port number to bind to */
+static int				/* O  - Socket or -1 on error */
+rresvport_af(int *port,			/* IO - Port number to bind to */
+             int family)		/* I  - Address family */
 {
-  struct sockaddr_in	addr;		/* Socket address */
-  int			fd;		/* Socket file descriptor */
+  http_addr_t	addr;			/* Socket address */
+  int		fd;			/* Socket file descriptor */
 
 
  /*
   * Try to create an IPv4 socket...
   */
 
-  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+  if ((fd = socket(family, SOCK_STREAM, 0)) < 0)
     return (-1);
 
  /*
@@ -958,24 +1153,24 @@ rresvport(int *port)			/* IO - Port number to bind to */
   */
 
   memset(&addr, 0, sizeof(addr));
-
-  addr.sin_family      = AF_INET;
-  addr.sin_addr.s_addr = INADDR_ANY;
+  addr.addr.sa_family = family;
 
  /*
-  * Try to bind the socket to a reserved port; unlike the standard
-  * BSD rresvport(), we limit the port number to 721 through 732
-  * (instead of 512 to 1023) since RFC 1179 defines the local port
-  * number between 721 and 732...
+  * Try to bind the socket to a reserved port...
   */
 
-  while (*port > 720)
+  while (*port > 511)
   {
    /*
     * Set the port number...
     */
 
-    addr.sin_port = htons(*port);
+#  ifdef AF_INET6
+    if (family == AF_INET6)
+      addr.ipv6.sin6_port = htons(*port);
+    else
+#  endif /* AF_INET6 */
+    addr.ipv4.sin_port = htons(*port);
 
    /*
     * Try binding the port to the socket; return if all is OK...
@@ -1019,7 +1214,7 @@ rresvport(int *port)			/* IO - Port number to bind to */
 
   return (-1);
 }
-#endif /* !HAVE_RRESVPORT */
+#endif /* !HAVE_RRESVPORT_AF */
 
 
 /*
@@ -1031,17 +1226,10 @@ sigterm_handler(int sig)		/* I - Signal */
 {
   (void)sig;	/* remove compiler warnings... */
 
- /*
-  * Remove the temporary file if necessary...
-  */
-
-  if (tmpfilename[0])
-    unlink(tmpfilename);
-
-  exit(1);
+  abort_job = 1;
 }
 
 
 /*
- * End of "$Id: lpd.c,v 1.65 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: lpd.c 5886 2006-08-24 19:53:17Z mike $".
  */
