@@ -1,9 +1,9 @@
 /*
- * "$Id: parallel.c,v 1.50 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: parallel.c 5726 2006-07-12 20:00:11Z mike $"
  *
  *   Parallel port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -33,12 +33,13 @@
  * Include necessary headers.
  */
 
-#include <cups/cups.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <cups/string.h>
-#include <signal.h>
+#include "backend-private.h"
+
+#ifdef __hpux
+#  include <sys/time.h>
+#else
+#  include <sys/select.h>
+#endif /* __hpux */
 
 #ifdef WIN32
 #  include <io.h>
@@ -46,6 +47,7 @@
 #  include <unistd.h>
 #  include <fcntl.h>
 #  include <termios.h>
+#  include <sys/socket.h>
 #endif /* WIN32 */
 
 #ifdef __sgi
@@ -75,27 +77,23 @@ void	list_devices(void);
  *    printer-uri job-id user title copies options [file]
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
-     char *argv[])	/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
+     char *argv[])			/* I - Command-line arguments */
 {
-  char		method[255],	/* Method in URI */
-		hostname[1024],	/* Hostname */
-		username[255],	/* Username info (not used) */
-		resource[1024],	/* Resource info (device and options) */
-		*options;	/* Pointer to options */
-  int		port;		/* Port number (not used) */
-  int		fp;		/* Print file */
-  int		copies;		/* Number of copies to print */
-  int		fd;		/* Parallel device */
-  int		wbytes;		/* Number of bytes written */
-  size_t	nbytes,		/* Number of bytes read */
-		tbytes;		/* Total number of bytes written */
-  char		buffer[8192],	/* Output buffer */
-		*bufptr;	/* Pointer into buffer */
-  struct termios opts;		/* Parallel port options */
+  char		method[255],		/* Method in URI */
+		hostname[1024],		/* Hostname */
+		username[255],		/* Username info (not used) */
+		resource[1024],		/* Resource info (device and options) */
+		*options;		/* Pointer to options */
+  int		port;			/* Port number (not used) */
+  int		print_fd,		/* Print file */
+		device_fd;		/* Parallel device */
+  int		copies;			/* Number of copies to print */
+  size_t	tbytes;			/* Total number of bytes written */
+  struct termios opts;			/* Parallel port options */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;	/* Actions for POSIX signals */
+  struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
@@ -126,12 +124,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fputs("Usage: parallel job-id user title copies options [file]\n", stderr);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
@@ -141,8 +139,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 6)
   {
-    fp     = 0;
-    copies = 1;
+    print_fd = 0;
+    copies   = 1;
   }
   else
   {
@@ -150,10 +148,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     * Try to open the print file...
     */
 
-    if ((fp = open(argv[6], O_RDONLY)) < 0)
+    if ((print_fd = open(argv[6], O_RDONLY)) < 0)
     {
       perror("ERROR: unable to open print file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     copies = atoi(argv[4]);
@@ -163,7 +161,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Extract the device name and options from the URI...
   */
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
+  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
+                  method, sizeof(method), username, sizeof(username),
+		  hostname, sizeof(hostname), &port,
+		  resource, sizeof(resource));
 
  /*
   * See if there are any options...
@@ -183,124 +184,103 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Open the parallel port device...
   */
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+
   do
   {
-    if ((fd = open(resource, O_WRONLY | O_EXCL)) == -1)
+    if ((device_fd = open(resource, O_WRONLY | O_EXCL)) == -1)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        fputs("INFO: Unable to open parallel port, queuing on next printer "
+	      "in class...\n", stderr);
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
       if (errno == EBUSY)
       {
-        fputs("INFO: Parallel port busy; will retry in 30 seconds...\n", stderr);
+        fputs("INFO: Parallel port busy; will retry in 30 seconds...\n",
+	      stderr);
 	sleep(30);
       }
       else if (errno == ENXIO || errno == EIO || errno == ENOENT)
       {
-        fputs("INFO: Printer not connected; will retry in 30 seconds...\n", stderr);
+        fputs("INFO: Printer not connected; will retry in 30 seconds...\n",
+	      stderr);
 	sleep(30);
       }
       else
       {
-	fprintf(stderr, "ERROR: Unable to open parallel port device file \"%s\": %s\n",
+	fprintf(stderr,
+	        "ERROR: Unable to open parallel port device file \"%s\": %s\n",
 	        resource, strerror(errno));
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
     }
   }
-  while (fd < 0);
+  while (device_fd < 0);
+
+  fputs("STATE: -connecting-to-device\n", stderr);
 
  /*
   * Set any options provided...
   */
 
-  tcgetattr(fd, &opts);
+  tcgetattr(device_fd, &opts);
 
   opts.c_lflag &= ~(ICANON | ECHO | ISIG);	/* Raw mode */
 
   /**** No options supported yet ****/
 
-  tcsetattr(fd, TCSANOW, &opts);
-
- /*
-  * Now that we are "connected" to the port, ignore SIGTERM so that we
-  * can finish out any page data the driver sends (e.g. to eject the
-  * current page...  Only ignore SIGTERM if we are printing data from
-  * stdin (otherwise you can't cancel raw jobs...)
-  */
-
-  if (argc < 7)
-  {
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-    sigset(SIGTERM, SIG_IGN);
-#elif defined(HAVE_SIGACTION)
-    memset(&action, 0, sizeof(action));
-
-    sigemptyset(&action.sa_mask);
-    action.sa_handler = SIG_IGN;
-    sigaction(SIGTERM, &action, NULL);
-#else
-    signal(SIGTERM, SIG_IGN);
-#endif /* HAVE_SIGSET */
-  }
+  tcsetattr(device_fd, TCSANOW, &opts);
 
  /*
   * Finally, send the print file...
   */
 
-  wbytes = 0;
+  tbytes = 0;
 
-  while (copies > 0)
+  while (copies > 0 && tbytes >= 0)
   {
     copies --;
 
-    if (fp != 0)
+    if (print_fd != 0)
     {
       fputs("PAGE: 1 1\n", stderr);
-      lseek(fp, 0, SEEK_SET);
+      lseek(print_fd, 0, SEEK_SET);
     }
 
-    tbytes = 0;
-    while ((nbytes = read(fp, buffer, sizeof(buffer))) > 0)
-    {
-     /*
-      * Write the print data to the printer...
-      */
+    tbytes = backendRunLoop(print_fd, device_fd, 1);
 
-      tbytes += nbytes;
-      bufptr = buffer;
-
-      while (nbytes > 0)
-      {
-	if ((wbytes = write(fd, bufptr, nbytes)) < 0)
-	  if (errno == ENOTTY)
-	    wbytes = write(fd, bufptr, nbytes);
-
-	if (wbytes < 0)
-	{
-	  perror("ERROR: Unable to send print file to printer");
-	  break;
-	}
-
-	nbytes -= wbytes;
-	bufptr += wbytes;
-      }
-
-      if (wbytes < 0)
-        break;
-
-      if (argc > 6)
-	fprintf(stderr, "INFO: Sending print file, %lu bytes...\n",
-	        (unsigned long)tbytes);
-    }
+    if (print_fd != 0 && tbytes >= 0)
+      fprintf(stderr, "INFO: Sent print file, " CUPS_LLFMT " bytes...\n",
+	      CUPS_LLCAST tbytes);
   }
 
  /*
   * Close the socket connection and input file and return...
   */
 
-  close(fd);
-  if (fp != 0)
-    close(fp);
+  close(device_fd);
 
-  return (wbytes < 0);
+  if (print_fd != 0)
+    close(print_fd);
+
+  return (tbytes < 0 ? CUPS_BACKEND_FAILED : CUPS_BACKEND_OK);
 }
 
 
@@ -320,120 +300,43 @@ list_devices(void)
   int	i;			/* Looping var */
   int	fd;			/* File descriptor */
   char	device[255],		/* Device filename */
-	probefile[255],		/* Probe filename */
-	basedevice[255];	/* Base device filename for ports */
-  FILE	*probe;			/* /proc/parport/n/autoprobe file */
-  char	line[1024],		/* Line from file */
-	*delim,			/* Delimiter in file */
-	make[IPP_MAX_NAME],	/* Make from file */
-	model[IPP_MAX_NAME];	/* Model from file */
+	basedevice[255],	/* Base device filename for ports */
+	device_id[1024],	/* Device ID string */
+	make_model[1024];	/* Make and model */
 
 
+  if (!access("/dev/parallel/", 0))
+    strcpy(basedevice, "/dev/parallel/");
+  else if (!access("/dev/printers/", 0))
+    strcpy(basedevice, "/dev/printers/");
+  else
+    strcpy(basedevice, "/dev/lp");
+
   for (i = 0; i < 4; i ++)
   {
    /*
-    * First open the device to make sure the driver module is loaded...
+    * Open the port, if available...
     */
 
-    if ((fd = open("/dev/parallel/0", O_WRONLY)) >= 0)
+    sprintf(device, "%s%d", basedevice, i);
+    if ((fd = open(device, O_RDWR | O_EXCL)) < 0)
+      fd = open(device, O_WRONLY);
+
+    if (fd >= 0)
     {
+     /*
+      * Now grab the IEEE 1284 device ID string...
+      */
+
+      if (!backendGetDeviceID(fd, device_id, sizeof(device_id),
+                              make_model, sizeof(make_model),
+			      NULL, NULL, 0))
+	printf("direct parallel:%s \"%s\" \"%s LPT #%d\" \"%s\"\n", device,
+	       make_model, make_model, i + 1, device_id);
+      else
+	printf("direct parallel:%s \"Unknown\" \"LPT #%d\"\n", device, i + 1);
+
       close(fd);
-      strcpy(basedevice, "/dev/parallel/");
-    }
-    else
-    {
-      sprintf(device, "/dev/lp%d", i);
-      if ((fd = open(device, O_WRONLY)) >= 0)
-      {
-	close(fd);
-	strcpy(basedevice, "/dev/lp");
-      }
-      else
-      {
-	sprintf(device, "/dev/par%d", i);
-	if ((fd = open(device, O_WRONLY)) >= 0)
-	{
-	  close(fd);
-	  strcpy(basedevice, "/dev/par");
-	}
-	else
-	{
-	  sprintf(device, "/dev/printers/%d", i);
-	  if ((fd = open(device, O_WRONLY)) >= 0)
-	  {
-	    close(fd);
-	    strcpy(basedevice, "/dev/printers/");
-	  }
-	  else
-	    strcpy(basedevice, "/dev/unknown-parallel");
-	}
-      }
-    }
-
-   /*
-    * Then try looking at the probe file...
-    */
-
-    sprintf(probefile, "/proc/parport/%d/autoprobe", i);
-    if ((probe = fopen(probefile, "r")) == NULL)
-    {
-     /*
-      * Linux 2.4 kernel has different path...
-      */
-
-      sprintf(probefile, "/proc/sys/dev/parport/parport%d/autoprobe", i);
-      probe = fopen(probefile, "r");
-    }
-
-    if (probe != NULL)
-    {
-     /*
-      * Found a probe file!
-      */
-
-      memset(make, 0, sizeof(make));
-      memset(model, 0, sizeof(model));
-      strcpy(model, "Unknown");
-
-      while (fgets(line, sizeof(line), probe) != NULL)
-      {
-       /*
-        * Strip trailing ; and/or newline.
-	*/
-
-        if ((delim = strrchr(line, ';')) != NULL)
-	  *delim = '\0';
-	else if ((delim = strrchr(line, '\n')) != NULL)
-	  *delim = '\0';
-
-       /*
-        * Look for MODEL and MANUFACTURER lines...
-	*/
-
-        if (strncmp(line, "MODEL:", 6) == 0 &&
-	    strncmp(line, "MODEL:Unknown", 13) != 0)
-	  strlcpy(model, line + 6, sizeof(model));
-	else if (strncmp(line, "MANUFACTURER:", 13) == 0 &&
-	         strncmp(line, "MANUFACTURER:Unknown", 20) != 0)
-	  strlcpy(make, line + 13, sizeof(make));
-      }
-
-      fclose(probe);
-
-      if (make[0])
-	printf("direct parallel:%s%d \"%s %s\" \"Parallel Port #%d\"\n",
-	       basedevice, i, make, model, i + 1);
-      else
-	printf("direct parallel:%s%d \"%s\" \"Parallel Port #%d\"\n",
-	       basedevice, i, model, i + 1);
-    }
-    else if (fd >= 0)
-    {
-     /*
-      * No probe file, but we know the port is there...
-      */
-
-      printf("direct parallel:%s \"Unknown\" \"Parallel Port #%d\"\n", device, i + 1);
     }
   }
 #elif defined(__sgi)
@@ -633,7 +536,7 @@ list_devices(void)
       printf("direct parallel:%s \"Unknown\" \"Parallel Port #%d\"\n", device, i + 1);
     }
   }
-#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
   int	i;			/* Looping var */
   int	fd;			/* File descriptor */
   char	device[255];		/* Device filename */
@@ -675,5 +578,5 @@ list_devices(void)
 
 
 /*
- * End of "$Id: parallel.c,v 1.50 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: parallel.c 5726 2006-07-12 20:00:11Z mike $".
  */
