@@ -1,9 +1,9 @@
 /*
- * "$Id: scsi.c,v 1.8 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: scsi.c 5023 2006-01-29 14:39:44Z mike $"
  *
  *   SCSI printer backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2003-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 2003-2006 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
  *   without modification, are permitted provided that the
@@ -53,6 +53,7 @@
  * Include necessary headers.
  */
 
+#include <cups/backend.h>
 #include <cups/cups.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -88,7 +89,7 @@ int	print_device(const char *resource, int fd, int copies);
  * Dummy functions that do nothing on unsupported platforms...
  */
 void	list_devices(void) {}
-int	print_device(const char *resource, int fd, int copies) { return (1); }
+int	print_device(const char *resource, int fd, int copies) { return (CUPS_BACKEND_FAILED); }
 #endif /* __linux */
 
 
@@ -145,12 +146,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fputs("Usage: scsi:/dev/file job-id user title copies options [file]\n", stderr);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
@@ -172,7 +173,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     if ((fp = open(argv[6], O_RDONLY)) < 0)
     {
       perror("ERROR: unable to open print file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     copies = atoi(argv[4]);
@@ -182,7 +183,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Extract the device name and options from the URI...
   */
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
+  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
+                  method, sizeof(method), username, sizeof(username),
+		  hostname, sizeof(hostname), &port,
+		  resource, sizeof(resource));
 
  /*
   * See if there are any options...
@@ -216,5 +220,5 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: scsi.c,v 1.8 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: scsi.c 5023 2006-01-29 14:39:44Z mike $".
  */
