@@ -1,9 +1,9 @@
 /*
- * "$Id: serial.c,v 1.56 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: serial.c 5597 2006-05-30 14:29:12Z mike $"
  *
  *   Serial port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -33,12 +33,7 @@
  * Include necessary headers.
  */
 
-#include <cups/cups.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <cups/string.h>
-#include <signal.h>
+#include "backend-private.h"
 
 #ifdef __hpux
 #  include <sys/modem.h>
@@ -50,6 +45,11 @@
 #  include <unistd.h>
 #  include <fcntl.h>
 #  include <termios.h>
+#  ifdef __hpux
+#    include <sys/time.h>
+#  else
+#    include <sys/select.h>
+#  endif /* __hpux */
 #  ifdef HAVE_SYS_IOCTL_H
 #    include <sys/ioctl.h>
 #  endif /* HAVE_SYS_IOCTL_H */
@@ -81,6 +81,11 @@
 #  include <IOKit/IOBSD.h>
 #endif /* __APPLE__ */
 
+#if defined(__linux) && defined(TIOCGSERIAL)
+#  include <linux/serial.h>
+#  include <linux/ioctl.h>
+#endif /* __linux && TIOCGSERIAL */
+
 
 /*
  * Local functions...
@@ -97,33 +102,38 @@ void	list_devices(void);
  *    printer-uri job-id user title copies options [file]
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
-     char *argv[])	/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
+     char *argv[])			/* I - Command-line arguments */
 {
-  char		method[255],	/* Method in URI */
-		hostname[1024],	/* Hostname */
-		username[255],	/* Username info (not used) */
-		resource[1024],	/* Resource info (device and options) */
-		*options,	/* Pointer to options */
-		name[255],	/* Name of option */
-		value[255],	/* Value of option */
-		*ptr;		/* Pointer into name or value */
-  int		port;		/* Port number (not used) */
-  int		fp;		/* Print file */
-  int		copies;		/* Number of copies to print */
-  int		fd;		/* Parallel device */
-  int		wbytes;		/* Number of bytes written */
-  size_t	nbytes,		/* Number of bytes read */
-		tbytes;		/* Total number of bytes written */
-  int		dtrdsr;		/* Do dtr/dsr flow control? */
-  int		bufsize;	/* Size of output buffer for writes */
-  char		buffer[8192],	/* Output buffer */
-		*bufptr;	/* Pointer into buffer */
-  struct termios opts;		/* Serial port options */
-  struct termios origopts;	/* Original port options */
+  char		method[255],		/* Method in URI */
+		hostname[1024],		/* Hostname */
+		username[255],		/* Username info (not used) */
+		resource[1024],		/* Resource info (device and options) */
+		*options,		/* Pointer to options */
+		name[255],		/* Name of option */
+		value[255],		/* Value of option */
+		*ptr;			/* Pointer into name or value */
+  int		port;			/* Port number (not used) */
+  int		copies;			/* Number of copies to print */
+  int		print_fd,		/* Print file */
+		device_fd;		/* Serial device */
+  int		nfds;			/* Maximum file descriptor value + 1 */
+  fd_set	input,			/* Input set for reading */
+		output;			/* Output set for writing */
+  ssize_t	print_bytes,		/* Print bytes read */
+		bc_bytes,		/* Backchannel bytes read */
+		total_bytes,		/* Total bytes written */
+		bytes;			/* Bytes written */
+  int		dtrdsr;			/* Do dtr/dsr flow control? */
+  int		print_size;		/* Size of output buffer for writes */
+  char		print_buffer[8192],	/* Print data buffer */
+		*print_ptr,		/* Pointer into print data buffer */
+		bc_buffer[1024];	/* Back-channel data buffer */
+  struct termios opts;			/* Serial port options */
+  struct termios origopts;		/* Original port options */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;	/* Actions for POSIX signals */
+  struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
@@ -154,12 +164,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fputs("Usage: serial job-id user title copies options [file]\n", stderr);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
@@ -169,8 +179,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 6)
   {
-    fp     = 0;
-    copies = 1;
+    print_fd = 0;
+    copies   = 1;
   }
   else
   {
@@ -178,10 +188,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     * Try to open the print file...
     */
 
-    if ((fp = open(argv[6], O_RDONLY)) < 0)
+    if ((print_fd = open(argv[6], O_RDONLY)) < 0)
     {
       perror("ERROR: unable to open print file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     copies = atoi(argv[4]);
@@ -191,7 +201,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Extract the device name and options from the URI...
   */
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
+  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
+                  method, sizeof(method), username, sizeof(username),
+		  hostname, sizeof(hostname), &port,
+		  resource, sizeof(resource));
 
  /*
   * See if there are any options...
@@ -211,10 +224,34 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Open the serial port device...
   */
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+
   do
   {
-    if ((fd = open(resource, O_WRONLY | O_NOCTTY | O_EXCL | O_NDELAY)) == -1)
+    if ((device_fd = open(resource, O_RDWR | O_NOCTTY | O_EXCL |
+                                    O_NDELAY)) == -1)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        fputs("INFO: Unable to open serial port, queuing on next printer in class...\n",
+	      stderr);
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
       if (errno == EBUSY)
       {
         fputs("INFO: Serial port busy; will retry in 30 seconds...\n", stderr);
@@ -224,26 +261,30 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       {
 	fprintf(stderr, "ERROR: Unable to open serial port device file \"%s\": %s\n",
 	        resource, strerror(errno));
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
     }
   }
-  while (fd < 0);
+  while (device_fd < 0);
+
+  fputs("STATE: -connecting-to-device\n", stderr);
 
  /*
   * Set any options provided...
   */
 
-  tcgetattr(fd, &origopts);
-  tcgetattr(fd, &opts);
+  tcgetattr(device_fd, &origopts);
+  tcgetattr(device_fd, &opts);
 
-  opts.c_lflag &= ~(ICANON | ECHO | ISIG);	/* Raw mode */
-  opts.c_oflag &= ~OPOST;			/* Don't post-process */
+  opts.c_lflag &= ~(ICANON | ECHO | ISIG);
+					/* Raw mode */
+  opts.c_oflag &= ~OPOST;		/* Don't post-process */
 
-  bufsize = 96;		/* 9600 baud / 10 bits/char / 10Hz */
-  dtrdsr  = 0;		/* No dtr/dsr flow control */
+  print_size = 96;			/* 9600 baud / 10 bits/char / 10Hz */
+  dtrdsr     = 0;			/* No dtr/dsr flow control */
 
-  if (options != NULL)
+  if (options)
+  {
     while (*options)
     {
      /*
@@ -263,12 +304,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
         options ++;
 
-	for (ptr = value; *options && *options != '+';)
+	for (ptr = value; *options && *options != '+' && *options != '&';)
           if (ptr < (value + sizeof(value) - 1))
             *ptr++ = *options++;
 	*ptr = '\0';
 
-	if (*options == '+')
+	if (*options == '+' || *options == '&')
 	  options ++;
       }
       else
@@ -278,13 +319,13 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       * Process the option...
       */
 
-      if (strcasecmp(name, "baud") == 0)
+      if (!strcasecmp(name, "baud"))
       {
        /*
         * Set the baud rate...
 	*/
 
-        bufsize = atoi(value) / 100;
+        print_size = atoi(value) / 100;
 
 #if B19200 == 19200
         cfsetispeed(&opts, atoi(value));
@@ -340,7 +381,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	}
 #endif /* B19200 == 19200 */
       }
-      else if (strcasecmp(name, "bits") == 0)
+      else if (!strcasecmp(name, "bits"))
       {
        /*
         * Set number of data bits...
@@ -361,48 +402,70 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	      break;
 	}
       }
-      else if (strcasecmp(name, "parity") == 0)
+      else if (!strcasecmp(name, "parity"))
       {
        /*
 	* Set parity checking...
 	*/
 
-	if (strcasecmp(value, "even") == 0)
+	if (!strcasecmp(value, "even"))
 	{
 	  opts.c_cflag |= PARENB;
           opts.c_cflag &= ~PARODD;
 	}
-	else if (strcasecmp(value, "odd") == 0)
+	else if (!strcasecmp(value, "odd"))
 	{
 	  opts.c_cflag |= PARENB;
           opts.c_cflag |= PARODD;
 	}
-	else if (strcasecmp(value, "none") == 0)
+	else if (!strcasecmp(value, "none"))
 	  opts.c_cflag &= ~PARENB;
+	else if (!strcasecmp(value, "space"))
+	{
+	 /*
+	  * Note: we only support space parity with 7 bits per character...
+	  */
+
+	  opts.c_cflag &= ~CSIZE;
+          opts.c_cflag |= CS8;
+	  opts.c_cflag &= ~PARENB;
+        }
+	else if (!strcasecmp(value, "mark"))
+	{
+	 /*
+	  * Note: we only support mark parity with 7 bits per character
+	  * and 1 stop bit...
+	  */
+
+	  opts.c_cflag &= ~CSIZE;
+          opts.c_cflag |= CS7;
+	  opts.c_cflag &= ~PARENB;
+          opts.c_cflag |= CSTOPB;
+        }
       }
-      else if (strcasecmp(name, "flow") == 0)
+      else if (!strcasecmp(name, "flow"))
       {
        /*
 	* Set flow control...
 	*/
 
-	if (strcasecmp(value, "none") == 0)
+	if (!strcasecmp(value, "none"))
 	{
 	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag &= ~CRTSCTS;
 	}
-	else if (strcasecmp(value, "soft") == 0)
+	else if (!strcasecmp(value, "soft"))
 	{
 	  opts.c_iflag |= IXON | IXOFF;
           opts.c_cflag &= ~CRTSCTS;
 	}
-	else if (strcasecmp(value, "hard") == 0 ||
-	         strcasecmp(value, "rtscts") == 0)
+	else if (!strcasecmp(value, "hard") ||
+	         !strcasecmp(value, "rtscts"))
         {
 	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag |= CRTSCTS;
 	}
-	else if (strcasecmp(value, "dtrdsr") == 0)
+	else if (!strcasecmp(value, "dtrdsr"))
 	{
 	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag &= ~CRTSCTS;
@@ -410,10 +473,24 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	  dtrdsr = 1;
 	}
       }
-    }
+      else if (!strcasecmp(name, "stop"))
+      {
+        switch (atoi(value))
+	{
+	  case 1 :
+	      opts.c_cflag &= ~CSTOPB;
+	      break;
 
-  tcsetattr(fd, TCSANOW, &opts);
-  fcntl(fd, F_SETFL, 0);
+	  case 2 :
+	      opts.c_cflag |= CSTOPB;
+	      break;
+	}
+      }
+    }
+  }
+
+  tcsetattr(device_fd, TCSANOW, &opts);
+  fcntl(device_fd, F_SETFL, 0);
 
  /*
   * Now that we are "connected" to the port, ignore SIGTERM so that we
@@ -422,7 +499,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * stdin (otherwise you can't cancel raw jobs...)
   */
 
-  if (argc < 7)
+  if (print_fd != 0)
   {
 #ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
     sigset(SIGTERM, SIG_IGN);
@@ -438,86 +515,181 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   }
 
  /*
-  * Finally, send the print file...
+  * Figure out the maximum file descriptor value to use with select()...
   */
 
-  if (bufsize > sizeof(buffer))
-    bufsize = sizeof(buffer);
+  nfds = (print_fd > device_fd ? print_fd : device_fd) + 1;
 
-  wbytes = 0;
+ /*
+  * Finally, send the print file.  Ordinarily we would just use the
+  * backendRunLoop() function, however since we need to use smaller
+  * writes and may need to do DSR/DTR flow control, we duplicate much
+  * of the code here instead...
+  */
+
+  if (print_size > sizeof(print_buffer))
+    print_size = sizeof(print_buffer);
+
+  total_bytes = 0;
 
   while (copies > 0)
   {
     copies --;
 
-    if (fp != 0)
+    if (print_fd != 0)
     {
       fputs("PAGE: 1 1\n", stderr);
-      lseek(fp, 0, SEEK_SET);
+      lseek(print_fd, 0, SEEK_SET);
     }
 
-    if (dtrdsr)
+   /*
+    * Now loop until we are out of data from print_fd...
+    */
+
+    for (print_bytes = 0, print_ptr = print_buffer;;)
     {
      /*
-      * Check the port and sleep until DSR is set...
+      * Use select() to determine whether we have data to copy around...
       */
 
-      int status;
+      FD_ZERO(&input);
+      if (!print_bytes)
+	FD_SET(print_fd, &input);
+      FD_SET(device_fd, &input);
 
+      FD_ZERO(&output);
+      if (print_bytes)
+	FD_SET(device_fd, &output);
 
-      if (!ioctl(fd, TIOCMGET, &status))
-        if (!(status & TIOCM_DSR))
-	{
-	 /*
-	  * Wait for DSR to go high...
-	  */
+      if (select(nfds, &input, &output, NULL, NULL) < 0)
+	continue;			/* Ignore errors here */
 
-	  fputs("DEBUG: DSR is low; waiting for device...\n", stderr);
-
-          do
-	  {
-	    sleep(1);
-	    if (ioctl(fd, TIOCMGET, &status))
-	      break;
-	  }
-	  while (!(status & TIOCM_DSR));
-
-	  fputs("DEBUG: DSR is high; writing to device...\n", stderr);
-        }
-    }
-
-    tbytes = 0;
-    while ((nbytes = read(fp, buffer, bufsize)) > 0)
-    {
      /*
-      * Write the print data to the printer...
+      * Check if we have back-channel data ready...
       */
 
-      tbytes += nbytes;
-      bufptr = buffer;
-
-      while (nbytes > 0)
+      if (FD_ISSET(device_fd, &input))
       {
-	if ((wbytes = write(fd, bufptr, nbytes)) < 0)
-	  if (errno == ENOTTY)
-	    wbytes = write(fd, bufptr, nbytes);
-
-	if (wbytes < 0)
+	if ((bc_bytes = read(device_fd, bc_buffer, sizeof(bc_buffer))) > 0)
 	{
-	  perror("ERROR: Unable to send print file to printer");
-	  break;
+	  fprintf(stderr,
+	          "DEBUG: Received " CUPS_LLFMT " bytes of back-channel data!\n",
+	          CUPS_LLCAST bc_bytes);
+          cupsBackChannelWrite(bc_buffer, bc_bytes, 1.0);
 	}
-
-	nbytes -= wbytes;
-	bufptr += wbytes;
       }
 
-      if (wbytes < 0)
-        break;
+     /*
+      * Check if we have print data ready...
+      */
 
-      if (argc > 6)
-	fprintf(stderr, "INFO: Sending print file, %lu bytes...\n",
-	        (unsigned long)tbytes);
+      if (FD_ISSET(print_fd, &input))
+      {
+	if ((print_bytes = read(print_fd, print_buffer, print_size)) < 0)
+	{
+	 /*
+          * Read error - bail if we don't see EAGAIN or EINTR...
+	  */
+
+	  if (errno != EAGAIN || errno != EINTR)
+	  {
+	    perror("ERROR: Unable to read print data");
+
+            tcsetattr(device_fd, TCSADRAIN, &origopts);
+
+	    close(device_fd);
+
+	    if (print_fd != 0)
+	      close(print_fd);
+
+	    return (CUPS_BACKEND_FAILED);
+	  }
+
+          print_bytes = 0;
+	}
+	else if (print_bytes == 0)
+	{
+	 /*
+          * End of file, break out of the loop...
+	  */
+
+          break;
+	}
+
+	print_ptr = print_buffer;
+      }
+
+     /*
+      * Check if the device is ready to receive data and we have data to
+      * send...
+      */
+
+      if (print_bytes && FD_ISSET(device_fd, &output))
+      {
+	if (dtrdsr)
+	{
+	 /*
+	  * Check the port and sleep until DSR is set...
+	  */
+
+	  int status;
+
+
+	  if (!ioctl(device_fd, TIOCMGET, &status))
+            if (!(status & TIOCM_DSR))
+	    {
+	     /*
+	      * Wait for DSR to go high...
+	      */
+
+	      fputs("DEBUG: DSR is low; waiting for device...\n", stderr);
+
+              do
+	      {
+	       /*
+	        * Poll every 100ms...
+		*/
+
+		usleep(100000);
+
+		if (ioctl(device_fd, TIOCMGET, &status))
+		  break;
+	      }
+	      while (!(status & TIOCM_DSR));
+
+	      fputs("DEBUG: DSR is high; writing to device...\n", stderr);
+            }
+	}
+
+	if ((bytes = write(device_fd, print_ptr, print_bytes)) < 0)
+	{
+	 /*
+          * Write error - bail if we don't see an error we can retry...
+	  */
+
+	  if (errno != EAGAIN && errno != EINTR && errno != ENOTTY)
+	  {
+	    perror("ERROR: Unable to write print data");
+
+            tcsetattr(device_fd, TCSADRAIN, &origopts);
+
+	    close(device_fd);
+
+	    if (print_fd != 0)
+	      close(print_fd);
+
+	    return (CUPS_BACKEND_FAILED);
+	  }
+	}
+	else
+	{
+          fprintf(stderr, "DEBUG: Wrote %d bytes...\n", (int)bytes);
+
+          print_bytes -= bytes;
+	  print_ptr   += bytes;
+	  total_bytes += bytes;
+	}
+      }
     }
   }
 
@@ -525,13 +697,14 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Close the serial port and input file and return...
   */
 
-  tcsetattr(fd, TCSADRAIN, &origopts);
+  tcsetattr(device_fd, TCSADRAIN, &origopts);
 
-  close(fd);
-  if (fp != 0)
-    close(fp);
+  close(device_fd);
 
-  return (wbytes < 0);
+  if (print_fd != 0)
+    close(print_fd);
+
+  return (total_bytes < 0 ? CUPS_BACKEND_FAILED : CUPS_BACKEND_OK);
 }
 
 
@@ -542,23 +715,50 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 void
 list_devices(void)
 {
-#if defined(__hpux) || defined(__sgi) || defined(__sun) || defined(__FreeBSD__) || defined(__OpenBSD__)
+#if defined(__hpux) || defined(__sgi) || defined(__sun) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
   static char	*funky_hex = "0123456789abcdefghijklmnopqrstuvwxyz";
-				/* Funky hex numbering used for some devices */
+					/* Funky hex numbering used for some *
+					 * devices                           */
 #endif /* __hpux || __sgi || __sun || __FreeBSD__ || __OpenBSD__ */
 
-#if defined(__linux) || defined(linux) || defined(__linux__)
-  int	i;		/* Looping var */
-  int	fd;		/* File descriptor */
-  char	device[255];	/* Device filename */
+#ifdef __linux
+  int			i, j;		/* Looping vars */
+  int			fd;		/* File descriptor */
+  char			device[255];	/* Device filename */
+#  ifdef TIOCGSERIAL
+  struct serial_struct	serinfo;	/* serial port info */
+#  endif /* TIOCGSERIAL */
 
 
   for (i = 0; i < 100; i ++)
   {
     sprintf(device, "/dev/ttyS%d", i);
+
     if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
     {
+#  ifdef TIOCGSERIAL
+     /*
+      * See if this port exists...
+      */
+
+      serinfo.reserved_char[0] = 0;
+
+      if (!ioctl(fd, TIOCGSERIAL, &serinfo))
+      {
+	if (serinfo.type == PORT_UNKNOWN)
+	{
+	 /*
+	  * Nope...
+	  */
+
+	  close(fd);
+	  continue;
+	}
+      }
+#  endif /* TIOCGSERIAL */
+
       close(fd);
+
 #  if defined(_ARCH_PPC) || defined(powerpc) || defined(__powerpc)
       printf("serial serial:%s?baud=230400 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
@@ -579,6 +779,21 @@ list_devices(void)
              device, i + 1);
     }
   }
+
+  for (i = 0; i < 64; i ++)
+  {
+    for (j = 0; j < 8; j ++)
+    {
+      sprintf(device, "/dev/ttyQ%02de%d", i, j);
+      if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
+      {
+        close(fd);
+        printf("serial serial:%s?baud=115200 \"Unknown\" "
+	       "\"Equinox ESP %d Port #%d\"\n",
+               device, i, j + 1);
+      }
+    }
+  }
 #elif defined(__sgi)
   int		i, j, n;	/* Looping vars */
   char		device[255];	/* Device filename */
@@ -690,13 +905,13 @@ list_devices(void)
   {
     sprintf(device, "/dev/cua/%c", 'a' + i);
     if (access(device, 0) == 0)
-#ifdef B115200
+#  ifdef B115200
       printf("serial serial:%s?baud=115200 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
-#else
+#  else
       printf("serial serial:%s?baud=38400 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
-#endif /* B115200 */
+#  endif /* B115200 */
   }
 
  /*
@@ -792,7 +1007,7 @@ list_devices(void)
       printf("serial serial:%s?baud=38400 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
   }
-#elif defined(__FreeBSD__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
   int	i, j;		/* Looping vars */
   int	fd;		/* File descriptor */
   char	device[255];	/* Device filename */
@@ -827,6 +1042,14 @@ list_devices(void)
 	printf("serial serial:%s?baud=115200 \"Unknown\" \"Cyclades #%d Serial Port #%d\"\n",
                device, i, j + 1);
       }
+
+      sprintf(device, "/dev/ttyC%d%c", i, funky_hex[j]);
+      if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
+      {
+	close(fd);
+	printf("serial serial:%s?baud=115200 \"Unknown\" \"Cyclades #%d Serial Port #%d\"\n",
+               device, i, j + 1);
+      }
     }
 
  /*
@@ -921,7 +1144,6 @@ list_devices(void)
   CFMutableDictionaryRef	classesToMatch;
   io_object_t			serialService;
 
-  printf("serial serial \"Unknown\" \"Serial Printer (serial)\"\n");
 
   kernResult = IOMasterPort(MACH_PORT_NULL, &masterPort);
   if (KERN_SUCCESS != kernResult)
@@ -975,8 +1197,8 @@ list_devices(void)
 	      CFRelease(bsdPathAsCFString);
 
 	      if (result)
-		printf("serial serial:%s?baud=115200 \"Unknown\" \"%s\"\n", bsdPath,
-		       serialName);
+		printf("serial serial:%s?baud=115200 \"Unknown\" \"%s\"\n",
+		       bsdPath, serialName);
 	    }
 	  }
 	}
@@ -984,7 +1206,11 @@ list_devices(void)
 	IOObjectRelease(serialService);
       }
 
-      IOObjectRelease(serialPortIterator);    /* Release the iterator. */
+     /*
+      * Release the iterator.
+      */
+
+      IOObjectRelease(serialPortIterator);
     }
   }
 #endif
@@ -992,5 +1218,5 @@ list_devices(void)
 
 
 /*
- * End of "$Id: serial.c,v 1.56 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: serial.c 5597 2006-05-30 14:29:12Z mike $".
  */
