@@ -1,9 +1,9 @@
 /*
- * "$Id: usb.c,v 1.57 2005/01/03 19:29:44 mike Exp $"
+ * "$Id: usb.c 5591 2006-05-26 19:51:59Z mike $"
  *
  *   USB port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -39,6 +39,7 @@
 #  include <ApplicationServices/ApplicationServices.h>
 #endif /* __APPLE__ */
 
+#include <cups/backend.h>
 #include <cups/cups.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -62,7 +63,7 @@
 void	list_devices(void);
 int	print_device(const char *uri, const char *hostname,
 	             const char *resource, const char *options,
-		     int fp, int copies);
+		     int print_fd, int copies, int argc, char *argv[]);
 
 
 /*
@@ -71,7 +72,7 @@ int	print_device(const char *uri, const char *hostname,
 
 #ifdef __APPLE__
 #  include "usb-darwin.c"
-#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 #  include "usb-unix.c"
 #else
 /*
@@ -107,8 +108,10 @@ print_device(const char *uri,		/* I - Device URI */
              const char *hostname,	/* I - Hostname/manufacturer */
              const char *resource,	/* I - Resource/modelname */
 	     const char *options,	/* I - Device options/serial number */
-	     int        fp,		/* I - File descriptor to print */
-	     int        copies)		/* I - Copies to print */
+	     int        print_fd,	/* I - File descriptor to print */
+	     int        copies,		/* I - Copies to print */
+	     int	argc,		/* I - Number of command-line arguments (6 or 7) */
+	     char	*argv[])	/* I - Command-line arguments */
 {
  /*
   * Can't print, so just reference the arguments to eliminate compiler
@@ -121,10 +124,12 @@ print_device(const char *uri,		/* I - Device URI */
   (void)hostname;
   (void)resource;
   (void)options;
-  (void)fp;
+  (void)print_fd;
   (void)copies;
+  (void)argc;
+  (void)argv;
 
-  return (1);
+  return (CUPS_BACKEND_FAILED);
 }
 #endif /* __APPLE__ */
 
@@ -141,7 +146,7 @@ int					/* O - Exit status */
 main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
      char *argv[])			/* I - Command-line arguments */
 {
-  int		fp;			/* Print file */
+  int		print_fd;		/* Print file */
   int		copies;			/* Number of copies to print */
   int		status;			/* Exit status */
   int		port;			/* Port number (not used) */
@@ -183,31 +188,29 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fputs("Usage: usb job-id user title copies options [file]\n", stderr);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
   * Extract the device name and options from the URI...
   */
 
-  if (strncmp(argv[0], "usb:", 4))
-    uri = getenv("DEVICE_URI");
-  else
-    uri = argv[0];
+  uri = cupsBackendDeviceURI(argv);
 
-  if (!uri)
+  if (httpSeparateURI(HTTP_URI_CODING_ALL, uri,
+                      method, sizeof(method), username, sizeof(username),
+		      hostname, sizeof(hostname), &port,
+		      resource, sizeof(resource)) < HTTP_URI_OK)
   {
     fputs("ERROR: No device URI found in argv[0] or in DEVICE_URI environment variable!\n", stderr);
     return (1);
   }
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
-
  /*
   * See if there are any options...
   */
@@ -229,8 +232,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 6)
   {
-    fp     = 0;
-    copies = 1;
+    print_fd = 0;
+    copies   = 1;
   }
   else
   {
@@ -238,11 +241,11 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     * Try to open the print file...
     */
 
-    if ((fp = open(argv[6], O_RDONLY)) < 0)
+    if ((print_fd = open(argv[6], O_RDONLY)) < 0)
     {
       fprintf(stderr, "ERROR: unable to open print file %s - %s\n",
               argv[6], strerror(errno));
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     copies = atoi(argv[4]);
@@ -252,19 +255,20 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   * Finally, send the print file...
   */
 
-  status = print_device(uri, hostname, resource, options, fp, copies);
+  status = print_device(uri, hostname, resource, options, print_fd, copies,
+                        argc, argv);
 
  /*
   * Close the input file and return...
   */
 
-  if (fp != 0)
-    close(fp);
+  if (print_fd != 0)
+    close(print_fd);
 
   return (status);
 }
 
 
 /*
- * End of "$Id: usb.c,v 1.57 2005/01/03 19:29:44 mike Exp $".
+ * End of "$Id: usb.c 5591 2006-05-26 19:51:59Z mike $".
  */
