@@ -1,9 +1,9 @@
 /*
- * "$Id: lpc.c,v 1.19 2004/02/25 20:14:50 mike Exp $"
+ * "$Id: lpc.c 5926 2006-09-05 20:45:47Z mike $"
  *
  *   "lpc" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -36,9 +36,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <cups/cups.h>
-#include <cups/language.h>
+#include <cups/i18n.h>
 #include <cups/debug.h>
 #include <cups/string.h>
 
@@ -47,10 +46,10 @@
  * Local functions...
  */
 
-static int	compare_strings(char *, char *, int);
-static void	do_command(http_t *, char *, char *);
-static void	show_help(char *);
-static void	show_status(http_t *, char *);
+static int	compare_strings(const char *, const char *, int);
+static void	do_command(http_t *, const char *, const char *);
+static void	show_help(const char *);
+static void	show_status(http_t *, const char *);
 
 
 /*
@@ -58,14 +57,16 @@ static void	show_status(http_t *, char *);
  */
 
 int
-main(int  argc,		/* I - Number of command-line arguments */
-     char *argv[])	/* I - Command-line arguments */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  http_t	*http;		/* Connection to server */
-  char		line[1024],	/* Input line from user */
-		*params;	/* Pointer to parameters */
+  http_t	*http;			/* Connection to server */
+  char		line[1024],		/* Input line from user */
+		*params;		/* Pointer to parameters */
 
 
+  _cupsSetLocale(argv);
+
  /*
   * Connect to the scheduler...
   */
@@ -86,22 +87,42 @@ main(int  argc,		/* I - Number of command-line arguments */
     * Do the command prompt thing...
     */
 
-    printf("lpc> ");
+    _cupsLangPuts(stdout, _("lpc> "));
     while (fgets(line, sizeof(line), stdin) != NULL)
     {
      /*
-      * Strip the trailing newline...
+      * Strip trailing whitespace...
       */
 
-      line[strlen(line) - 1] = '\0';
+      for (params = line + strlen(line) - 1; params >= line;)
+        if (!isspace(*params & 255))
+	  break;
+	else
+	  *params-- = '\0';
+
+     /*
+      * Strip leading whitespace...
+      */
+
+      for (params = line; isspace(*params & 255); params ++);
+
+      if (params > line)
+        _cups_strcpy(line, params);
+
+      if (!line[0])
+      {
+       /*
+        * Nothing left, just show a prompt...
+	*/
+
+	_cupsLangPuts(stdout, _("lpc> "));
+	continue;
+      }
 
      /*
       * Find any options in the string...
       */
 
-      while (isspace(line[0] & 255))
-        cups_strcpy(line, line + 1);
-
       for (params = line; *params != '\0'; params ++)
         if (isspace(*params & 255))
 	  break;
@@ -117,8 +138,8 @@ main(int  argc,		/* I - Number of command-line arguments */
       * The "quit" and "exit" commands exit; otherwise, process as needed...
       */
 
-      if (compare_strings(line, "quit", 1) == 0 ||
-          compare_strings(line, "exit", 2) == 0)
+      if (!compare_strings(line, "quit", 1) ||
+          !compare_strings(line, "exit", 2))
         break;
 
       if (*params == '\0')
@@ -130,7 +151,7 @@ main(int  argc,		/* I - Number of command-line arguments */
       * Put another prompt out to the user...
       */
 
-      printf("lpc> ");
+      _cupsLangPuts(stdout, _("lpc> "));
     }
   }
 
@@ -148,12 +169,12 @@ main(int  argc,		/* I - Number of command-line arguments */
  * 'compare_strings()' - Compare two command-line strings.
  */
 
-static int			/* O - -1 or 1 = no match, 0 = match */
-compare_strings(char *s,	/* I - Command-line string */
-                char *t,	/* I - Option string */
-                int  tmin)	/* I - Minimum number of unique chars in option */
+static int				/* O - -1 or 1 = no match, 0 = match */
+compare_strings(const char *s,		/* I - Command-line string */
+                const char *t,		/* I - Option string */
+                int        tmin)	/* I - Minimum number of unique chars in option */
 {
-  int	slen;			/* Length of command-line string */
+  int	slen;				/* Length of command-line string */
 
 
   slen = strlen(s);
@@ -169,17 +190,18 @@ compare_strings(char *s,	/* I - Command-line string */
  */
 
 static void
-do_command(http_t *http,	/* I - HTTP connection to server */
-           char   *command,	/* I - Command string */
-	   char   *params)	/* I - Parameters for command */
+do_command(http_t     *http,		/* I - HTTP connection to server */
+           const char *command,		/* I - Command string */
+	   const char *params)		/* I - Parameters for command */
 {
-  if (compare_strings(command, "status", 4) == 0)
+  if (!compare_strings(command, "status", 4))
     show_status(http, params);
-  else if (compare_strings(command, "help", 1) == 0 ||
-           strcmp(command, "?") == 0)
+  else if (!compare_strings(command, "help", 1) || !strcmp(command, "?"))
     show_help(params);
   else
-    printf("%s is not implemented by the CUPS version of lpc.\n", command);
+    _cupsLangPrintf(stdout,
+                    _("%s is not implemented by the CUPS version of lpc.\n"),
+		    command);
 }
 
 
@@ -188,21 +210,21 @@ do_command(http_t *http,	/* I - HTTP connection to server */
  */
 
 static void
-show_help(char *command)	/* I - Command to describe or NULL */
+show_help(const char *command)		/* I - Command to describe or NULL */
 {
-  if (command == NULL)
+  if (!command)
   {
-    puts("Commands may be abbreviated.  Commands are:");
-    puts("");
-    puts("exit    help    quit    status  ?");
+    _cupsLangPrintf(stdout,
+                    _("Commands may be abbreviated.  Commands are:\n"
+		      "\n"
+		      "exit    help    quit    status  ?\n"));
   }
-  else if (compare_strings(command, "help", 1) == 0 ||
-           strcmp(command, "?") == 0)
-    puts("help\t\tget help on commands");
-  else if (compare_strings(command, "status", 4) == 0)
-    puts("status\t\tshow status of daemon and queue");
+  else if (!compare_strings(command, "help", 1) || !strcmp(command, "?"))
+    _cupsLangPrintf(stdout, _("help\t\tget help on commands\n"));
+  else if (!compare_strings(command, "status", 4))
+    _cupsLangPrintf(stdout, _("status\t\tshow status of daemon and queue\n"));
   else
-    puts("?Invalid help command unknown");
+    _cupsLangPrintf(stdout, _("?Invalid help command unknown\n"));
 }
 
 
@@ -211,28 +233,28 @@ show_help(char *command)	/* I - Command to describe or NULL */
  */
 
 static void
-show_status(http_t *http,	/* I - HTTP connection to server */
-            char   *dests)	/* I - Destinations */
+show_status(http_t     *http,		/* I - HTTP connection to server */
+            const char *dests)		/* I - Destinations */
 {
-  ipp_t		*request,	/* IPP Request */
-		*response,	/* IPP Response */
-		*jobs;		/* IPP Get Jobs response */
-  ipp_attribute_t *attr,	/* Current attribute */
-		*jattr;		/* Current job attribute */
-  cups_lang_t	*language;	/* Default language */
-  char		*printer,	/* Printer name */
-		*device,	/* Device URI */
-                *delimiter;     /* Char search result */
-  ipp_pstate_t	pstate;		/* Printer state */
-  int		accepting;	/* Is printer accepting jobs? */
-  int		jobcount;	/* Count of current jobs */
-  char		*dptr,		/* Pointer into destination list */
-		*ptr;		/* Pointer into printer name */
-  int		match;		/* Non-zero if this job matches */
+  ipp_t		*request,		/* IPP Request */
+		*response,		/* IPP Response */
+		*jobs;			/* IPP Get Jobs response */
+  ipp_attribute_t *attr,		/* Current attribute */
+		*jattr;			/* Current job attribute */
+  cups_lang_t	*language;		/* Default language */
+  char		*printer,		/* Printer name */
+		*device,		/* Device URI */
+                *delimiter;		/* Char search result */
+  ipp_pstate_t	pstate;			/* Printer state */
+  int		accepting;		/* Is printer accepting jobs? */
+  int		jobcount;		/* Count of current jobs */
+  const char	*dptr,			/* Pointer into destination list */
+		*ptr;			/* Pointer into printer name */
+  int		match;			/* Non-zero if this job matches */
   char		printer_uri[HTTP_MAX_URI];
-				/* Printer URI */
-  static const char *requested[] =
-		{		/* Requested attributes */
+					/* Printer URI */
+  static const char *requested[] =	/* Requested attributes */
+		{
 		  "printer-name",
 		  "device-uri",
 		  "printer-state",
@@ -240,7 +262,7 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 		};
 
 
-  DEBUG_printf(("show_status(%08x, %08x)\n", http, dests));
+  DEBUG_printf(("show_status(http=%p, dests=\"%s\")\n", http, dests));
 
   if (http == NULL)
     return;
@@ -274,7 +296,7 @@ show_status(http_t *http,	/* I - HTTP connection to server */
   * Do the request and get back a response...
   */
 
-  if ((response = cupsDoRequest(http, request, "/printers/")) != NULL)
+  if ((response = cupsDoRequest(http, request, "/")) != NULL)
   {
     DEBUG_puts("show_status: request succeeded...");
 
@@ -307,19 +329,19 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 
       while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
       {
-        if (strcmp(attr->name, "printer-name") == 0 &&
+        if (!strcmp(attr->name, "printer-name") &&
 	    attr->value_tag == IPP_TAG_NAME)
 	  printer = attr->values[0].string.text;
 
-        if (strcmp(attr->name, "device-uri") == 0 &&
+        if (!strcmp(attr->name, "device-uri") &&
 	    attr->value_tag == IPP_TAG_URI)
 	  device = attr->values[0].string.text;
 
-        if (strcmp(attr->name, "printer-state") == 0 &&
+        if (!strcmp(attr->name, "printer-state") &&
 	    attr->value_tag == IPP_TAG_ENUM)
 	  pstate = (ipp_pstate_t)attr->values[0].integer;
 
-        if (strcmp(attr->name, "printer-is-accepting-jobs") == 0 &&
+        if (!strcmp(attr->name, "printer-is-accepting-jobs") &&
 	    attr->value_tag == IPP_TAG_BOOLEAN)
 	  accepting = attr->values[0].boolean;
 
@@ -338,6 +360,13 @@ show_status(http_t *http,	/* I - HTTP connection to server */
           continue;
       }
 
+     /*
+      * A single 'all' printer name is special, meaning all printers.
+      */
+
+      if (dests != NULL && !strcmp(dests, "all"))
+        dests = NULL;
+
      /*
       * See if this is a printer we're interested in...
       */
@@ -393,83 +422,92 @@ show_status(http_t *http,	/* I - HTTP connection to server */
       if (match)
       {
        /*
-        * If the printer state is "IPP_PRINTER_PROCESSING", then grab the
-	* current job for the printer.
+	* Build an IPP_GET_JOBS request, which requires the following
+	* attributes:
+	*
+	*    attributes-charset
+	*    attributes-natural-language
+	*    printer-uri
+	*    limit
 	*/
 
-        if (pstate == IPP_PRINTER_PROCESSING)
+	request = ippNew();
+
+	request->request.op.operation_id = IPP_GET_JOBS;
+	request->request.op.request_id   = 1;
+
+	language = cupsLangDefault();
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
+                     "attributes-charset", NULL,
+		     cupsLangEncoding(language));
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+                     "attributes-natural-language", NULL,
+		     language->language);
+
+        httpAssembleURIf(HTTP_URI_CODING_ALL, printer_uri, sizeof(printer_uri),
+	                 "ipp", NULL, "localhost", 631, "/printers/%s",
+			 printer);
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+	             "printer-uri", NULL, printer_uri);
+
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+	             "requested-attributes", NULL, "job-id");
+
+        if ((jobs = cupsDoRequest(http, request, "/")) != NULL)
 	{
 	 /*
-	  * Build an IPP_GET_JOBS request, which requires the following
-	  * attributes:
-	  *
-	  *    attributes-charset
-	  *    attributes-natural-language
-	  *    printer-uri
-	  *    limit
+	  * Grab the number of jobs for the printer.
 	  */
 
-	  request = ippNew();
+	  for (jattr = jobs->attrs; jattr != NULL; jattr = jattr->next)
+	    if (jattr->name && !strcmp(jattr->name, "job-id"))
+	      jobcount ++;
 
-	  request->request.op.operation_id = IPP_GET_JOBS;
-	  request->request.op.request_id   = 1;
-
-	  language = cupsLangDefault();
-
-	  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-                       "attributes-charset", NULL,
-		       cupsLangEncoding(language));
-
-	  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-                       "attributes-natural-language", NULL,
-		       language->language);
-
-          snprintf(printer_uri, sizeof(printer_uri),
-                   "ipp://localhost/printers/%s", printer);
-	  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
-	               "printer-uri", NULL, printer_uri);
-
-	  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
-	               "requested-attributes", NULL, "job-id");
-
-          if ((jobs = cupsDoRequest(http, request, "/jobs/")) != NULL)
-	  {
-	    for (jattr = jobs->attrs; jattr != NULL; jattr = jattr->next)
-	      if (jattr->name && strcmp(jattr->name, "job-id") == 0)
-	        jobcount ++;
-
-            ippDelete(jobs);
-	  }
-        }
+          ippDelete(jobs);
+	}
 
        /*
         * Display it...
 	*/
 
         printf("%s:\n", printer);
-	if (strncmp(device, "file:", 5) == 0)
-	  printf("\tprinter is on device \'%s\' speed -1\n", device + 5);
+	if (!strncmp(device, "file:", 5))
+	  _cupsLangPrintf(stdout,
+	                  _("\tprinter is on device \'%s\' speed -1\n"),
+			  device + 5);
 	else
 	{
 	 /*
-	  * Just show the method...
+	  * Just show the scheme...
 	  */
 
 	  if ((delimiter = strchr(device, ':')) != NULL )
 	  {
-	      *delimiter = '\0';
-	      printf("\tprinter is on device \'%s\' speed -1\n", device);
+	    *delimiter = '\0';
+	    _cupsLangPrintf(stdout,
+	                    _("\tprinter is on device \'%s\' speed -1\n"),
+			    device);
 	  }
 	}
 
-	printf("\tqueuing is %sabled\n", accepting ? "en" : "dis");
-	printf("\tprinting is %sabled\n",
-	       pstate == IPP_PRINTER_STOPPED ? "dis" : "en");
-	if (jobcount == 0)
-	  puts("\tno entries");
+        if (accepting)
+	  _cupsLangPuts(stdout, _("\tqueuing is enabled\n"));
 	else
-	  printf("\t%d entries\n", jobcount);
-	puts("\tdaemon present");
+	  _cupsLangPuts(stdout, _("\tqueuing is disabled\n"));
+
+        if (pstate != IPP_PRINTER_STOPPED)
+	  _cupsLangPuts(stdout, _("\tprinting is enabled\n"));
+	else
+	  _cupsLangPuts(stdout, _("\tprinting is disabled\n"));
+
+	if (jobcount == 0)
+	  _cupsLangPuts(stdout, _("\tno entries\n"));
+	else
+	  _cupsLangPrintf(stdout, _("\t%d entries\n"), jobcount);
+
+	_cupsLangPuts(stdout, _("\tdaemon present\n"));
       }
 
       if (attr == NULL)
@@ -482,5 +520,5 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpc.c,v 1.19 2004/02/25 20:14:50 mike Exp $".
+ * End of "$Id: lpc.c 5926 2006-09-05 20:45:47Z mike $".
  */
