@@ -1,9 +1,9 @@
 /*
- * "$Id: auth.c,v 1.10 2004/06/29 03:44:52 mike Exp $"
+ * "$Id: auth.c 5961 2006-09-16 19:08:36Z mike $"
  *
  *   Authentication functions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,7 +15,7 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
+ *       Hollywood, Maryland 20636 USA
  *
  *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
@@ -25,7 +25,7 @@
  *
  * Contents:
  *
- *   cupsDoAuthentication() - Authenticate a request...
+ *   cupsDoAuthentication() - Authenticate a request.
  *   cups_local_auth()      - Get the local authorization certificate if
  *                            available/applicable...
  */
@@ -34,10 +34,7 @@
  * Include necessary headers...
  */
 
-#include "cups.h"
-#include "ipp.h"
-#include "language.h"
-#include "string.h"
+#include "globals.h"
 #include "debug.h"
 #include <stdlib.h>
 #include <ctype.h>
@@ -59,7 +56,12 @@ static int	cups_local_auth(http_t *http);
 
 
 /*
- * 'cupsDoAuthentication()' - Authenticate a request...
+ * 'cupsDoAuthentication()' - Authenticate a request.
+ *
+ * This function should be called in response to a HTTP_UNAUTHORIZED
+ * status, prior to resubmitting your request.
+ *
+ * @since CUPS 1.1.20@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -79,6 +81,8 @@ cupsDoAuthentication(http_t     *http,	/* I - HTTP connection to server */
                 http, method, resource));
   DEBUG_printf(("cupsDoAuthentication: digest_tries=%d, userpass=\"%s\"\n",
                 http->digest_tries, http->userpass));
+  DEBUG_printf(("cupsDoAuthentication: WWW-Authenticate=\"%s\"\n",
+                httpGetField(http, HTTP_FIELD_WWW_AUTHENTICATE)));
 
  /*
   * Clear the current authentication string...
@@ -90,9 +94,13 @@ cupsDoAuthentication(http_t     *http,	/* I - HTTP connection to server */
   * See if we can do local authentication...
   */
 
-  if (!cups_local_auth(http))
+  if (http->digest_tries < 3 && !cups_local_auth(http))
   {
     DEBUG_printf(("cupsDoAuthentication: authstring=\"%s\"\n", http->authstring));
+
+    if (http->status == HTTP_UNAUTHORIZED)
+      http->digest_tries ++;
+
     return (0);
   }
 
@@ -106,11 +114,11 @@ cupsDoAuthentication(http_t     *http,	/* I - HTTP connection to server */
     * Nope - get a new password from the user...
     */
 
-    snprintf(prompt, sizeof(prompt), "Password for %s on %s? ", cupsUser(),
-             http->hostname);
+    snprintf(prompt, sizeof(prompt), _("Password for %s on %s? "), cupsUser(),
+             http->hostname[0] == '/' ? "localhost" : http->hostname);
 
     http->digest_tries  = strncasecmp(http->fields[HTTP_FIELD_WWW_AUTHENTICATE],
-                                      "Basic", 5) == 0;
+                                      "Digest", 5) != 0;
     http->userpass[0]   = '\0';
 
     if ((password = cupsGetPassword(prompt)) == NULL)
@@ -129,13 +137,14 @@ cupsDoAuthentication(http_t     *http,	/* I - HTTP connection to server */
   * Got a password; encode it for the server...
   */
 
-  if (strncasecmp(http->fields[HTTP_FIELD_WWW_AUTHENTICATE], "Basic", 5) == 0)
+  if (strncmp(http->fields[HTTP_FIELD_WWW_AUTHENTICATE], "Digest", 6))
   {
    /*
     * Basic authentication...
     */
 
-    httpEncode64(encode, http->userpass);
+    httpEncode64_2(encode, sizeof(encode), http->userpass,
+                   strlen(http->userpass));
     snprintf(http->authstring, sizeof(http->authstring), "Basic %s", encode);
   }
   else
@@ -180,17 +189,17 @@ cups_local_auth(http_t *http)		/* I - HTTP connection to server */
   FILE		*fp;			/* Certificate file */
   char		filename[1024],		/* Certificate filename */
 		certificate[33];	/* Certificate string */
-  const char	*root;			/* Server root directory */
+  _cups_globals_t *cg = _cupsGlobals();	/* Global data */
 
 
-  DEBUG_printf(("cups_local_auth(http=%p) hostaddr=%08x, hostname=\"%s\"\n",
-                http, ntohl(http->hostaddr.sin_addr.s_addr), http->hostname));
+  DEBUG_printf(("cups_local_auth(http=%p) hostaddr=%s, hostname=\"%s\"\n",
+                http, httpAddrString(http->hostaddr, filename, sizeof(filename)), http->hostname));
 
  /*
   * See if we are accessing localhost...
   */
 
-  if (ntohl(http->hostaddr.sin_addr.s_addr) != 0x7f000001 &&
+  if (!httpAddrLocalhost(http->hostaddr) &&
       strcasecmp(http->hostname, "localhost") != 0)
   {
     DEBUG_puts("cups_local_auth: Not a local connection!");
@@ -202,17 +211,14 @@ cups_local_auth(http_t *http)		/* I - HTTP connection to server */
   * try the root certificate...
   */
 
-  if ((root = getenv("CUPS_SERVERROOT")) == NULL)
-    root = CUPS_SERVERROOT;
-
   pid = getpid();
-  snprintf(filename, sizeof(filename), "%s/certs/%d", root, pid);
+  snprintf(filename, sizeof(filename), "%s/certs/%d", cg->cups_statedir, pid);
   if ((fp = fopen(filename, "r")) == NULL && pid > 0)
   {
     DEBUG_printf(("cups_local_auth: Unable to open file %s: %s\n",
                   filename, strerror(errno)));
 
-    snprintf(filename, sizeof(filename), "%s/certs/0", root);
+    snprintf(filename, sizeof(filename), "%s/certs/0", cg->cups_statedir);
     fp = fopen(filename, "r");
   }
 
@@ -245,5 +251,5 @@ cups_local_auth(http_t *http)		/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: auth.c,v 1.10 2004/06/29 03:44:52 mike Exp $".
+ * End of "$Id: auth.c 5961 2006-09-16 19:08:36Z mike $".
  */
