@@ -1,9 +1,9 @@
 /*
- * "$Id: cups.h,v 1.59 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: cups.h 5930 2006-09-07 19:49:34Z mike $"
  *
  *   API definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -33,6 +33,19 @@
 
 #  include "ipp.h"
 #  include "ppd.h"
+#  include "language.h"
+
+
+/*
+ * With GCC 3.0 and higher, we can mark old APIs "deprecated" so you get
+ * a warning at compile-time.
+ */
+
+#  if defined(__GNUC__) && __GNUC__ > 2
+#    define _CUPS_DEPRECATED __attribute__ ((__deprecated__))
+#  else
+#    define _CUPS_DEPRECATED
+#  endif /* __GNUC__ && __GNUC__ > 2 */
 
 
 /*
@@ -48,10 +61,10 @@ extern "C" {
  * Constants...
  */
 
-#  define CUPS_VERSION		1.0123
+#  define CUPS_VERSION		1.0204
 #  define CUPS_VERSION_MAJOR	1
-#  define CUPS_VERSION_MINOR	1
-#  define CUPS_VERSION_PATCH	23
+#  define CUPS_VERSION_MINOR	2
+#  define CUPS_VERSION_PATCH	4
 #  define CUPS_DATE_ANY		-1
 
 
@@ -60,7 +73,7 @@ extern "C" {
  */
 
 typedef unsigned cups_ptype_t;		/**** Printer Type/Capability Bits ****/
-enum					/* Not a typedef'd enum so we can OR */
+enum cups_ptype_e			/* Not a typedef'd enum so we can OR */
 {
   CUPS_PRINTER_LOCAL = 0x0000,		/* Local printer or class */
   CUPS_PRINTER_CLASS = 0x0001,		/* Printer class */
@@ -83,16 +96,23 @@ enum					/* Not a typedef'd enum so we can OR */
   CUPS_PRINTER_DEFAULT = 0x20000,	/* Default printer on network */
   CUPS_PRINTER_FAX = 0x40000,		/* Fax queue */
   CUPS_PRINTER_REJECTING = 0x80000,	/* Printer is rejecting jobs */
-  CUPS_PRINTER_OPTIONS = 0x6fffc	/* ~(CLASS | REMOTE | IMPLICIT) */
+  CUPS_PRINTER_DELETE = 0x100000,	/* Delete printer @since CUPS 1.2@ */
+  CUPS_PRINTER_NOT_SHARED = 0x200000,	/* Printer is not shared @since CUPS 1.2@ */
+  CUPS_PRINTER_AUTHENTICATED = 0x400000,/* Printer requires authentication @since CUPS 1.2@ */
+  CUPS_PRINTER_COMMANDS = 0x800000,	/* Printer supports maintenance commands @since CUPS 1.2@ */
+  CUPS_PRINTER_OPTIONS = 0xe6fffc	/* ~(CLASS | REMOTE | IMPLICIT) */
 };
 
-typedef struct				/**** Printer Options ****/
+typedef const char *(*cups_password_cb_t)(const char *);
+					/**** Password callback ****/
+
+typedef struct cups_option_s		/**** Printer Options ****/
 {
   char		*name;			/* Name of option */
   char		*value;			/* Value of option */
 } cups_option_t;
 
-typedef struct				/**** Destination ****/
+typedef struct cups_dest_s		/**** Destination ****/
 {
   char		*name,			/* Printer or class name */
 		*instance;		/* Local instance name or NULL */
@@ -101,19 +121,19 @@ typedef struct				/**** Destination ****/
   cups_option_t	*options;		/* Options */
 } cups_dest_t;
 
-typedef struct				/**** Job ****/
+typedef struct cups_job_s		/**** Job ****/
 {
   int		id;			/* The job ID */
-  char		*dest,			/* Printer or class name */
-		*title,			/* Title/job name */
-		*user,			/* User the submitted the job */
-		*format;		/* Document format */
+  char		*dest;			/* Printer or class name */
+  char		*title;			/* Title/job name */
+  char		*user;			/* User the submitted the job */
+  char		*format;		/* Document format */
   ipp_jstate_t	state;			/* Job state */
-  int		size,			/* Size in kilobytes */
-		priority;		/* Priority (1-100) */
-  time_t	completed_time,		/* Time the job was completed */
-		creation_time,		/* Time the job was created */
-		processing_time;	/* Time the job was processed */
+  int		size;			/* Size in kilobytes */
+  int		priority;		/* Priority (1-100) */
+  time_t	completed_time;		/* Time the job was completed */
+  time_t	creation_time;		/* Time the job was created */
+  time_t	processing_time;	/* Time the job was processed */
 } cups_job_t;
 
 
@@ -122,17 +142,19 @@ typedef struct				/**** Job ****/
  */
 
 extern int		cupsCancelJob(const char *printer, int job);
-#define			cupsDoRequest(http,request,resource) cupsDoFileRequest((http),(request),(resource),NULL)
 extern ipp_t		*cupsDoFileRequest(http_t *http, ipp_t *request,
-			                   const char *resource, const char *filename);
+			                   const char *resource,
+					   const char *filename);
+extern ipp_t		*cupsDoRequest(http_t *http, ipp_t *request,
+			               const char *resource);
 extern http_encryption_t cupsEncryption(void);
 extern void		cupsFreeJobs(int num_jobs, cups_job_t *jobs);
-extern int		cupsGetClasses(char ***classes);
+extern int		cupsGetClasses(char ***classes) _CUPS_DEPRECATED;
 extern const char	*cupsGetDefault(void);
 extern int		cupsGetJobs(cups_job_t **jobs, const char *dest,
 			            int myjobs, int completed);
 extern const char	*cupsGetPPD(const char *printer);
-extern int		cupsGetPrinters(char ***printers);
+extern int		cupsGetPrinters(char ***printers) _CUPS_DEPRECATED;
 extern ipp_status_t	cupsLastError(void);
 extern int		cupsPrintFile(const char *printer, const char *filename,
 			              const char *title, int num_options,
@@ -140,7 +162,7 @@ extern int		cupsPrintFile(const char *printer, const char *filename,
 extern int		cupsPrintFiles(const char *printer, int num_files,
 			               const char **files, const char *title,
 				       int num_options, cups_option_t *options);
-extern char		*cupsTempFile(char *filename, int len);
+extern char		*cupsTempFile(char *filename, int len) _CUPS_DEPRECATED;
 extern int		cupsTempFd(char *filename, int len);
 
 extern int		cupsAddDest(const char *name, const char *instance,
@@ -158,15 +180,15 @@ extern void		cupsEncodeOptions(ipp_t *ipp, int num_options,
 extern void		cupsFreeOptions(int num_options, cups_option_t *options);
 extern const char	*cupsGetOption(const char *name, int num_options,
 			               cups_option_t *options);
-extern int		cupsParseOptions(const char *arg, int num_options,
-			                 cups_option_t **options);
 extern int		cupsMarkOptions(ppd_file_t *ppd, int num_options,
 			                cups_option_t *options);
+extern int		cupsParseOptions(const char *arg, int num_options,
+			                 cups_option_t **options);
 
 extern const char	*cupsGetPassword(const char *prompt);
 extern const char	*cupsServer(void);
 extern void		cupsSetEncryption(http_encryption_t e);
-extern void		cupsSetPasswordCB(const char *(*cb)(const char *));
+extern void		cupsSetPasswordCB(cups_password_cb_t cb);
 extern void		cupsSetServer(const char *server);
 extern void		cupsSetUser(const char *user);
 extern const char	*cupsUser(void);
@@ -199,6 +221,21 @@ extern int		cupsPrintFiles2(http_t *http, const char *printer,
 extern int		cupsSetDests2(http_t *http, int num_dests,
 			              cups_dest_t *dests);
 
+/**** New in CUPS 1.2 ****/
+extern ssize_t		cupsBackChannelRead(char *buffer, size_t bytes,
+			                    double timeout);
+extern ssize_t		cupsBackChannelWrite(const char *buffer, size_t bytes,
+			                     double timeout);
+extern void		cupsEncodeOptions2(ipp_t *ipp, int num_options,
+					   cups_option_t *options,
+					   ipp_tag_t group_tag);
+extern const char	*cupsLastErrorString(void);
+extern char		*cupsNotifySubject(cups_lang_t *lang, ipp_t *event);
+extern char		*cupsNotifyText(cups_lang_t *lang, ipp_t *event);
+extern int		cupsRemoveOption(const char *name, int num_options,
+			                 cups_option_t **options);
+extern cups_file_t	*cupsTempFile2(char *filename, int len);
+
 
 #  ifdef __cplusplus
 }
@@ -207,5 +244,5 @@ extern int		cupsSetDests2(http_t *http, int num_dests,
 #endif /* !_CUPS_CUPS_H_ */
 
 /*
- * End of "$Id: cups.h,v 1.59 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: cups.h 5930 2006-09-07 19:49:34Z mike $".
  */
