@@ -1,10 +1,10 @@
 /*
- * "$Id: dest.c,v 1.40 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: dest.c 5753 2006-07-18 19:53:24Z mike $"
  *
  *   User-defined destination (and option) support for the Common UNIX
  *   Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -29,9 +29,10 @@
  *   cupsAddDest()     - Add a destination to the list of destinations.
  *   cupsFreeDests()   - Free the memory used by the list of destinations.
  *   cupsGetDest()     - Get the named destination from the list.
- *   cupsGetDests()    - Get the list of destinations.
- *   cupsGetDests2()   - Get the list of destinations using a HTTP connection.
- *   cupsSetDests()    - Set the list of destinations.
+ *   cupsGetDests()    - Get the list of destinations from the default server.
+ *   cupsGetDests2()   - Get the list of destinations from the specified server.
+ *   cupsSetDests()    - Set the list of destinations for the default server.
+ *   cupsSetDests2()   - Set the list of destinations for the specified server.
  *   cups_get_dests()  - Get destinations from a file.
  *   cups_get_sdests() - Get destinations from a server.
  */
@@ -40,11 +41,14 @@
  * Include necessary headers...
  */
 
-#include "cups.h"
-#include "language.h"
-#include "string.h"
+#include "globals.h"
 #include <stdlib.h>
 #include <ctype.h>
+#include <sys/stat.h>
+
+#ifdef HAVE_NOTIFY_H
+#  include <notify.h>
+#endif /* HAVE_NOTIFY_H */
 
 
 /*
@@ -59,12 +63,15 @@ static int	cups_get_sdests(http_t *http, ipp_op_t op, int num_dests,
 
 /*
  * 'cupsAddDest()' - Add a destination to the list of destinations.
+ *
+ * Use the cupsSaveDests() function to save the updated list of destinations
+ * to the user's lpoptions file.
  */
 
-int					/* O - New number of destinations */
-cupsAddDest(const char  *name,		/* I - Name of destination */
-            const char	*instance,	/* I - Instance of destination */
-            int         num_dests,	/* I - Number of destinations */
+int					/* O  - New number of destinations */
+cupsAddDest(const char  *name,		/* I  - Name of destination */
+            const char	*instance,	/* I  - Instance of destination or NULL for none/primary */
+            int         num_dests,	/* I  - Number of destinations */
             cups_dest_t **dests)	/* IO - Destinations */
 {
   int		i;			/* Looping var */
@@ -94,7 +101,7 @@ cupsAddDest(const char  *name,		/* I - Name of destination */
   for (i = num_dests; i > 0; i --, dest ++)
     if (strcasecmp(name, dest->name) < 0)
       break;
-    else if (strcasecmp(name, dest->name) == 0 &&
+    else if (!strcasecmp(name, dest->name) &&
              instance != NULL && dest->instance != NULL &&
              strcasecmp(instance, dest->instance) < 0)
       break;
@@ -147,6 +154,9 @@ cupsFreeDests(int         num_dests,	/* I - Number of destinations */
 
 /*
  * 'cupsGetDest()' - Get the named destination from the list.
+ *
+ * Use the cupsGetDests() or cupsGetDests2() functions to get a
+ * list of supported destinations for the current user.
  */
 
 cups_dest_t *				/* O - Destination pointer or NULL */
@@ -204,7 +214,12 @@ cupsGetDest(const char  *name,		/* I - Name of destination */
 
 
 /*
- * 'cupsGetDests()' - Get the list of destinations.
+ * 'cupsGetDests()' - Get the list of destinations from the default server.
+ *
+ * Starting with CUPS 1.2, the returned list of destinations include the
+ * printer-info, printer-is-accepting-jobs, printer-is-shared,
+ * printer-make-and-model, printer-state, printer-state-change-time,
+ * printer-state-reasons, and printer-type attributes as options.
  */
 
 int					/* O - Number of destinations */
@@ -230,7 +245,14 @@ cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
 
 
 /*
- * 'cupsGetDests2()' - Get the list of destinations.
+ * 'cupsGetDests2()' - Get the list of destinations from the specified server.
+ *
+ * Starting with CUPS 1.2, the returned list of destinations include the
+ * printer-info, printer-is-accepting-jobs, printer-is-shared,
+ * printer-make-and-model, printer-state, printer-state-change-time,
+ * printer-state-reasons, and printer-type attributes as options.
+ *
+ * @since CUPS 1.1.21@
  */
 
 int					/* O - Number of destinations */
@@ -241,12 +263,13 @@ cupsGetDests2(http_t      *http,	/* I - HTTP connection */
   int		num_dests;		/* Number of destinations */
   cups_dest_t	*dest;			/* Destination pointer */
   const char	*home;			/* HOME environment variable */
-  char		filename[1024];		/* Local ~/.lpoptions file */
+  char		filename[1024];		/* Local ~/.cups/lpoptions file */
   const char	*defprinter;		/* Default printer */
   char		name[1024],		/* Copy of printer name */
 		*instance;		/* Pointer to instance name */
   int		num_reals;		/* Number of real queues */
   cups_dest_t	*reals;			/* Real queues */
+  _cups_globals_t *cg = _cupsGlobals();	/* Global data */
 
 
  /*
@@ -323,26 +346,24 @@ cupsGetDests2(http_t      *http,	/* I - HTTP connection */
   }
 
  /*
-  * Load the /etc/cups/lpoptions and ~/.lpoptions files...
+  * Load the /etc/cups/lpoptions and ~/.cups/lpoptions files...
   */
 
-  if ((home = getenv("CUPS_SERVERROOT")) != NULL)
-  {
-    snprintf(filename, sizeof(filename), "%s/lpoptions", home);
-    num_dests = cups_get_dests(filename, num_dests, dests);
-  }
-  else
-    num_dests = cups_get_dests(CUPS_SERVERROOT "/lpoptions", num_dests, dests);
+  snprintf(filename, sizeof(filename), "%s/lpoptions", cg->cups_serverroot);
+  num_dests = cups_get_dests(filename, num_dests, dests);
 
   if ((home = getenv("HOME")) != NULL)
   {
-    snprintf(filename, sizeof(filename), "%s/.lpoptions", home);
+    snprintf(filename, sizeof(filename), "%s/.cups/lpoptions", home);
+    if (access(filename, 0))
+      snprintf(filename, sizeof(filename), "%s/.lpoptions", home);
+
     num_dests = cups_get_dests(filename, num_dests, dests);
   }
 
  /*
   * Validate the current default destination - this prevents old
-  * Default lines in /etc/cups/lpoptions and ~/.lpoptions from
+  * Default lines in /etc/cups/lpoptions and ~/.cups/lpoptions from
   * pointing to a non-existent printer or class...
   */
 
@@ -391,7 +412,10 @@ cupsGetDests2(http_t      *http,	/* I - HTTP connection */
 
 
 /*
- * 'cupsSetDests()' - Set the list of destinations.
+ * 'cupsSetDests()' - Save the list of destinations for the default server.
+ *
+ * This function saves the destinations to /etc/cups/lpoptions when run
+ * as root and ~/.cups/lpoptions when run as a normal user.
  */
 
 void
@@ -415,7 +439,12 @@ cupsSetDests(int         num_dests,	/* I - Number of destinations */
 
 
 /*
- * 'cupsSetDests()' - Set the list of destinations.
+ * 'cupsSetDests2()' - Save the list of destinations for the specified server.
+ *
+ * This function saves the destinations to /etc/cups/lpoptions when run
+ * as root and ~/.cups/lpoptions when run as a normal user.
+ *
+ * @since CUPS 1.1.21@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -427,6 +456,7 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
   int		wrote;			/* Wrote definition? */
   cups_dest_t	*dest;			/* Current destination */
   cups_option_t	*option;		/* Current option */
+  _ipp_option_t	*match;			/* Matching attribute for option */
   FILE		*fp;			/* File pointer */
   const char	*home;			/* HOME environment variable */
   char		filename[1024];		/* lpoptions file */
@@ -434,6 +464,7 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
   cups_dest_t	*temps,			/* Temporary destinations */
 		*temp;			/* Current temporary dest */
   const char	*val;			/* Value of temporary option */
+  _cups_globals_t *cg = _cupsGlobals();	/* Global data */
 
 
  /*
@@ -454,10 +485,7 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
   * Figure out which file to write to...
   */
 
-  if ((home = getenv("CUPS_SERVERROOT")) != NULL)
-    snprintf(filename, sizeof(filename), "%s/lpoptions", home);
-  else
-    strcpy(filename, CUPS_SERVERROOT "/lpoptions");
+  snprintf(filename, sizeof(filename), "%s/lpoptions", cg->cups_serverroot);
 
 #ifndef WIN32
   if (getuid())
@@ -473,7 +501,24 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
     */
 
     if ((home = getenv("HOME")) != NULL)
+    {
+     /*
+      * Remove the old ~/.lpoptions file...
+      */
+
       snprintf(filename, sizeof(filename), "%s/.lpoptions", home);
+      unlink(filename);
+
+     /*
+      * Create ~/.cups subdirectory...
+      */
+
+      snprintf(filename, sizeof(filename), "%s/.cups", home);
+      if (access(filename, 0))
+        mkdir(filename, 0700);
+
+      snprintf(filename, sizeof(filename), "%s/.cups/lpoptions", home);
+    }
   }
 #endif /* !WIN32 */
 
@@ -487,6 +532,16 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
     return (-1);
   }
 
+#ifndef WIN32
+ /*
+  * Set the permissions to 0644 when saving to the /etc/cups/lpoptions
+  * file...
+  */
+
+  if (!getuid())
+    fchmod(fileno(fp), 0644);
+#endif /* !WIN32 */
+
  /*
   * Write each printer; each line looks like:
   *
@@ -513,17 +568,24 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
 
       for (j = dest->num_options, option = dest->options; j > 0; j --, option ++)
       {
+       /*
+        * See if this option is a printer attribute; if so, skip it...
+	*/
+
+        if ((match = _ippFindOption(option->name)) != NULL &&
+	    match->group_tag == IPP_TAG_PRINTER)
+	  continue;
+
        /*
 	* See if the server/global options match these; if so, don't
 	* write 'em.
 	*/
 
-        if (temp && (val = cupsGetOption(option->name, temp->num_options,
-	                                 temp->options)) != NULL)
-	{
-	  if (strcasecmp(val, option->value) == 0)
-	    continue;
-	}
+        if (temp &&
+	    (val = cupsGetOption(option->name, temp->num_options,
+	                         temp->options)) != NULL &&
+            !strcasecmp(val, option->value))
+	  continue;
 
        /*
         * Options don't match, write to the file...
@@ -539,10 +601,35 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
         
         if (option->value[0])
 	{
-	  if (strchr(option->value, ' ') != NULL)
-	    fprintf(fp, " %s=\"%s\"", option->name, option->value);
-          else
+	  if (strchr(option->value, ' ') ||
+	      strchr(option->value, '\\') ||
+	      strchr(option->value, '\"') ||
+	      strchr(option->value, '\''))
+	  {
+	   /*
+	    * Quote the value...
+	    */
+
+	    fprintf(fp, " %s=\"", option->name);
+
+	    for (val = option->value; *val; val ++)
+	    {
+	      if (strchr("\"\'\\", *val))
+	        putc('\\', fp);
+
+              putc(*val, fp);
+	    }
+
+	    putc('\"', fp);
+          }
+	  else
+	  {
+	   /*
+	    * Store the literal value...
+	    */
+
 	    fprintf(fp, " %s=%s", option->name, option->value);
+          }
 	}
 	else
 	  fprintf(fp, " %s", option->name);
@@ -553,16 +640,21 @@ cupsSetDests2(http_t      *http,	/* I - HTTP connection */
     }
 
  /*
-  * Free the temporary destinations...
+  * Free the temporary destinations and close the file...
   */
 
   cupsFreeDests(num_temps, temps);
 
+  fclose(fp);
+
+#ifdef HAVE_NOTIFY_POST
  /*
-  * Close the file and return...
+  * Send a notification so that MacOS X applications can know about the
+  * change, too.
   */
 
-  fclose(fp);
+  notify_post("com.apple.printerListChange");
+#endif /* HAVE_NOTIFY_POST */
 
   return (0);
 }
@@ -730,17 +822,35 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
                 int         num_dests,	/* I - Number of destinations */
                 cups_dest_t **dests)	/* IO - Destinations */
 {
+  int		i;			/* Looping var */
   cups_dest_t	*dest;			/* Current destination */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
-  cups_lang_t	*language;		/* Default language */
-  const char	*name;			/* printer-name attribute */
-  char		job_sheets[1024];	/* job-sheets option */
+  int		accepting,		/* printer-is-accepting-jobs attribute */
+		shared,			/* printer-is-shared attribute */
+		state,			/* printer-state attribute */
+		change_time,		/* printer-state-change-time attribute */
+		type;			/* printer-type attribute */
+  const char	*info,			/* printer-info attribute */
+		*make_model,		/* printer-make-and-model attribute */
+		*name;			/* printer-name attribute */
+  char		job_sheets[1024],	/* job-sheets option */
+		reasons[1024],		/* printer-state-reasons attribute */
+		*rptr,			/* Pointer into reasons string */
+		temp[255];		/* Temporary string for numbers */
   static const char * const pattrs[] =	/* Attributes we're interested in */
 		{
+		  "job-sheets-default",
+		  "printer-info",
+		  "printer-is-accepting-jobs",
+		  "printer-is-shared",
+		  "printer-make-and-model",
 		  "printer-name",
-		  "job-sheets-default"
+		  "printer-state",
+		  "printer-state-change-time",
+		  "printer-state-reasons",
+		  "printer-type"
 		};
 
 
@@ -750,27 +860,18 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
   *
   *    attributes-charset
   *    attributes-natural-language
+  *    requesting-user-name
   */
 
-  request = ippNew();
-
-  request->request.op.operation_id = op;
-  request->request.op.request_id   = 1;
-
-  language = cupsLangDefault();
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-               "attributes-charset", NULL, cupsLangEncoding(language));
-
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-               "attributes-natural-language", NULL, language->language);
-
-  cupsLangFree(language);
+  request = ippNewRequest(op);
 
   ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                 "requested-attributes", sizeof(pattrs) / sizeof(pattrs[0]),
 		NULL, pattrs);
 
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+               "requesting-user-name", NULL, cupsUser());
+
  /*
   * Do the request and get back a response...
   */
@@ -793,17 +894,21 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
       * Pull the needed attributes from this job...
       */
 
-      name = NULL;
+      accepting   = 0;
+      change_time = 0;
+      info        = NULL;
+      make_model  = NULL;
+      name        = NULL;
+      shared      = 1;
+      state       = IPP_PRINTER_IDLE;
+      type        = CUPS_PRINTER_LOCAL;
 
       strcpy(job_sheets, "");
+      strcpy(reasons, "");
 
       while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
       {
-        if (strcmp(attr->name, "printer-name") == 0 &&
-	    attr->value_tag == IPP_TAG_NAME)
-	  name = attr->values[0].string.text;
-
-        if (strcmp(attr->name, "job-sheets-default") == 0 &&
+        if (!strcmp(attr->name, "job-sheets-default") &&
 	    (attr->value_tag == IPP_TAG_KEYWORD ||
 	     attr->value_tag == IPP_TAG_NAME))
         {
@@ -811,8 +916,46 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
 	    snprintf(job_sheets, sizeof(job_sheets), "%s,%s",
 	             attr->values[0].string.text, attr->values[1].string.text);
 	  else
-	    strcpy(job_sheets, attr->values[0].string.text);
+	    strlcpy(job_sheets, attr->values[0].string.text,
+	            sizeof(job_sheets));
         }
+        else if (!strcmp(attr->name, "printer-info") &&
+	         attr->value_tag == IPP_TAG_TEXT)
+	  info = attr->values[0].string.text;
+	else if (!strcmp(attr->name, "printer-is-accepting-jobs") &&
+	         attr->value_tag == IPP_TAG_BOOLEAN)
+          accepting = attr->values[0].boolean;
+	else if (!strcmp(attr->name, "printer-is-shared") &&
+	         attr->value_tag == IPP_TAG_BOOLEAN)
+          shared = attr->values[0].boolean;
+        else if (!strcmp(attr->name, "printer-make-and-model") &&
+	         attr->value_tag == IPP_TAG_TEXT)
+	  make_model = attr->values[0].string.text;
+        else if (!strcmp(attr->name, "printer-name") &&
+	         attr->value_tag == IPP_TAG_NAME)
+	  name = attr->values[0].string.text;
+	else if (!strcmp(attr->name, "printer-state") &&
+	         attr->value_tag == IPP_TAG_ENUM)
+          state = attr->values[0].integer;
+	else if (!strcmp(attr->name, "printer-state-change-time") &&
+	         attr->value_tag == IPP_TAG_INTEGER)
+          change_time = attr->values[0].integer;
+        else if (!strcmp(attr->name, "printer-state-reasons") &&
+	         attr->value_tag == IPP_TAG_KEYWORD)
+	{
+	  strlcpy(reasons, attr->values[0].string.text, sizeof(reasons));
+	  for (i = 1, rptr = reasons + strlen(reasons);
+	       i < attr->num_values;
+	       i ++)
+	  {
+	    snprintf(rptr, sizeof(reasons) - (rptr - reasons), ",%s",
+	             attr->values[i].string.text);
+	    rptr += strlen(rptr);
+	  }
+	}
+	else if (!strcmp(attr->name, "printer-type") &&
+	         attr->value_tag == IPP_TAG_ENUM)
+          type = attr->values[0].integer;
 
         attr = attr->next;
       }
@@ -832,10 +975,56 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
       num_dests = cupsAddDest(name, NULL, num_dests, dests);
 
       if ((dest = cupsGetDest(name, NULL, num_dests, *dests)) != NULL)
+      {
         if (job_sheets[0])
-          dest->num_options = cupsAddOption("job-sheets", job_sheets, 0,
+          dest->num_options = cupsAddOption("job-sheets", job_sheets,
+	                                    dest->num_options,
 	                                    &(dest->options));
 
+        if (info)
+          dest->num_options = cupsAddOption("printer-info", info,
+	                                    dest->num_options,
+	                                    &(dest->options));
+
+        sprintf(temp, "%d", accepting);
+	dest->num_options = cupsAddOption("printer-is-accepting-jobs", temp,
+					  dest->num_options,
+					  &(dest->options));
+
+        sprintf(temp, "%d", shared);
+	dest->num_options = cupsAddOption("printer-is-shared", temp,
+					  dest->num_options,
+					  &(dest->options));
+
+        if (make_model)
+          dest->num_options = cupsAddOption("printer-make-and-model",
+	                                    make_model, dest->num_options,
+	                                    &(dest->options));
+
+        sprintf(temp, "%d", state);
+	dest->num_options = cupsAddOption("printer-state", temp,
+					  dest->num_options,
+					  &(dest->options));
+
+        if (change_time)
+	{
+	  sprintf(temp, "%d", change_time);
+	  dest->num_options = cupsAddOption("printer-state-change-time", temp,
+					    dest->num_options,
+					    &(dest->options));
+        }
+
+        if (reasons[0])
+          dest->num_options = cupsAddOption("printer-state-reasons", reasons,
+					    dest->num_options,
+	                                    &(dest->options));
+
+        sprintf(temp, "%d", type);
+	dest->num_options = cupsAddOption("printer-type", temp,
+					  dest->num_options,
+					  &(dest->options));
+      }
+
       if (attr == NULL)
 	break;
     }
@@ -852,5 +1041,5 @@ cups_get_sdests(http_t      *http,	/* I - HTTP connection */
 
 
 /*
- * End of "$Id: dest.c,v 1.40 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: dest.c 5753 2006-07-18 19:53:24Z mike $".
  */
