@@ -1,9 +1,9 @@
 /*
- * "$Id: emit.c,v 1.42 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: emit.c 5934 2006-09-11 14:54:40Z mike $"
  *
  *   PPD code emission routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -27,13 +27,19 @@
  *
  * Contents:
  *
- *   ppdCollect()       - Collect all marked options that reside in the
- *                        specified section.
- *   ppdEmit()          - Emit code for marked options to a file.
- *   ppdEmitFd()        - Emit code for marked options to a file.
- *   ppdEmitJCL()       - Emit code for JCL options to a file.
- *   ppd_handle_media() - Handle media selection...
- *   ppd_sort()         - Sort options by ordering numbers...
+ *   ppdCollect()        - Collect all marked options that reside in the
+ *                         specified section.
+ *   ppdCollect2()       - Collect all marked options that reside in the
+ *                         specified section and minimum order.
+ *   ppdEmit()           - Emit code for marked options to a file.
+ *   ppdEmitAfterOrder() - Emit a subset of the code for marked options to a
+ *                         file.
+ *   ppdEmitFd()         - Emit code for marked options to a file.
+ *   ppdEmitJCL()        - Emit code for JCL options to a file.
+ *   ppdEmitJCLEnd()     - Emit JCLEnd code to a file.
+ *   ppdEmitString()     - Get a string containing the code for marked options.
+ *   ppd_handle_media()  - Handle media selection...
+ *   ppd_sort()          - Sort options by ordering numbers...
  */
 
 /*
@@ -43,6 +49,8 @@
 #include "ppd.h"
 #include <stdlib.h>
 #include "string.h"
+#include <errno.h>
+#include "debug.h"
 
 #if defined(WIN32) || defined(__EMX__)
 #  include <io.h>
@@ -63,7 +71,7 @@ static int	ppd_sort(ppd_choice_t **c1, ppd_choice_t **c2);
  * Local globals...
  */
 
-static const char *ppd_custom_code =
+static const char ppd_custom_code[] =
 		"pop pop pop\n"
 		"<</PageSize[5 -2 roll]/ImagingBBox null>>setpagedevice\n";
 
@@ -77,6 +85,23 @@ int					/* O - Number of options marked */
 ppdCollect(ppd_file_t    *ppd,		/* I - PPD file data */
            ppd_section_t section,	/* I - Section to collect */
            ppd_choice_t  ***choices)	/* O - Pointers to choices */
+{
+  return (ppdCollect2(ppd, section, 0.0, choices));
+}
+
+
+/*
+ * 'ppdCollect2()' - Collect all marked options that reside in the
+ *                   specified section and minimum order.
+ *
+ * @since CUPS 1.2@
+ */
+
+int					/* O - Number of options marked */
+ppdCollect2(ppd_file_t    *ppd,		/* I - PPD file data */
+            ppd_section_t section,	/* I - Section to collect */
+	    float         min_order,	/* I - Minimum OrderDependency value */
+            ppd_choice_t  ***choices)	/* O - Pointers to choices */
 {
   int		i, j, k, m;		/* Looping vars */
   ppd_group_t	*g,			/* Current group */
@@ -87,6 +112,9 @@ ppdCollect(ppd_file_t    *ppd,		/* I - PPD file data */
   ppd_choice_t	**collect;		/* Collected choices */
 
 
+  DEBUG_printf(("ppdCollect2(ppd=%p, section=%d, min_order=%f, choices=%p)\n",
+                ppd, section, min_order, choices));
+
   if (ppd == NULL)
     return (0);
 
@@ -104,20 +132,24 @@ ppdCollect(ppd_file_t    *ppd,		/* I - PPD file data */
   for (i = ppd->num_groups, g = ppd->groups; i > 0; i --, g ++)
   {
     for (j = g->num_options, o = g->options; j > 0; j --, o ++)
-      if (o->section == section)
+      if (o->section == section && o->order >= min_order)
 	for (k = o->num_choices, c = o->choices; k > 0; k --, c ++)
 	  if (c->marked && count < 1000)
 	  {
+	    DEBUG_printf(("ppdCollect2: %s=%s marked...\n", o->keyword,
+	                  c->choice));
             collect[count] = c;
 	    count ++;
 	  }
 
     for (j = g->num_subgroups, sg = g->subgroups; j > 0; j --, sg ++)
       for (k = sg->num_options, o = sg->options; k > 0; k --, o ++)
-	if (o->section == section)
+	if (o->section == section && o->order >= min_order)
 	  for (m = o->num_choices, c = o->choices; m > 0; m --, c ++)
 	    if (c->marked && count < 1000)
 	    {
+	      DEBUG_printf(("ppdCollect2: %s=%s marked...\n", o->keyword,
+	                    c->choice));
               collect[count] = c;
 	      count ++;
 	    }
@@ -131,6 +163,8 @@ ppdCollect(ppd_file_t    *ppd,		/* I - PPD file data */
     qsort(collect, count, sizeof(ppd_choice_t *),
           (int (*)(const void *, const void *))ppd_sort);
 
+  DEBUG_printf(("ppdCollect2: %d marked choices...\n", count));
+
  /*
   * Return the array and number of choices; if 0, free the array since
   * it isn't needed.
@@ -159,12 +193,327 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
         FILE          *fp,		/* I - File to write to */
         ppd_section_t section)		/* I - Section to write */
 {
-  int		i,			/* Looping var */
+  return (ppdEmitAfterOrder(ppd, fp, section, 0, 0.0));
+}
+
+
+/*
+ * 'ppdEmitAfterOrder()' - Emit a subset of the code for marked options to a file.
+ *
+ * When "limit" is non-zero, this function only emits options whose
+ * OrderDependency value is greater than or equal to "min_order".
+ *
+ * When "limit" is zero, this function is identical to ppdEmit().
+ *
+ * @since CUPS 1.2@
+ */
+
+int					/* O - 0 on success, -1 on failure */
+ppdEmitAfterOrder(
+    ppd_file_t    *ppd,			/* I - PPD file record */
+    FILE          *fp,			/* I - File to write to */
+    ppd_section_t section,		/* I - Section to write */
+    int		  limit,		/* I - Non-zero to use min_order */
+    float         min_order)		/* I - Lowest OrderDependency */
+{
+  char	*buffer;			/* Option code */
+  int	status;				/* Return status */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!ppd || !fp)
+    return (-1);
+
+ /*
+  * Get the string...
+  */
+
+  buffer = ppdEmitString(ppd, section, min_order);
+
+ /*
+  * Write it as needed and return...
+  */
+
+  if (buffer)
+  {
+    status = fputs(buffer, fp) < 0 ? -1 : 0;
+
+    free(buffer);
+  }
+  else
+    status = 0;
+
+  return (status);
+}
+
+
+/*
+ * 'ppdEmitFd()' - Emit code for marked options to a file.
+ */
+
+int					/* O - 0 on success, -1 on failure */
+ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
+          int           fd,		/* I - File to write to */
+          ppd_section_t section)	/* I - Section to write */
+{
+  char		*buffer,		/* Option code */
+		*bufptr;		/* Pointer into code */
+  size_t	buflength;		/* Length of option code */
+  ssize_t	bytes;			/* Bytes written */
+  int		status;			/* Return status */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!ppd || fd < 0)
+    return (-1);
+
+ /*
+  * Get the string...
+  */
+
+  buffer = ppdEmitString(ppd, section, 0.0);
+
+ /*
+  * Write it as needed and return...
+  */
+
+  if (buffer)
+  {
+    buflength = strlen(buffer);
+    bufptr    = buffer;
+    bytes     = 0;
+
+    while (buflength > 0)
+    {
+      if ((bytes = write(fd, bufptr, buflength)) < 0)
+      {
+        if (errno == EAGAIN || errno == EINTR)
+	  continue;
+
+	break;
+      }
+
+      buflength -= bytes;
+      bufptr    += bytes;
+    }
+
+    status = bytes < 0 ? -1 : 0;
+
+    free(buffer);
+  }
+  else
+    status = 0;
+
+  return (status);
+}
+
+
+/*
+ * 'ppdEmitJCL()' - Emit code for JCL options to a file.
+ */
+
+int					/* O - 0 on success, -1 on failure */
+ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
+           FILE       *fp,		/* I - File to write to */
+           int        job_id,		/* I - Job ID */
+	   const char *user,		/* I - Username */
+	   const char *title)		/* I - Title */
+{
+  char		*ptr;			/* Pointer into JCL string */
+  char		temp[81];		/* Local title string */
+
+
+ /*
+  * Range check the input...
+  */
+
+  if (!ppd || !ppd->jcl_begin || !ppd->jcl_ps)
+    return (0);
+
+ /*
+  * See if the printer supports HP PJL...
+  */
+
+  if (!strncmp(ppd->jcl_begin, "\033%-12345X@", 10))
+  {
+   /*
+    * This printer uses HP PJL commands for output; filter the output
+    * so that we only have a single "@PJL JOB" command in the header...
+    *
+    * To avoid bugs in the PJL implementation of certain vendors' products
+    * (Xerox in particular), we add a dummy "@PJL" command at the beginning
+    * of the PJL commands to initialize PJL processing.
+    */
+
+    fputs("\033%-12345X@PJL\n", fp);
+    for (ptr = ppd->jcl_begin + 9; *ptr;)
+      if (!strncmp(ptr, "@PJL JOB", 8))
+      {
+       /*
+        * Skip job command...
+	*/
+
+        for (;*ptr; ptr ++)
+	  if (*ptr == '\n')
+	    break;
+
+	if (*ptr)
+	  ptr ++;
+      }
+      else
+      {
+       /*
+        * Copy line...
+	*/
+
+        for (;*ptr; ptr ++)
+	{
+	  putc(*ptr, fp);
+	  if (*ptr == '\n')
+	    break;
+	}
+
+	if (*ptr)
+	  ptr ++;
+      }
+
+   /*
+    * Eliminate any path info from the job title...
+    */
+
+    if ((ptr = strrchr(title, '/')) != NULL)
+      title = ptr + 1;
+
+   /*
+    * Replace double quotes with single quotes and 8-bit characters with
+    * question marks so that the title does not cause a PJL syntax error.
+    */
+
+    strlcpy(temp, title, sizeof(temp));
+
+    for (ptr = temp; *ptr; ptr ++)
+      if (*ptr == '\"')
+        *ptr = '\'';
+      else if (*ptr & 128)
+        *ptr = '?';
+
+   /*
+    * Send PJL JOB and PJL RDYMSG commands before we enter PostScript mode...
+    */
+
+    fprintf(fp, "@PJL JOB NAME = \"%s\" DISPLAY = \"%d %s %s\"\n", temp,
+            job_id, user, temp);
+    fprintf(fp, "@PJL RDYMSG DISPLAY = \"%d %s %s\"\n", job_id, user, temp);
+  }
+  else
+    fputs(ppd->jcl_begin, fp);
+
+  ppdEmit(ppd, fp, PPD_ORDER_JCL);
+  fputs(ppd->jcl_ps, fp);
+
+  return (0);
+}
+
+
+/*
+ * 'ppdEmitJCLEnd()' - Emit JCLEnd code to a file.
+ *
+ * @since CUPS 1.2@
+ */
+
+int					/* O - 0 on success, -1 on failure */
+ppdEmitJCLEnd(ppd_file_t *ppd,		/* I - PPD file record */
+              FILE       *fp)		/* I - File to write to */
+{
+ /*
+  * Range check the input...
+  */
+
+  if (!ppd)
+    return (0);
+
+  if (!ppd->jcl_end)
+  {
+    if (ppd->num_filters == 0)
+      putc(0x04, fp);
+
+    return (0);
+  }
+
+ /*
+  * See if the printer supports HP PJL...
+  */
+
+  if (!strncmp(ppd->jcl_end, "\033%-12345X@", 10))
+  {
+   /*
+    * This printer uses HP PJL commands for output; filter the output
+    * so that we only have a single "@PJL JOB" command in the header...
+    *
+    * To avoid bugs in the PJL implementation of certain vendors' products
+    * (Xerox in particular), we add a dummy "@PJL" command at the beginning
+    * of the PJL commands to initialize PJL processing.
+    */
+
+    fputs("\033%-12345X@PJL\n", fp);
+    fputs("@PJL RDYMSG DISPLAY = \"READY\"\n", fp);
+    fputs(ppd->jcl_end + 9, fp);
+  }
+  else
+    fputs(ppd->jcl_end, fp);
+
+  return (0);
+}
+
+
+/*
+ * 'ppdEmitString()' - Get a string containing the code for marked options.
+ *
+ * When "min_order" is greater than zero, this function only includes options
+ * whose OrderDependency value is greater than or equal to "min_order".
+ * Otherwise, all options in the specified section are included in the
+ * returned string.
+ *
+ * The return string is allocated on the heap and should be freed using
+ * free() when you are done with it.
+ *
+ * @since CUPS 1.2@
+ */
+
+char *					/* O - String containing option code */
+ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
+              ppd_section_t section,	/* I - Section to write */
+	      float         min_order)	/* I - Lowest OrderDependency */
+{
+  int		i, j,			/* Looping vars */
 		count;			/* Number of choices */
   ppd_choice_t	**choices;		/* Choices */
   ppd_size_t	*size;			/* Custom page size */
+  ppd_coption_t	*coption;		/* Custom option */
+  ppd_cparam_t	*cparam;		/* Custom parameter */
+  size_t	bufsize;		/* Size of string buffer needed */
+  char		*buffer,		/* String buffer */
+		*bufptr,		/* Pointer into buffer */
+		*bufend;		/* End of buffer */
+  struct lconv	*loc;			/* Locale data */
 
 
+  DEBUG_printf(("ppdEmitString(ppd=%p, section=%d, min_order=%f)\n",
+                ppd, section, min_order));
+
+ /*
+  * Range check input...
+  */
+
+  if (!ppd)
+    return (NULL);
+
  /*
   * Use PageSize or PageRegion as required...
   */
@@ -172,33 +521,118 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
   ppd_handle_media(ppd);
 
  /*
-  * Collect the options we need to emit and emit them!
+  * Collect the options we need to emit...
   */
 
-  if ((count = ppdCollect(ppd, section, &choices)) == 0)
-    return (0);
+  if ((count = ppdCollect2(ppd, section, min_order, &choices)) == 0)
+    return (NULL);
 
-  for (i = 0; i < count; i ++)
+ /*
+  * Count the number of bytes that are required to hold all of the
+  * option code...
+  */
+
+  for (i = 0, bufsize = 1; i < count; i ++)
+  {
+    if (section != PPD_ORDER_EXIT && section != PPD_ORDER_JCL)
+    {
+      bufsize += 3;			/* [{\n */
+
+      if ((!strcasecmp(choices[i]->option->keyword, "PageSize") ||
+           !strcasecmp(choices[i]->option->keyword, "PageRegion")) &&
+          !strcasecmp(choices[i]->choice, "Custom"))
+      {
+        DEBUG_puts("ppdEmitString: Custom size set!");
+
+        bufsize += 37;			/* %%BeginFeature: *CustomPageSize True\n */
+        bufsize += 50;			/* Five 9-digit numbers + newline */
+      }
+      else if (!strcasecmp(choices[i]->choice, "Custom") &&
+               (coption = ppdFindCustomOption(ppd,
+	                                      choices[i]->option->keyword))
+	           != NULL)
+      {
+        bufsize += 17 + strlen(choices[i]->option->keyword) + 6;
+					/* %%BeginFeature: *keyword True\n */
+
+        
+        for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+	     cparam;
+	     cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+	{
+          switch (cparam->type)
+	  {
+	    case PPD_CUSTOM_CURVE :
+	    case PPD_CUSTOM_INVCURVE :
+	    case PPD_CUSTOM_POINTS :
+	    case PPD_CUSTOM_REAL :
+	    case PPD_CUSTOM_INT :
+	        bufsize += 10;
+	        break;
+
+	    case PPD_CUSTOM_PASSCODE :
+	    case PPD_CUSTOM_PASSWORD :
+	    case PPD_CUSTOM_STRING :
+	        bufsize += 3 + 4 * strlen(cparam->current.custom_string);
+	        break;
+          }
+	}
+      }
+      else
+        bufsize += 17 + strlen(choices[i]->option->keyword) + 1 +
+	           strlen(choices[i]->choice) + 1;
+					/* %%BeginFeature: *keyword choice\n */
+
+      bufsize += 13;			/* %%EndFeature\n */
+      bufsize += 22;			/* } stopped cleartomark\n */
+    }
+
+    if (choices[i]->code)
+      bufsize += strlen(choices[i]->code) + 1;
+    else
+      bufsize += strlen(ppd_custom_code);
+  }
+
+ /*
+  * Allocate memory...
+  */
+
+  DEBUG_printf(("ppdEmitString: Allocating %d bytes for string...\n", bufsize));
+
+  if ((buffer = calloc(1, bufsize)) == NULL)
+  {
+    free(choices);
+    return (NULL);
+  }
+
+  bufend = buffer + bufsize - 1;
+  loc    = localeconv();
+
+ /*
+  * Copy the option code to the buffer...
+  */
+
+  for (i = 0, bufptr = buffer; i < count; i ++, bufptr += strlen(bufptr))
     if (section != PPD_ORDER_EXIT && section != PPD_ORDER_JCL)
     {
      /*
-      * Send wrapper commands to prevent printer errors for unsupported
+      * Add wrapper commands to prevent printer errors for unsupported
       * options...
       */
 
-      if (fputs("[{\n", fp) < 0)
-      {
-        free(choices);
-        return (-1);
-      }
+      strlcpy(bufptr, "[{\n", bufend - bufptr + 1);
+      bufptr += 3;
 
      /*
       * Send DSC comments with option...
       */
 
-      if ((strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageSize") == 0 ||
-           strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageRegion") == 0) &&
-          strcasecmp(choices[i]->choice, "Custom") == 0)
+      DEBUG_printf(("Adding code for %s=%s...\n", choices[i]->option->keyword,
+                    choices[i]->choice));
+
+      if ((!strcasecmp(choices[i]->option->keyword, "PageSize") ||
+           !strcasecmp(choices[i]->option->keyword, "PageRegion")) &&
+          !strcasecmp(choices[i]->choice, "Custom"))
       {
        /*
         * Variable size; write out standard size options, using the
@@ -207,11 +641,13 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
 
         ppd_attr_t	*attr;		/* PPD attribute */
 	int		pos,		/* Position of custom value */
-			values[5],	/* Values for custom command */
 			orientation;	/* Orientation to use */
+	float		values[5];	/* Values for custom command */
 
 
-        fputs("%%BeginFeature: *CustomPageSize True\n", fp);
+        strlcpy(bufptr, "%%BeginFeature: *CustomPageSize True\n",
+	        bufend - bufptr + 1);
+        bufptr += 37;
 
         size = ppdPageSize(ppd, "Custom");
 
@@ -227,7 +663,7 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
 	else
 	  pos = 0;
 
-	values[pos] = (int)size->width;
+	values[pos] = size->width;
 
 	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Height")) != NULL)
 	{
@@ -239,7 +675,7 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
 	else
 	  pos = 1;
 
-	values[pos] = (int)size->length;
+	values[pos] = size->length;
 
        /*
         * According to the Adobe PPD specification, an orientation of 1
@@ -291,217 +727,13 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
 
 	values[pos] = orientation;
 
-        fprintf(fp, "%d %d %d %d %d\n", values[0], values[1],
-	        values[2], values[3], values[4]);
-
-	if (choices[i]->code == NULL)
+        for (pos = 0; pos < 5; pos ++)
 	{
-	 /*
-	  * This can happen with certain buggy PPD files that don't include
-	  * a CustomPageSize command sequence...  We just use a generic
-	  * Level 2 command sequence...
-	  */
-
-	  fputs(ppd_custom_code, fp);
-	}
-      }
-      else if (fprintf(fp, "%%%%BeginFeature: *%s %s\n",
-                       ((ppd_option_t *)choices[i]->option)->keyword,
-		       choices[i]->choice) < 0)
-      {
-        free(choices);
-        return (-1);
-      }
-
-      if (choices[i]->code != NULL && choices[i]->code[0] != '\0')
-      {
-        if (fputs(choices[i]->code, fp) < 0)
-        {
-          free(choices);
-          return (-1);
+	  bufptr    = _cupsStrFormatd(bufptr, bufend, values[pos], loc);
+	  *bufptr++ = '\n';
         }
 
-        if (choices[i]->code[strlen(choices[i]->code) - 1] != '\n')
-          putc('\n', fp);
-      }
-
-      if (fputs("%%EndFeature\n", fp) < 0)
-      {
-        free(choices);
-        return (-1);
-      }
-
-      if (fputs("} stopped cleartomark\n", fp) < 0)
-      {
-        free(choices);
-        return (-1);
-      }
-    }
-    else if (fputs(choices[i]->code, fp) < 0)
-    {
-      free(choices);
-      return (-1);
-    }
-
-  free(choices);
-  return (0);
-}
-
-
-/*
- * 'ppdEmitFd()' - Emit code for marked options to a file.
- */
-
-int					/* O - 0 on success, -1 on failure */
-ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
-          int           fd,		/* I - File to write to */
-          ppd_section_t section)	/* I - Section to write */
-{
-  int		i,			/* Looping var */
-		count,			/* Number of choices */
-		custom_size;		/* Non-zero if this option is a custom size */
-  ppd_choice_t	**choices;		/* Choices */
-  ppd_size_t	*size;			/* Custom page size */
-  char		buf[1024];		/* Output buffer for feature */
-
-
- /*
-  * Use PageSize or PageRegion as required...
-  */
-
-  ppd_handle_media(ppd);
-
- /*
-  * Collect the options we need to emit and emit them!
-  */
-
-  if ((count = ppdCollect(ppd, section, &choices)) == 0)
-    return (0);
-
-  for (i = 0; i < count; i ++)
-    if (section != PPD_ORDER_EXIT && section != PPD_ORDER_JCL)
-    {
-     /*
-      * Send wrapper commands to prevent printer errors for unsupported
-      * options...
-      */
-
-      if (write(fd, "[{\n", 3) < 1)
-      {
-        free(choices);
-        return (-1);
-      }
-
-     /*
-      * Send DSC comments with option...
-      */
-
-      if ((strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageSize") == 0 ||
-           strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageRegion") == 0) &&
-          strcasecmp(choices[i]->choice, "Custom") == 0)
-      {
-        custom_size = 1;
-
-	strcpy(buf, "%%BeginFeature: *CustomPageSize True\n");
-      }
-      else
-      {
-        custom_size = 0;
-
-	snprintf(buf, sizeof(buf), "%%%%BeginFeature: *%s %s\n",
-        	 ((ppd_option_t *)choices[i]->option)->keyword,
-		 choices[i]->choice);
-      }
-
-      if (write(fd, buf, strlen(buf)) < 1)
-      {
-        free(choices);
-        return (-1);
-      }
-
-      if (custom_size)
-      {
-       /*
-        * Variable size; write out standard size options, using the
-	* parameter positions defined in the PPD file...
-	*/
-
-        ppd_attr_t	*attr;		/* PPD attribute */
-	int		pos,		/* Position of custom value */
-			values[5],	/* Values for custom command */
-			orientation;	/* Orientation to use */
-
-
-        size = ppdPageSize(ppd, "Custom");
-
-        memset(values, 0, sizeof(values));
-
-	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Width")) != NULL)
-	{
-	  pos = atoi(attr->value) - 1;
-
-          if (pos < 0 || pos > 4)
-	    pos = 0;
-	}
-	else
-	  pos = 0;
-
-	values[pos] = (int)size->width;
-
-	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Height")) != NULL)
-	{
-	  pos = atoi(attr->value) - 1;
-
-          if (pos < 0 || pos > 4)
-	    pos = 1;
-	}
-	else
-	  pos = 1;
-
-	values[pos] = (int)size->length;
-
-        if (size->width < size->length)
-	  orientation = 1;
-	else
-	  orientation = 0;
-
-	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize",
-	                        "Orientation")) != NULL)
-	{
-	  int min_orient, max_orient;	/* Minimum and maximum orientations */
-
-
-          if (sscanf(attr->value, "%d%*s%d%d", &pos, &min_orient,
-	             &max_orient) != 3)
-	    pos = 4;
-	  else
-	  {
-	    pos --;
-
-            if (pos < 0 || pos > 4)
-	      pos = 4;
-
-            if (orientation > max_orient)
-	      orientation = max_orient;
-	    else if (orientation < min_orient)
-	      orientation = min_orient;
-	  }
-	}
-	else
-	  pos = 4;
-
-	values[pos] = orientation;
-
-        snprintf(buf, sizeof(buf), "%d %d %d %d %d\n", values[0], values[1],
-	         values[2], values[3], values[4]);
-
-	if (write(fd, buf, strlen(buf)) < 1)
-	{
-          free(choices);
-          return (-1);
-	}
-
-	if (choices[i]->code == NULL)
+	if (!choices[i]->code)
 	{
 	 /*
 	  * This can happen with certain buggy PPD files that don't include
@@ -509,147 +741,106 @@ ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
 	  * Level 2 command sequence...
 	  */
 
-	  if (write(fd, ppd_custom_code, strlen(ppd_custom_code)) < 1)
-	  {
-            free(choices);
-            return (-1);
-	  }
+	  strlcpy(bufptr, ppd_custom_code, bufend - bufptr + 1);
+          bufptr += strlen(bufptr);
 	}
       }
-
-      if (choices[i]->code != NULL && choices[i]->code[0] != '\0')
-      {
-	if (write(fd, choices[i]->code, strlen(choices[i]->code)) < 1)
-	{
-          free(choices);
-          return (-1);
-	}
-      }
-
-      if (write(fd, "%%EndFeature\n", 13) < 1)
-      {
-        free(choices);
-        return (-1);
-      }
-
-      if (write(fd, "} stopped cleartomark\n", 22) < 1)
-      {
-        free(choices);
-        return (-1);
-      }
-    }
-    else if (write(fd, choices[i]->code, strlen(choices[i]->code)) < 1)
-    {
-      free(choices);
-      return (-1);
-    }
-
-  free(choices);
-  return (0);
-}
-
-
-/*
- * 'ppdEmitJCL()' - Emit code for JCL options to a file.
- */
-
-int					/* O - 0 on success, -1 on failure */
-ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
-           FILE       *fp,		/* I - File to write to */
-           int        job_id,		/* I - Job ID */
-	   const char *user,		/* I - Username */
-	   const char *title)		/* I - Title */
-{
-  char		*ptr;			/* Pointer into JCL string */
-  char		temp[81];		/* Local title string */
-
-
- /*
-  * Range check the input...
-  */
-
-  if (ppd == NULL || ppd->jcl_begin == NULL || ppd->jcl_ps == NULL)
-    return (0);
-
- /*
-  * See if the printer supports HP PJL...
-  */
-
-  if (strncmp(ppd->jcl_begin, "\033%-12345X@", 10) == 0)
-  {
-   /*
-    * This printer uses HP PJL commands for output; filter the output
-    * so that we only have a single "@PJL JOB" command in the header...
-    *
-    * To avoid bugs in the PJL implementation of certain vendors' products
-    * (Xerox in particular), we add a dummy "@PJL" command at the beginning
-    * of the PJL commands to initialize PJL processing.
-    */
-
-    fputs("\033%-12345X@PJL\n", fp);
-    for (ptr = ppd->jcl_begin + 9; *ptr;)
-      if (strncmp(ptr, "@PJL JOB", 8) == 0)
+      else if (!strcasecmp(choices[i]->choice, "Custom") &&
+               (coption = ppdFindCustomOption(ppd,
+	                                      choices[i]->option->keyword))
+	           != NULL)
       {
        /*
-        * Skip job command...
+        * Custom option...
 	*/
 
-        for (;*ptr; ptr ++)
-	  if (*ptr == '\n')
-	    break;
+        const char	*s;		/* Pointer into string value */
 
-	if (*ptr)
-	  ptr ++;
+
+        snprintf(bufptr, bufend - bufptr + 1,
+	         "%%%%BeginFeature: *Custom%s True\n", coption->keyword);
+        bufptr += strlen(bufptr);
+
+        for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+	     cparam;
+	     cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+	{
+          switch (cparam->type)
+	  {
+	    case PPD_CUSTOM_CURVE :
+	    case PPD_CUSTOM_INVCURVE :
+	    case PPD_CUSTOM_POINTS :
+	    case PPD_CUSTOM_REAL :
+	        bufptr    = _cupsStrFormatd(bufptr, bufend,
+		                            cparam->current.custom_real, loc);
+                *bufptr++ = '\n';
+	        break;
+
+	    case PPD_CUSTOM_INT :
+	        snprintf(bufptr, bufend - bufptr + 1, "%d\n",
+		         cparam->current.custom_int);
+		bufptr += strlen(bufptr);
+	        break;
+
+	    case PPD_CUSTOM_PASSCODE :
+	    case PPD_CUSTOM_PASSWORD :
+	    case PPD_CUSTOM_STRING :
+	        *bufptr++ = '(';
+
+		for (s = cparam->current.custom_string; *s; s ++)
+		  if (*s < ' ' || *s == '(' || *s == ')' || *s >= 127)
+		  {
+		    snprintf(bufptr, bufend - bufptr + 1, "\\%03o", *s & 255);
+		    bufptr += strlen(bufptr);
+		  }
+		  else
+		    *bufptr++ = *s;
+
+	        *bufptr++ = ')';
+		*bufptr++ = '\n';
+	        break;
+          }
+	}
       }
       else
       {
-       /*
-        * Copy line...
-	*/
-
-        for (;*ptr; ptr ++)
-	{
-	  putc(*ptr, fp);
-	  if (*ptr == '\n')
-	    break;
-	}
-
-	if (*ptr)
-	  ptr ++;
+        snprintf(bufptr, bufend - bufptr + 1, "%%%%BeginFeature: *%s %s\n",
+                 choices[i]->option->keyword, choices[i]->choice);
+	bufptr += strlen(bufptr);
       }
 
-   /*
-    * Eliminate any path info from the job title...
-    */
+      if (choices[i]->code && choices[i]->code[0])
+      {
+        j = strlen(choices[i]->code);
+	memcpy(bufptr, choices[i]->code, j);
+	bufptr += j;
 
-    if ((ptr = strrchr(title, '/')) != NULL)
-      title = ptr + 1;
+	if (choices[i]->code[j - 1] != '\n')
+	  *bufptr++ = '\n';
+      }
 
-   /*
-    * Replace double quotes with single quotes so that the title
-    * does not cause a PJL syntax error.
-    */
+      strlcpy(bufptr, "%%EndFeature\n"
+		      "} stopped cleartomark\n", bufend - bufptr + 1);
+      bufptr += strlen(bufptr);
 
-    strlcpy(temp, title, sizeof(temp));
+      DEBUG_printf(("ppdEmitString: Offset in string is %d...\n",
+                    bufptr - buffer));
+    }
+    else
+    {
+      strlcpy(bufptr, choices[i]->code, bufend - bufptr + 1);
+      bufptr += strlen(bufptr);
+    }
 
-    for (ptr = temp; *ptr; ptr ++)
-      if (*ptr == '\"')
-        *ptr = '\'';
+ /*
+  * Nul-terminate, free, and return...
+  */
 
-   /*
-    * Send PJL JOB command before we enter PostScript mode...
-    */
+  *bufptr = '\0';
 
-    fprintf(fp, "@PJL JOB NAME = \"%s\" DISPLAY = \"%d %s %s\"\n", temp,
-            job_id, user, temp);
-  }
-  else
-    fputs(ppd->jcl_begin, fp);
+  free(choices);
 
-  ppdEmit(ppd, fp, PPD_ORDER_JCL);
-  fputs(ppd->jcl_ps, fp);
-
-  return (0);
+  return (buffer);
 }
 
 
@@ -688,10 +879,9 @@ ppd_handle_media(ppd_file_t *ppd)
   if (!rpr)
     rpr = ppdFindAttr(ppd, "RequiresPageRegion", "All");
 
-  if (strcasecmp(size->name, "Custom") == 0 ||
-      (manual_feed == NULL && input_slot == NULL) ||
-      (manual_feed != NULL && strcasecmp(manual_feed->choice, "False") == 0) ||
-      (input_slot != NULL && (input_slot->code == NULL || !input_slot->code[0])))
+  if (!strcasecmp(size->name, "Custom") || (!manual_feed && !input_slot) ||
+      !((manual_feed && !strcasecmp(manual_feed->choice, "True")) ||
+        (input_slot && input_slot->code && input_slot->code[0])))
   {
    /*
     * Manual feed was not selected and/or the input slot selection does
@@ -742,9 +932,9 @@ static int			/* O - -1 if c1 < c2, 0 if equal, 1 otherwise */
 ppd_sort(ppd_choice_t **c1,	/* I - First choice */
          ppd_choice_t **c2)	/* I - Second choice */
 {
-  if (((ppd_option_t *)(*c1)->option)->order < ((ppd_option_t *)(*c2)->option)->order)
+  if ((*c1)->option->order < (*c2)->option->order)
     return (-1);
-  else if (((ppd_option_t *)(*c1)->option)->order > ((ppd_option_t *)(*c2)->option)->order)
+  else if ((*c1)->option->order > (*c2)->option->order)
     return (1);
   else
     return (0);
@@ -752,5 +942,5 @@ ppd_sort(ppd_choice_t **c1,	/* I - First choice */
 
 
 /*
- * End of "$Id: emit.c,v 1.42 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: emit.c 5934 2006-09-11 14:54:40Z mike $".
  */
