@@ -1,9 +1,9 @@
 /*
- * "$Id: options.c,v 1.35 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: options.c 5151 2006-02-22 22:43:17Z mike $"
  *
  *   Option routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -25,11 +25,12 @@
  *
  * Contents:
  *
- *   cupsAddOption()    - Add an option to an option array.
- *   cupsFreeOptions()  - Free all memory used by options.
- *   cupsGetOption()    - Get an option value.
- *   cupsParseOptions() - Parse options from a command-line argument.
- *   cupsMarkOptions()  - Mark command-line options in a PPD file.
+ *   cupsAddOption()     - Add an option to an option array.
+ *   cupsFreeOptions()   - Free all memory used by options.
+ *   cupsGetOption()     - Get an option value.
+ *   cupsMarkOptions()   - Mark command-line options in a PPD file.
+ *   cupsParseOptions()  - Parse options from a command-line argument.
+ *   cupsRemoveOptions() - Remove an option from an option array.
  */
 
 /*
@@ -47,14 +48,14 @@
  * 'cupsAddOption()' - Add an option to an option array.
  */
 
-int						/* O - Number of options */
-cupsAddOption(const char    *name,		/* I - Name of option */
-              const char    *value,		/* I - Value of option */
-	      int           num_options,	/* I - Number of options */
-              cups_option_t **options)		/* IO - Pointer to options */
+int					/* O - Number of options */
+cupsAddOption(const char    *name,	/* I - Name of option */
+              const char    *value,	/* I - Value of option */
+	      int           num_options,/* I - Number of options */
+              cups_option_t **options)	/* IO - Pointer to options */
 {
-  int		i;				/* Looping var */
-  cups_option_t	*temp;				/* Pointer to new option */
+  int		i;			/* Looping var */
+  cups_option_t	*temp;			/* Pointer to new option */
 
 
   if (name == NULL || !name[0] || value == NULL ||
@@ -109,10 +110,11 @@ cupsAddOption(const char    *name,		/* I - Name of option */
  */
 
 void
-cupsFreeOptions(int           num_options,	/* I - Number of options */
-                cups_option_t *options)		/* I - Pointer to options */
+cupsFreeOptions(
+    int           num_options,		/* I - Number of options */
+    cups_option_t *options)		/* I - Pointer to options */
 {
-  int	i;					/* Looping var */
+  int	i;				/* Looping var */
 
 
   if (num_options <= 0 || options == NULL)
@@ -152,18 +154,249 @@ cupsGetOption(const char    *name,	/* I - Name of option */
 
 
 /*
- * 'cupsParseOptions()' - Parse options from a command-line argument.
+ * 'cupsMarkOptions()' - Mark command-line options in a PPD file.
  */
 
-int						/* O - Number of options found */
-cupsParseOptions(const char    *arg,		/* I - Argument to parse */
-                 int           num_options,	/* I - Number of options */
-                 cups_option_t **options)	/* O - Options found */
+int					/* O - 1 if conflicting */
+cupsMarkOptions(
+    ppd_file_t    *ppd,			/* I - PPD file */
+    int           num_options,		/* I - Number of options */
+    cups_option_t *options)		/* I - Options */
 {
-  char	*copyarg,				/* Copy of input string */
-	*ptr,					/* Pointer into string */
-	*name,					/* Pointer to name */
-	*value;					/* Pointer to value */
+  int		i, j, k;		/* Looping vars */
+  int		conflict;		/* Option conflicts */
+  char		*val,			/* Pointer into value */
+		*ptr,			/* Pointer into string */
+		s[255];			/* Temporary string */
+  const char	*page_size;		/* PageSize option */
+  cups_option_t	*optptr;		/* Current option */
+  ppd_option_t	*option;		/* PPD option */
+  static const char * const duplex_options[] =
+		{			/* Duplex option names */
+		  "Duplex",		/* Adobe */
+		  "EFDuplex",		/* EFI */
+		  "EFDuplexing",	/* EFI */
+		  "KD03Duplex",		/* Kodak */
+		  "JCLDuplex"		/* Samsung */
+		};
+  static const char * const duplex_one[] =
+		{			/* one-sided names */
+		  "None",
+		  "False"
+		};
+  static const char * const duplex_two_long[] =
+		{			/* two-sided-long-edge names */
+		  "DuplexNoTumble",	/* Adobe */
+		  "LongEdge",		/* EFI */
+		  "Top"			/* EFI */
+		};
+  static const char * const duplex_two_short[] =
+		{			/* two-sided-long-edge names */
+		  "DuplexTumble",	/* Adobe */
+		  "ShortEdge",		/* EFI */
+		  "Bottom"		/* EFI */
+		};
+
+
+ /*
+  * Check arguments...
+  */
+
+  if (ppd == NULL || num_options <= 0 || options == NULL)
+    return (0);
+
+ /*
+  * Mark options...
+  */
+
+  conflict  = 0;
+
+  for (i = num_options, optptr = options; i > 0; i --, optptr ++)
+    if (!strcasecmp(optptr->name, "media"))
+    {
+     /*
+      * Loop through the option string, separating it at commas and
+      * marking each individual option as long as the corresponding
+      * PPD option (PageSize, InputSlot, etc.) is not also set.
+      *
+      * For PageSize, we also check for an empty option value since
+      * some versions of MacOS X use it to specify auto-selection
+      * of the media based solely on the size.
+      */
+
+      page_size = cupsGetOption("PageSize", num_options, options);
+
+      for (val = optptr->value; *val;)
+      {
+       /*
+        * Extract the sub-option from the string...
+	*/
+
+        for (ptr = s; *val && *val != ',' && (ptr - s) < (sizeof(s) - 1);)
+	  *ptr++ = *val++;
+	*ptr++ = '\0';
+
+	if (*val == ',')
+	  val ++;
+
+       /*
+        * Mark it...
+	*/
+
+        if (!page_size || !page_size[0])
+	  if (ppdMarkOption(ppd, "PageSize", s))
+            conflict = 1;
+
+        if (cupsGetOption("InputSlot", num_options, options) == NULL)
+	  if (ppdMarkOption(ppd, "InputSlot", s))
+            conflict = 1;
+
+        if (cupsGetOption("MediaType", num_options, options) == NULL)
+	  if (ppdMarkOption(ppd, "MediaType", s))
+            conflict = 1;
+
+        if (cupsGetOption("EFMediaType", num_options, options) == NULL)
+	  if (ppdMarkOption(ppd, "EFMediaType", s))
+            conflict = 1;
+
+        if (cupsGetOption("EFMediaQualityMode", num_options, options) == NULL)
+	  if (ppdMarkOption(ppd, "EFMediaQualityMode", s))	/* EFI */
+            conflict = 1;
+
+	if (strcasecmp(s, "manual") == 0 &&
+	    cupsGetOption("ManualFeed", num_options, options) == NULL)
+          if (ppdMarkOption(ppd, "ManualFeed", "True"))
+	    conflict = 1;
+      }
+    }
+    else if (!strcasecmp(optptr->name, "sides"))
+    {
+      for (j = 0; j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])); j ++)
+        if (cupsGetOption(duplex_options[j], num_options, options) != NULL)
+	  break;
+
+      if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
+      {
+       /*
+        * Don't override the PPD option with the IPP attribute...
+	*/
+
+        continue;
+      }
+
+      if (!strcasecmp(optptr->value, "one-sided"))
+      {
+       /*
+        * Mark the appropriate duplex option for one-sided output...
+	*/
+
+        for (j = 0; j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])); j ++)
+	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
+	    break;
+
+	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
+	{
+          for (k = 0; k < (int)(sizeof(duplex_one) / sizeof(duplex_one[0])); k ++)
+            if (ppdFindChoice(option, duplex_one[k]))
+	    {
+	      if (ppdMarkOption(ppd, duplex_options[j], duplex_one[k]))
+		conflict = 1;
+
+	      break;
+            }
+        }
+      }
+      else if (!strcasecmp(optptr->value, "two-sided-long-edge"))
+      {
+       /*
+        * Mark the appropriate duplex option for two-sided-long-edge output...
+	*/
+
+        for (j = 0; j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])); j ++)
+	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
+	    break;
+
+	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
+	{
+          for (k = 0; k < (int)(sizeof(duplex_two_long) / sizeof(duplex_two_long[0])); k ++)
+            if (ppdFindChoice(option, duplex_two_long[k]))
+	    {
+	      if (ppdMarkOption(ppd, duplex_options[j], duplex_two_long[k]))
+		conflict = 1;
+
+	      break;
+            }
+        }
+      }
+      else if (!strcasecmp(optptr->value, "two-sided-short-edge"))
+      {
+       /*
+        * Mark the appropriate duplex option for two-sided-short-edge output...
+	*/
+
+        for (j = 0; j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])); j ++)
+	  if ((option = ppdFindOption(ppd, duplex_options[j])) != NULL)
+	    break;
+
+	if (j < (int)(sizeof(duplex_options) / sizeof(duplex_options[0])))
+	{
+          for (k = 0; k < (int)(sizeof(duplex_two_short) / sizeof(duplex_two_short[0])); k ++)
+            if (ppdFindChoice(option, duplex_two_short[k]))
+	    {
+	      if (ppdMarkOption(ppd, duplex_options[j], duplex_two_short[k]))
+		conflict = 1;
+
+	      break;
+            }
+        }
+      }
+    }
+    else if (!strcasecmp(optptr->name, "resolution") ||
+             !strcasecmp(optptr->name, "printer-resolution"))
+    {
+      if (ppdMarkOption(ppd, "Resolution", optptr->value))
+        conflict = 1;
+      if (ppdMarkOption(ppd, "SetResolution", optptr->value))
+      	/* Calcomp, Linotype, QMS, Summagraphics, Tektronix, Varityper */
+        conflict = 1;
+      if (ppdMarkOption(ppd, "JCLResolution", optptr->value))	/* HP */
+        conflict = 1;
+      if (ppdMarkOption(ppd, "CNRes_PGP", optptr->value))	/* Canon */
+        conflict = 1;
+    }
+    else if (!strcasecmp(optptr->name, "output-bin"))
+    {
+      if (cupsGetOption("OutputBin", num_options, options) == NULL)
+        if (ppdMarkOption(ppd, "OutputBin", optptr->value))
+          conflict = 1;
+    }
+    else if (ppdMarkOption(ppd, optptr->name, optptr->value))
+      conflict = 1;
+
+  return (conflict);
+}
+
+
+/*
+ * 'cupsParseOptions()' - Parse options from a command-line argument.
+ *
+ * This function converts space-delimited name/value pairs according
+ * to the PAPI text option ABNF specification. Collection values
+ * ("name={a=... b=... c=...}") are stored with the curley brackets
+ * intact - use cupsParseOptions() on the value to extract the collection
+ * attributes.
+ */
+
+int					/* O - Number of options found */
+cupsParseOptions(
+    const char    *arg,			/* I - Argument to parse */
+    int           num_options,		/* I - Number of options */
+    cups_option_t **options)		/* O - Options found */
+{
+  char	*copyarg,			/* Copy of input string */
+	*ptr,				/* Pointer into string */
+	*name,				/* Pointer to name */
+	*value;				/* Pointer to value */
 
 
   if (arg == NULL || options == NULL || num_options < 0)
@@ -244,7 +477,7 @@ cupsParseOptions(const char    *arg,		/* I - Argument to parse */
       while (*ptr != '\'' && *ptr != '\0')
       {
         if (*ptr == '\\')
-	  cups_strcpy(ptr, ptr + 1);
+	  _cups_strcpy(ptr, ptr + 1);
 
         ptr ++;
       }
@@ -264,7 +497,7 @@ cupsParseOptions(const char    *arg,		/* I - Argument to parse */
       while (*ptr != '\"' && *ptr != '\0')
       {
         if (*ptr == '\\')
-	  cups_strcpy(ptr, ptr + 1);
+	  _cups_strcpy(ptr, ptr + 1);
 
         ptr ++;
       }
@@ -297,7 +530,7 @@ cupsParseOptions(const char    *arg,		/* I - Argument to parse */
 	  }
         }
         else if (*ptr == '\\')
-	  cups_strcpy(ptr, ptr + 1);
+	  _cups_strcpy(ptr, ptr + 1);
 
       if (*ptr != '\0')
         *ptr++ = '\0';
@@ -313,7 +546,7 @@ cupsParseOptions(const char    *arg,		/* I - Argument to parse */
       while (!isspace(*ptr & 255) && *ptr != '\0')
       {
         if (*ptr == '\\')
-	  cups_strcpy(ptr, ptr + 1);
+	  _cups_strcpy(ptr, ptr + 1);
 
         ptr ++;
       }
@@ -345,155 +578,61 @@ cupsParseOptions(const char    *arg,		/* I - Argument to parse */
 
 
 /*
- * 'cupsMarkOptions()' - Mark command-line options in a PPD file.
+ * 'cupsRemoveOptions()' - Remove an option from an option array.
+ *
+ * @since CUPS 1.2@
  */
 
-int						/* O - 1 if conflicting */
-cupsMarkOptions(ppd_file_t    *ppd,		/* I - PPD file */
-                int           num_options,	/* I - Number of options */
-                cups_option_t *options)		/* I - Options */
+int					/* O  - New number of options */
+cupsRemoveOption(
+    const char    *name,		/* I  - Option name */
+    int           num_options,		/* I  - Current number of options */
+    cups_option_t **options)		/* IO - Options */
 {
-  int		i;				/* Looping var */
-  int		conflict;			/* Option conflicts */
-  char		*val,				/* Pointer into value */
-		*ptr,				/* Pointer into string */
-		s[255];				/* Temporary string */
-  cups_option_t	*optptr;			/* Current option */
+  int		i;			/* Looping var */
+  cups_option_t	*option;		/* Current option */
 
 
  /*
-  * Check arguments...
+  * Range check input...
   */
 
-  if (ppd == NULL || num_options <= 0 || options == NULL)
-    return (0);
+  if (!name || num_options < 1 || !options)
+    return (num_options);
 
  /*
-  * Mark options...
+  * Loop for the option...
   */
 
-  conflict = 0;
+  for (i = num_options, option = *options; i > 0; i --, option ++)
+    if (!strcasecmp(name, option->name))
+      break;
 
-  for (i = num_options, optptr = options; i > 0; i --, optptr ++)
-    if (strcasecmp(optptr->name, "media") == 0)
-    {
-     /*
-      * Loop through the option string, separating it at commas and
-      * marking each individual option.
-      */
+  if (i)
+  {
+   /*
+    * Remove this option from the array...
+    */
 
-      for (val = optptr->value; *val;)
-      {
-       /*
-        * Extract the sub-option from the string...
-	*/
+    num_options --;
+    i --;
 
-        for (ptr = s; *val && *val != ',' && (ptr - s) < (sizeof(s) - 1);)
-	  *ptr++ = *val++;
-	*ptr++ = '\0';
+    free(option->name);
+    if (option->value)
+      free(option->value);
 
-	if (*val == ',')
-	  val ++;
+    if (i > 0)
+      memmove(option, option + 1, i * sizeof(cups_option_t *));
+  }
 
-       /*
-        * Mark it...
-	*/
+ /*
+  * Return the new number of options...
+  */
 
-        if (cupsGetOption("PageSize", num_options, options) == NULL)
-	  if (ppdMarkOption(ppd, "PageSize", s))
-            conflict = 1;
-
-        if (cupsGetOption("InputSlot", num_options, options) == NULL)
-	  if (ppdMarkOption(ppd, "InputSlot", s))
-            conflict = 1;
-
-        if (cupsGetOption("MediaType", num_options, options) == NULL)
-	  if (ppdMarkOption(ppd, "MediaType", s))
-            conflict = 1;
-
-        if (cupsGetOption("EFMediaQualityMode", num_options, options) == NULL)
-	  if (ppdMarkOption(ppd, "EFMediaQualityMode", s))	/* EFI */
-            conflict = 1;
-
-	if (strcasecmp(s, "manual") == 0 &&
-	    cupsGetOption("ManualFeed", num_options, options) == NULL)
-          if (ppdMarkOption(ppd, "ManualFeed", "True"))
-	    conflict = 1;
-      }
-    }
-    else if (strcasecmp(optptr->name, "sides") == 0)
-    {
-      if (cupsGetOption("Duplex", num_options, options) != NULL ||
-          cupsGetOption("JCLDuplex", num_options, options) != NULL ||
-          cupsGetOption("EFDuplex", num_options, options) != NULL ||
-          cupsGetOption("KD03Duplex", num_options, options) != NULL)
-      {
-       /*
-        * Don't override the PPD option with the IPP attribute...
-	*/
-
-        continue;
-      }
-      else if (strcasecmp(optptr->value, "one-sided") == 0)
-      {
-        if (ppdMarkOption(ppd, "Duplex", "None"))
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "JCLDuplex", "None"))	/* Samsung */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "EFDuplex", "None"))	/* EFI */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "KD03Duplex", "None"))	/* Kodak */
-	  conflict = 1;
-      }
-      else if (strcasecmp(optptr->value, "two-sided-long-edge") == 0)
-      {
-        if (ppdMarkOption(ppd, "Duplex", "DuplexNoTumble"))
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "JCLDuplex", "DuplexNoTumble"))	/* Samsung */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "EFDuplex", "DuplexNoTumble"))	/* EFI */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "KD03Duplex", "DuplexNoTumble"))	/* Kodak */
-	  conflict = 1;
-      }
-      else if (strcasecmp(optptr->value, "two-sided-short-edge") == 0)
-      {
-        if (ppdMarkOption(ppd, "Duplex", "DuplexTumble"))
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "JCLDuplex", "DuplexTumble"))	/* Samsung */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "EFDuplex", "DuplexTumble"))	/* EFI */
-	  conflict = 1;
-        if (ppdMarkOption(ppd, "KD03Duplex", "DuplexTumble"))	/* Kodak */
-	  conflict = 1;
-      }
-    }
-    else if (strcasecmp(optptr->name, "resolution") == 0 ||
-             strcasecmp(optptr->name, "printer-resolution") == 0)
-    {
-      if (ppdMarkOption(ppd, "Resolution", optptr->value))
-        conflict = 1;
-      if (ppdMarkOption(ppd, "SetResolution", optptr->value))
-      	/* Calcomp, Linotype, QMS, Summagraphics, Tektronix, Varityper */
-        conflict = 1;
-      if (ppdMarkOption(ppd, "JCLResolution", optptr->value))	/* HP */
-        conflict = 1;
-      if (ppdMarkOption(ppd, "CNRes_PGP", optptr->value))	/* Canon */
-        conflict = 1;
-    }
-    else if (strcasecmp(optptr->name, "output-bin") == 0)
-    {
-      if (cupsGetOption("OutputBin", num_options, options) == NULL)
-        if (ppdMarkOption(ppd, "OutputBin", optptr->value))
-          conflict = 1;
-    }
-    else if (ppdMarkOption(ppd, optptr->name, optptr->value))
-      conflict = 1;
-
-  return (conflict);
+  return (num_options);
 }
 
 
 /*
- * End of "$Id: options.c,v 1.35 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: options.c 5151 2006-02-22 22:43:17Z mike $".
  */
