@@ -1,5 +1,5 @@
 /*
- * "$Id: snprintf.c,v 1.14 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: snprintf.c 5529 2006-05-15 20:06:46Z mike $"
  *
  *   snprintf functions for the Common UNIX Printing System (CUPS).
  *
@@ -25,8 +25,8 @@
  *
  * Contents:
  *
- *   cups_vsnprintf() - Format a string into a fixed size buffer.
- *   cups_snprintf()  - Format a string into a fixed size buffer.
+ *   _cups_vsnprintf() - Format a string into a fixed size buffer.
+ *   _cups_snprintf()  - Format a string into a fixed size buffer.
  */
 
 /*
@@ -40,24 +40,24 @@
 
 #ifndef HAVE_VSNPRINTF
 /*
- * 'cups_vsnprintf()' - Format a string into a fixed size buffer.
+ * '_cups_vsnprintf()' - Format a string into a fixed size buffer.
  */
 
 int					/* O - Number of bytes formatted */
-cups_vsnprintf(char       *buffer,	/* O - Output buffer */
-               size_t     bufsize,	/* O - Size of output buffer */
-	       const char *format,	/* I - printf-style format string */
-	       va_list    ap)		/* I - Pointer to additional arguments */
+_cups_vsnprintf(char       *buffer,	/* O - Output buffer */
+                size_t     bufsize,	/* O - Size of output buffer */
+	        const char *format,	/* I - printf-style format string */
+	        va_list    ap)		/* I - Pointer to additional arguments */
 {
   char		*bufptr,		/* Pointer to position in buffer */
 		*bufend,		/* Pointer to end of buffer */
 		sign,			/* Sign of format width */
 		size,			/* Size character (h, l, L) */
 		type;			/* Format type character */
-  const char	*bufformat;		/* Start of format */
   int		width,			/* Width of field */
 		prec;			/* Number of characters of precision */
   char		tformat[100],		/* Temporary format string for sprintf() */
+		*tptr,			/* Pointer into temporary format */
 		temp[1024];		/* Buffer for formatted numbers */
   char		*s;			/* Pointer to string */
   int		slen;			/* Length of string */
@@ -76,30 +76,80 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
   {
     if (*format == '%')
     {
-      bufformat = format;
-      format ++;
+      tptr = tformat;
+      *tptr++ = *format++;
 
       if (*format == '%')
       {
-        *bufptr++ = *format++;
+        if (bufptr && bufptr < bufend) *bufptr++ = *format;
+        bytes ++;
+        format ++;
 	continue;
       }
       else if (strchr(" -+#\'", *format))
+      {
+        *tptr++ = *format;
         sign = *format++;
+      }
       else
         sign = 0;
 
-      width = 0;
-      while (isdigit(*format))
-        width = width * 10 + *format++ - '0';
+      if (*format == '*')
+      {
+       /*
+        * Get width from argument...
+	*/
+
+	format ++;
+	width = va_arg(ap, int);
+
+	snprintf(tptr, sizeof(tformat) - (tptr - tformat), "%d", width);
+	tptr += strlen(tptr);
+      }
+      else
+      {
+	width = 0;
+
+	while (isdigit(*format & 255))
+	{
+	  if (tptr < (tformat + sizeof(tformat) - 1))
+	    *tptr++ = *format;
+
+	  width = width * 10 + *format++ - '0';
+	}
+      }
 
       if (*format == '.')
       {
-        format ++;
-	prec = 0;
+	if (tptr < (tformat + sizeof(tformat) - 1))
+	  *tptr++ = *format;
 
-	while (isdigit(*format))
-          prec = prec * 10 + *format++ - '0';
+        format ++;
+
+        if (*format == '*')
+	{
+         /*
+	  * Get precision from argument...
+	  */
+
+	  format ++;
+	  prec = va_arg(ap, int);
+
+	  snprintf(tptr, sizeof(tformat) - (tptr - tformat), "%d", prec);
+	  tptr += strlen(tptr);
+	}
+	else
+	{
+	  prec = 0;
+
+	  while (isdigit(*format & 255))
+	  {
+	    if (tptr < (tformat + sizeof(tformat) - 1))
+	      *tptr++ = *format;
+
+	    prec = prec * 10 + *format++ - '0';
+	  }
+	}
       }
       else
         prec = -1;
@@ -107,15 +157,31 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
       if (*format == 'l' && format[1] == 'l')
       {
         size = 'L';
+
+	if (tptr < (tformat + sizeof(tformat) - 2))
+	{
+	  *tptr++ = 'l';
+	  *tptr++ = 'l';
+	}
+
 	format += 2;
       }
       else if (*format == 'h' || *format == 'l' || *format == 'L')
+      {
+	if (tptr < (tformat + sizeof(tformat) - 1))
+	  *tptr++ = *format;
+
         size = *format++;
+      }
 
       if (!*format)
         break;
 
-      type = *format++;
+      if (tptr < (tformat + sizeof(tformat) - 1))
+        *tptr++ = *format;
+
+      type  = *format++;
+      *tptr = '\0';
 
       switch (type)
       {
@@ -124,13 +190,9 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	case 'e' :
 	case 'f' :
 	case 'g' :
-	    if ((format - bufformat + 1) > sizeof(tformat) ||
-	        (width + 2) > sizeof(temp))
+	    if ((width + 2) > sizeof(temp))
 	      break;
 
-	    strncpy(tformat, bufformat, format - bufformat);
-	    tformat[format - bufformat] = '\0';
-
 	    sprintf(temp, tformat, va_arg(ap, double));
 
             bytes += strlen(temp);
@@ -139,9 +201,8 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    {
 	      if ((bufptr + strlen(temp)) > bufend)
 	      {
-		strncpy(bufptr, temp, bufend - bufptr);
+		strncpy(bufptr, temp, (size_t)(bufend - bufptr));
 		bufptr = bufend;
-		break;
 	      }
 	      else
 	      {
@@ -159,13 +220,9 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	case 'o' :
 	case 'u' :
 	case 'x' :
-	    if ((format - bufformat + 1) > sizeof(tformat) ||
-	        (width + 2) > sizeof(temp))
+	    if ((width + 2) > sizeof(temp))
 	      break;
 
-	    strncpy(tformat, bufformat, format - bufformat);
-	    tformat[format - bufformat] = '\0';
-
 	    sprintf(temp, tformat, va_arg(ap, int));
 
             bytes += strlen(temp);
@@ -174,9 +231,8 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    {
 	      if ((bufptr + strlen(temp)) > bufend)
 	      {
-		strncpy(bufptr, temp, bufend - bufptr);
+		strncpy(bufptr, temp, (size_t)(bufend - bufptr));
 		bufptr = bufend;
-		break;
 	      }
 	      else
 	      {
@@ -185,14 +241,10 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	      }
 	    }
 	    break;
-	    
-	case 'p' : /* Pointer value */
-	    if ((format - bufformat + 1) > sizeof(tformat) ||
-	        (width + 2) > sizeof(temp))
-	      break;
 
-	    strncpy(tformat, bufformat, format - bufformat);
-	    tformat[format - bufformat] = '\0';
+	case 'p' : /* Pointer value */
+	    if ((width + 2) > sizeof(temp))
+	      break;
 
 	    sprintf(temp, tformat, va_arg(ap, void *));
 
@@ -202,9 +254,8 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    {
 	      if ((bufptr + strlen(temp)) > bufend)
 	      {
-		strncpy(bufptr, temp, bufend - bufptr);
+		strncpy(bufptr, temp, (size_t)(bufend - bufptr));
 		bufptr = bufend;
-		break;
 	      }
 	      else
 	      {
@@ -220,13 +271,13 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    if (bufptr)
 	    {
 	      if (width <= 1)
-		*bufptr++ = va_arg(ap, int);
+	        *bufptr++ = va_arg(ap, int);
 	      else
 	      {
 		if ((bufptr + width) > bufend)
-	          width = bufend - bufptr;
+		  width = bufend - bufptr;
 
-		memcpy(bufptr, va_arg(ap, char *), width);
+		memcpy(bufptr, va_arg(ap, char *), (size_t)width);
 		bufptr += width;
 	      }
 	    }
@@ -245,20 +296,20 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    if (bufptr)
 	    {
 	      if ((bufptr + width) > bufend)
-		width = bufend - bufptr;
+	        width = bufend - bufptr;
 
               if (slen > width)
-		slen = width;
+	        slen = width;
 
 	      if (sign == '-')
 	      {
-		strncpy(bufptr, s, slen);
-		memset(bufptr + slen, ' ', width - slen);
+		strncpy(bufptr, s, (size_t)slen);
+		memset(bufptr + slen, ' ', (size_t)(width - slen));
 	      }
 	      else
 	      {
-		memset(bufptr, ' ', width - slen);
-		strncpy(bufptr + width - slen, s, slen);
+		memset(bufptr, ' ', (size_t)(width - slen));
+		strncpy(bufptr + width - slen, s, (size_t)slen);
 	      }
 
 	      bufptr += width;
@@ -266,31 +317,7 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 	    break;
 
 	case 'n' : /* Output number of chars so far */
-	    if ((format - bufformat + 1) > sizeof(tformat) ||
-	        (width + 2) > sizeof(temp))
-	      break;
-
-	    strncpy(tformat, bufformat, format - bufformat);
-	    tformat[format - bufformat] = '\0';
-
-	    sprintf(temp, tformat, va_arg(ap, int));
-
-            bytes += strlen(temp);
-
-	    if (bufptr)
-	    {
-	      if ((bufptr + strlen(temp)) > bufend)
-	      {
-		strncpy(bufptr, temp, bufend - bufptr);
-		bufptr = bufend;
-		break;
-	      }
-	      else
-	      {
-		strcpy(bufptr, temp);
-		bufptr += strlen(temp);
-	      }
-	    }
+	    *(va_arg(ap, int *)) = bytes;
 	    break;
       }
     }
@@ -299,7 +326,9 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
       bytes ++;
 
       if (bufptr && bufptr < bufend)
-	*bufptr++ = *format++;
+        *bufptr++ = *format;
+
+      format ++;
     }
   }
 
@@ -316,14 +345,14 @@ cups_vsnprintf(char       *buffer,	/* O - Output buffer */
 
 #ifndef HAVE_SNPRINTF
 /*
- * 'cups_snprintf()' - Format a string into a fixed size buffer.
+ * '_cups_snprintf()' - Format a string into a fixed size buffer.
  */
 
 int					/* O - Number of bytes formatted */
-cups_snprintf(char       *buffer,	/* O - Output buffer */
-              size_t     bufsize,	/* O - Size of output buffer */
-              const char *format,	/* I - printf-style format string */
-	      ...)			/* I - Additional arguments as needed */
+_cups_snprintf(char       *buffer,	/* O - Output buffer */
+               size_t     bufsize,	/* O - Size of output buffer */
+               const char *format,	/* I - printf-style format string */
+	       ...)			/* I - Additional arguments as needed */
 {
   int		bytes;			/* Number of bytes formatted */
   va_list 	ap;			/* Pointer to additional arguments */
@@ -339,6 +368,6 @@ cups_snprintf(char       *buffer,	/* O - Output buffer */
 
 
 /*
- * End of "$Id: snprintf.c,v 1.14 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: snprintf.c 5529 2006-05-15 20:06:46Z mike $".
  */
 
