@@ -1,9 +1,9 @@
 /*
- * "$Id: string.h,v 1.19 2005/01/03 19:29:45 mike Exp $"
+ * "$Id: string.h 5466 2006-04-26 19:52:27Z mike $"
  *
  *   String definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -31,11 +31,13 @@
  * Include necessary headers...
  */
 
-#  include <config.h>
-
 #  include <stdio.h>
+#  include <stdlib.h>
 #  include <stdarg.h>
 #  include <ctype.h>
+#  include <locale.h>
+
+#  include <config.h>
 
 #  ifdef HAVE_STRING_H
 #    include <string.h>
@@ -69,51 +71,81 @@ extern "C" {
 #  endif /* __cplusplus */
 
 
+/*
+ * String pool structures...
+ */
+
+typedef struct _cups_sp_item_s		/**** String Pool Item ****/
+{
+  char		*str;			/* String */
+  unsigned int	ref_count;		/* Reference count */
+} _cups_sp_item_t;
+
+
 /*
  * Prototypes...
  */
 
-extern void	cups_strcpy(char *dst, const char *src);
+extern void	_cups_strcpy(char *dst, const char *src);
 
 #  ifndef HAVE_STRDUP
-extern char	*cups_strdup(const char *);
-#    define strdup cups_strdup
+extern char	*_cups_strdup(const char *);
+#    define strdup _cups_strdup
 #  endif /* !HAVE_STRDUP */
 
 #  ifndef HAVE_STRCASECMP
-extern int	cups_strcasecmp(const char *, const char *);
-#    define strcasecmp cups_strcasecmp
+extern int	_cups_strcasecmp(const char *, const char *);
+#    define strcasecmp _cups_strcasecmp
 #  endif /* !HAVE_STRCASECMP */
 
 #  ifndef HAVE_STRNCASECMP
-extern int	cups_strncasecmp(const char *, const char *, size_t n);
-#    define strncasecmp cups_strncasecmp
+extern int	_cups_strncasecmp(const char *, const char *, size_t n);
+#    define strncasecmp _cups_strncasecmp
 #  endif /* !HAVE_STRNCASECMP */
 
 #  ifndef HAVE_STRLCAT
-extern size_t cups_strlcat(char *, const char *, size_t);
-#    define strlcat cups_strlcat
+extern size_t _cups_strlcat(char *, const char *, size_t);
+#    define strlcat _cups_strlcat
 #  endif /* !HAVE_STRLCAT */
 
 #  ifndef HAVE_STRLCPY
-extern size_t cups_strlcpy(char *, const char *, size_t);
-#    define strlcpy cups_strlcpy
+extern size_t _cups_strlcpy(char *, const char *, size_t);
+#    define strlcpy _cups_strlcpy
 #  endif /* !HAVE_STRLCPY */
 
 #  ifndef HAVE_SNPRINTF
-extern int	cups_snprintf(char *, size_t, const char *, ...)
+extern int	_cups_snprintf(char *, size_t, const char *, ...)
 #    ifdef __GNUC__
 __attribute__ ((__format__ (__printf__, 3, 4)))
 #    endif /* __GNUC__ */
 ;
-#    define snprintf cups_snprintf
+#    define snprintf _cups_snprintf
 #  endif /* !HAVE_SNPRINTF */
 
 #  ifndef HAVE_VSNPRINTF
-extern int	cups_vsnprintf(char *, size_t, const char *, va_list);
-#    define vsnprintf cups_vsnprintf
+extern int	_cups_vsnprintf(char *, size_t, const char *, va_list);
+#    define vsnprintf _cups_vsnprintf
 #  endif /* !HAVE_VSNPRINTF */
 
+/*
+ * String pool functions...
+ */
+
+extern char	*_cupsStrAlloc(const char *s);
+extern void	_cupsStrFlush(void);
+extern void	_cupsStrFree(const char *s);
+extern size_t	_cupsStrStatistics(size_t *alloc_bytes, size_t *total_bytes);
+
+
+/*
+ * Floating point number functions...
+ */
+
+extern char	*_cupsStrFormatd(char *buf, char *bufend, double number,
+		                 struct lconv *loc);
+extern double	_cupsStrScand(const char *buf, char **bufptr,
+		              struct lconv *loc);
+
 
 /*
  * C++ magic...
@@ -126,5 +158,5 @@ extern int	cups_vsnprintf(char *, size_t, const char *, va_list);
 #endif /* !_CUPS_STRING_H_ */
 
 /*
- * End of "$Id: string.h,v 1.19 2005/01/03 19:29:45 mike Exp $".
+ * End of "$Id: string.h 5466 2006-04-26 19:52:27Z mike $".
  */
