@@ -1,9 +1,9 @@
 /*
- * "$Id: imagetops.c,v 1.58 2005/01/03 19:29:57 mike Exp $"
+ * "$Id: imagetops.c 5379 2006-04-07 13:48:37Z mike $"
  *
  *   Image file to PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2005 by Easy Software Products.
+ *   Copyright 1993-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -54,57 +54,58 @@ int	Flip = 0,		/* Flip/mirror pages */
  * Local functions...
  */
 
-static void	ps_hex(ib_t *, int, int);
-static void	ps_ascii85(ib_t *, int, int);
+static void	ps_hex(cups_ib_t *, int, int);
+static void	ps_ascii85(cups_ib_t *, int, int);
 
 
 /*
  * 'main()' - Main entry...
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments */
-     char *argv[])	/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  image_t	*img;		/* Image to print */
-  float		xprint,		/* Printable area */
+  cups_image_t	*img;			/* Image to print */
+  float		xprint,			/* Printable area */
 		yprint,
-		xinches,	/* Total size in inches */
+		xinches,		/* Total size in inches */
 		yinches;
-  float		xsize,		/* Total size in points */
+  float		xsize,			/* Total size in points */
 		ysize,
 		xsize2,
 		ysize2;
-  float		aspect;		/* Aspect ratio */
-  int		xpages,		/* # x pages */
-		ypages,		/* # y pages */
-		xpage,		/* Current x page */
-		ypage,		/* Current y page */
-		page;		/* Current page number */
-  int		x0, y0,		/* Corners of the page in image coords */
-		x1, y1;
-  ib_t		*row;		/* Current row */
-  int		y;		/* Current Y coordinate in image */
-  int		colorspace;	/* Output colorspace */
-  int		out_offset,	/* Offset into output buffer */
-		out_length;	/* Length of output buffer */
-  ppd_file_t	*ppd;		/* PPD file */
-  ppd_choice_t	*choice;	/* PPD option choice */
-  int		num_options;	/* Number of print options */
-  cups_option_t	*options;	/* Print options */
-  const char	*val;		/* Option value */
-  int		slowcollate;	/* Collate copies the slow way */
-  float		g;		/* Gamma correction value */
-  float		b;		/* Brightness factor */
-  float		zoom;		/* Zoom facter */
-  int		xppi, yppi;	/* Pixels-per-inch */
-  int		hue, sat;	/* Hue and saturation adjustment */
-  int		realcopies;	/* Real copies being printed */
-  float		left, top;	/* Left and top of image */
-  char		filename[1024];	/* Name of file to print */
-  time_t	curtime;	/* Current time */
-  struct tm	*curtm;		/* Current date */
-  char		curdate[255];	/* Current date string */
+  float		aspect;			/* Aspect ratio */
+  int		xpages,			/* # x pages */
+		ypages,			/* # y pages */
+		xpage,			/* Current x page */
+		ypage,			/* Current y page */
+		page;			/* Current page number */
+  int		xc0, yc0,			/* Corners of the page in image coords */
+		xc1, yc1;
+  cups_ib_t	*row;			/* Current row */
+  int		y;			/* Current Y coordinate in image */
+  int		colorspace;		/* Output colorspace */
+  int		out_offset,		/* Offset into output buffer */
+		out_length;		/* Length of output buffer */
+  ppd_file_t	*ppd;			/* PPD file */
+  ppd_choice_t	*choice;		/* PPD option choice */
+  int		num_options;		/* Number of print options */
+  cups_option_t	*options;		/* Print options */
+  const char	*val;			/* Option value */
+  int		slowcollate;		/* Collate copies the slow way */
+  float		g;			/* Gamma correction value */
+  float		b;			/* Brightness factor */
+  float		zoom;			/* Zoom facter */
+  int		xppi, yppi;		/* Pixels-per-inch */
+  int		hue, sat;		/* Hue and saturation adjustment */
+  int		realcopies,		/* Real copies being printed */
+		emit_jcl;		/* Emit JCL? */
+  float		left, top;		/* Left and top of image */
+  char		filename[1024];		/* Name of file to print */
+  time_t	curtime;		/* Current time */
+  struct tm	*curtm;			/* Current date */
+  char		curdate[255];		/* Current date string */
 
 
  /*
@@ -192,13 +193,37 @@ main(int  argc,		/* I - Number of command-line arguments */
     Collate = 1;
 
   if ((val = cupsGetOption("gamma", num_options, options)) != NULL)
+  {
+   /*
+    * Get gamma value from 1 to 10000...
+    */
+
     g = atoi(val) * 0.001f;
 
+    if (g < 0.001f)
+      g = 0.001f;
+    else if (g > 10.0f)
+      g = 10.0f;
+  }
+
   if ((val = cupsGetOption("brightness", num_options, options)) != NULL)
+  {
+   /*
+    * Get brightness value from 10 to 1000.
+    */
+
     b = atoi(val) * 0.01f;
 
+    if (b < 0.1f)
+      b = 0.1f;
+    else if (b > 10.0f)
+      b = 10.0f;
+  }
+
   if ((val = cupsGetOption("scaling", num_options, options)) != NULL)
     zoom = atoi(val) * 0.01;
+  else if (cupsGetOption("fitplot", num_options, options))
+    zoom = 1.0;
 
   if ((val = cupsGetOption("ppi", num_options, options)) != NULL)
     if (sscanf(val, "%dx%d", &xppi, &yppi) < 2)
@@ -263,13 +288,20 @@ main(int  argc,		/* I - Number of command-line arguments */
       strcasecmp(val, "True") == 0)
     Flip = 1;
 
+  if ((val = cupsGetOption("emit-jcl", num_options, options)) != NULL &&
+      (!strcasecmp(val, "false") || !strcasecmp(val, "off") ||
+       !strcasecmp(val, "no") || !strcmp(val, "0")))
+    emit_jcl = 0;
+  else
+    emit_jcl = 1;
+
  /*
   * Open the input image to print...
   */
 
-  colorspace = ColorDevice ? IMAGE_RGB_CMYK : IMAGE_WHITE;
+  colorspace = ColorDevice ? CUPS_IMAGE_RGB_CMYK : CUPS_IMAGE_WHITE;
 
-  img = ImageOpen(filename, colorspace, IMAGE_WHITE, sat, hue, NULL);
+  img = cupsImageOpen(filename, colorspace, CUPS_IMAGE_WHITE, sat, hue, NULL);
 
   if (argc == 6)
     unlink(filename);
@@ -281,7 +313,7 @@ main(int  argc,		/* I - Number of command-line arguments */
     return (1);
   }
 
-  colorspace = img->colorspace;
+  colorspace = cupsImageGetColorSpace(img);
 
  /*
   * Scale as necessary...
@@ -289,8 +321,8 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   if (zoom == 0.0 && xppi == 0)
   {
-    xppi = img->xppi;
-    yppi = img->yppi;
+    xppi = cupsImageGetXPPI(img);
+    yppi = cupsImageGetYPPI(img);
   }
 
   if (yppi == 0)
@@ -319,8 +351,8 @@ main(int  argc,		/* I - Number of command-line arguments */
     fprintf(stderr, "DEBUG: Before scaling: xprint=%.1f, yprint=%.1f\n",
             xprint, yprint);
 
-    xinches = (float)img->xsize / (float)xppi;
-    yinches = (float)img->ysize / (float)yppi;
+    xinches = (float)cupsImageGetWidth(img) / (float)xppi;
+    yinches = (float)cupsImageGetHeight(img) / (float)yppi;
 
     fprintf(stderr, "DEBUG: Image size is %.1f x %.1f inches...\n",
             xinches, yinches);
@@ -364,30 +396,30 @@ main(int  argc,		/* I - Number of command-line arguments */
 
     xprint = (PageRight - PageLeft) / 72.0;
     yprint = (PageTop - PageBottom) / 72.0;
-    aspect = (float)img->yppi / (float)img->xppi;
+    aspect = (float)cupsImageGetYPPI(img) / (float)cupsImageGetXPPI(img);
 
     fprintf(stderr, "DEBUG: Before scaling: xprint=%.1f, yprint=%.1f\n",
             xprint, yprint);
 
-    fprintf(stderr, "DEBUG: img->xppi = %d, img->yppi = %d, aspect = %f\n",
-            img->xppi, img->yppi, aspect);
+    fprintf(stderr, "DEBUG: cupsImageGetXPPI(img) = %d, cupsImageGetYPPI(img) = %d, aspect = %f\n",
+            cupsImageGetXPPI(img), cupsImageGetYPPI(img), aspect);
 
     xsize = xprint * zoom;
-    ysize = xsize * img->ysize / img->xsize / aspect;
+    ysize = xsize * cupsImageGetHeight(img) / cupsImageGetWidth(img) / aspect;
 
     if (ysize > (yprint * zoom))
     {
       ysize = yprint * zoom;
-      xsize = ysize * img->xsize * aspect / img->ysize;
+      xsize = ysize * cupsImageGetWidth(img) * aspect / cupsImageGetHeight(img);
     }
 
     xsize2 = yprint * zoom;
-    ysize2 = xsize2 * img->ysize / img->xsize / aspect;
+    ysize2 = xsize2 * cupsImageGetHeight(img) / cupsImageGetWidth(img) / aspect;
 
     if (ysize2 > (xprint * zoom))
     {
       ysize2 = xprint * zoom;
-      xsize2 = ysize2 * img->xsize * aspect / img->ysize;
+      xsize2 = ysize2 * cupsImageGetWidth(img) * aspect / cupsImageGetHeight(img);
     }
 
     fprintf(stderr, "DEBUG: Portrait size is %.2f x %.2f inches\n", xsize, ysize);
@@ -557,7 +589,8 @@ main(int  argc,		/* I - Number of command-line arguments */
   * Write any JCL commands that are needed to print PostScript code...
   */
 
-  ppdEmitJCL(ppd, stdout, atoi(argv[1]), argv[2], argv[3]);
+  if (emit_jcl)
+    ppdEmitJCL(ppd, stdout, atoi(argv[1]), argv[2], argv[3]);
 
  /*
   * Start sending the document with any commands needed...
@@ -574,7 +607,7 @@ main(int  argc,		/* I - Number of command-line arguments */
   puts("%%DocumentData: Clean7Bit");
   puts("%%DocumentNeededResources: font Helvetica-Bold");
   puts("%%Creator: imagetops/" CUPS_SVERSION);
-  strftime(curdate, sizeof(curdate), CUPS_STRFTIME_FORMAT, curtm);
+  strftime(curdate, sizeof(curdate), "%c", curtm);
   printf("%%%%CreationDate: %s\n", curdate);
   printf("%%%%Title: %s\n", argv[3]);
   printf("%%%%For: %s\n", argv[2]);
@@ -636,7 +669,7 @@ main(int  argc,		/* I - Number of command-line arguments */
   * Output the pages...
   */
 
-  row = malloc(img->xsize * abs(colorspace) + 3);
+  row = malloc(cupsImageGetWidth(img) * abs(colorspace) + 3);
 
   fprintf(stderr, "DEBUG: XPosition=%d, YPosition=%d, Orientation=%d\n",
           XPosition, YPosition, Orientation);
@@ -648,7 +681,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   switch (Orientation)
   {
-    case 0 :
+    default :
 	switch (XPosition)
 	{
 	  case -1 :
@@ -796,45 +829,45 @@ main(int  argc,		/* I - Number of command-line arguments */
 
         puts("gsave");
 
-	x0 = img->xsize * xpage / xpages;
-	x1 = img->xsize * (xpage + 1) / xpages - 1;
-	y0 = img->ysize * ypage / ypages;
-	y1 = img->ysize * (ypage + 1) / ypages - 1;
+	xc0 = cupsImageGetWidth(img) * xpage / xpages;
+	xc1 = cupsImageGetWidth(img) * (xpage + 1) / xpages - 1;
+	yc0 = cupsImageGetHeight(img) * ypage / ypages;
+	yc1 = cupsImageGetHeight(img) * (ypage + 1) / ypages - 1;
 
         printf("%.1f %.1f translate\n", left, top);
 
 	printf("%.3f %.3f scale\n\n",
-	       xprint * 72.0 / (x1 - x0 + 1),
-	       yprint * 72.0 / (y1 - y0 + 1));
+	       xprint * 72.0 / (xc1 - xc0 + 1),
+	       yprint * 72.0 / (yc1 - yc0 + 1));
 
 	if (LanguageLevel == 1)
 	{
-	  printf("/picture %d string def\n", (x1 - x0 + 1) * abs(colorspace));
-	  printf("%d %d 8[1 0 0 -1 0 1]", (x1 - x0 + 1), (y1 - y0 + 1));
+	  printf("/picture %d string def\n", (xc1 - xc0 + 1) * abs(colorspace));
+	  printf("%d %d 8[1 0 0 -1 0 1]", (xc1 - xc0 + 1), (yc1 - yc0 + 1));
 
-          if (colorspace == IMAGE_WHITE)
+          if (colorspace == CUPS_IMAGE_WHITE)
             puts("{currentfile picture readhexstring pop} image");
           else
             printf("{currentfile picture readhexstring pop} false %d colorimage\n",
 	           abs(colorspace));
 
-          for (y = y0; y <= y1; y ++)
+          for (y = yc0; y <= yc1; y ++)
           {
-            ImageGetRow(img, x0, y, x1 - x0 + 1, row);
-            ps_hex(row, (x1 - x0 + 1) * abs(colorspace), y == y1);
+            cupsImageGetRow(img, xc0, y, xc1 - xc0 + 1, row);
+            ps_hex(row, (xc1 - xc0 + 1) * abs(colorspace), y == yc1);
           }
 	}
 	else
 	{
           switch (colorspace)
 	  {
-	    case IMAGE_WHITE :
+	    case CUPS_IMAGE_WHITE :
                 puts("/DeviceGray setcolorspace");
 		break;
-            case IMAGE_RGB :
+            case CUPS_IMAGE_RGB :
                 puts("/DeviceRGB setcolorspace");
 		break;
-            case IMAGE_CMYK :
+            case CUPS_IMAGE_CMYK :
                 puts("/DeviceCMYK setcolorspace");
 		break;
           }
@@ -844,36 +877,36 @@ main(int  argc,		/* I - Number of command-line arguments */
 		 "/Width %d"
 		 "/Height %d"
 		 "/BitsPerComponent 8",
-		 x1 - x0 + 1, y1 - y0 + 1);
+		 xc1 - xc0 + 1, yc1 - yc0 + 1);
 
           switch (colorspace)
 	  {
-	    case IMAGE_WHITE :
+	    case CUPS_IMAGE_WHITE :
                 fputs("/Decode[0 1]", stdout);
 		break;
-            case IMAGE_RGB :
+            case CUPS_IMAGE_RGB :
                 fputs("/Decode[0 1 0 1 0 1]", stdout);
 		break;
-            case IMAGE_CMYK :
+            case CUPS_IMAGE_CMYK :
                 fputs("/Decode[0 1 0 1 0 1 0 1]", stdout);
 		break;
           }
 
-          fputs("/DataSource currentfile /ASCII85Decode filter", stdout);
+          fputs("\n/DataSource currentfile/ASCII85Decode filter", stdout);
 
-          if (((x1 - x0 + 1) / xprint) < 100.0)
+          if (((xc1 - xc0 + 1) / xprint) < 100.0)
             fputs("/Interpolate true", stdout);
 
           puts("/ImageMatrix[1 0 0 -1 0 1]>>image");
 
-          for (y = y0, out_offset = 0; y <= y1; y ++)
+          for (y = yc0, out_offset = 0; y <= yc1; y ++)
           {
-            ImageGetRow(img, x0, y, x1 - x0 + 1, row + out_offset);
+            cupsImageGetRow(img, xc0, y, xc1 - xc0 + 1, row + out_offset);
 
-            out_length = (x1 - x0 + 1) * abs(colorspace) + out_offset;
+            out_length = (xc1 - xc0 + 1) * abs(colorspace) + out_offset;
             out_offset = out_length & 3;
 
-            ps_ascii85(row, out_length, y == y1);
+            ps_ascii85(row, out_length, y == yc1);
 
             if (out_offset > 0)
               memcpy(row, row + out_length - out_offset, out_offset);
@@ -892,16 +925,19 @@ main(int  argc,		/* I - Number of command-line arguments */
   * End the job with the appropriate JCL command or CTRL-D otherwise.
   */
 
-  if (ppd != NULL && ppd->jcl_end)
-    fputs(ppd->jcl_end, stdout);
-  else
-    putchar(0x04);
+  if (emit_jcl)
+  {
+    if (ppd && ppd->jcl_end)
+      ppdEmitJCLEnd(ppd, stdout);
+    else
+      putchar(0x04);
+  }
 
  /*
   * Close files...
   */
 
-  ImageClose(img);
+  cupsImageClose(img);
   ppdClose(ppd);
 
   return (0);
@@ -913,13 +949,13 @@ main(int  argc,		/* I - Number of command-line arguments */
  */
 
 static void
-ps_hex(ib_t *data,		/* I - Data to print */
-       int  length,		/* I - Number of bytes to print */
-       int  last_line)		/* I - Last line of raster data? */
+ps_hex(cups_ib_t *data,			/* I - Data to print */
+       int       length,		/* I - Number of bytes to print */
+       int       last_line)		/* I - Last line of raster data? */
 {
-  static int	col = 0;	/* Current column */
+  static int	col = 0;		/* Current column */
   static char	*hex = "0123456789ABCDEF";
-				/* Hex digits */
+					/* Hex digits */
 
 
   while (length > 0)
@@ -956,13 +992,13 @@ ps_hex(ib_t *data,		/* I - Data to print */
  */
 
 static void
-ps_ascii85(ib_t *data,		/* I - Data to print */
-	   int  length,		/* I - Number of bytes to print */
-	   int  last_line)	/* I - Last line of raster data? */
+ps_ascii85(cups_ib_t *data,		/* I - Data to print */
+	   int       length,		/* I - Number of bytes to print */
+	   int       last_line)		/* I - Last line of raster data? */
 {
-  unsigned	b;		/* Binary data word */
-  unsigned char	c[5];		/* ASCII85 encoded chars */
-  static int	col = 0;	/* Current column */
+  unsigned	b;			/* Binary data word */
+  unsigned char	c[5];			/* ASCII85 encoded chars */
+  static int	col = 0;		/* Current column */
 
 
   while (length > 3)
@@ -1027,5 +1063,5 @@ ps_ascii85(ib_t *data,		/* I - Data to print */
 
 
 /*
- * End of "$Id: imagetops.c,v 1.58 2005/01/03 19:29:57 mike Exp $".
+ * End of "$Id: imagetops.c 5379 2006-04-07 13:48:37Z mike $".
  */
