@@ -1,5 +1,5 @@
 /*
- * "$Id: texttops.c,v 1.52 2005/01/03 19:29:58 mike Exp $"
+ * "$Id: texttops.c 4680 2005-09-21 09:28:39Z mike $"
  *
  *   Text to PostScript filter for the Common UNIX Printing System (CUPS).
  *
@@ -201,7 +201,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 
   curtime = time(NULL);
   curtm   = localtime(&curtime);
-  strftime(curdate, sizeof(curdate), CUPS_STRFTIME_FORMAT, curtm);
+  strftime(curdate, sizeof(curdate), "%c", curtm);
 
   puts("%!PS-Adobe-3.0");
   printf("%%%%BoundingBox: 0 0 %.0f %.0f\n", PageWidth, PageLength);
@@ -1307,5 +1307,5 @@ write_text(const char *s)	/* I - String to write */
 
 
 /*
- * End of "$Id: texttops.c,v 1.52 2005/01/03 19:29:58 mike Exp $".
+ * End of "$Id: texttops.c 4680 2005-09-21 09:28:39Z mike $".
  */
