@@ -1,10 +1,10 @@
 //
-// "$Id: pdftops.cxx,v 1.20 2004/02/06 21:01:51 mike Exp $"
+// "$Id: pdftops.cxx 4906 2006-01-10 20:53:28Z mike $"
 //
 //   PDF to PostScript filter front-end for the Common UNIX Printing
 //   System (CUPS).
 //
-//   Copyright 1997-2003 by Easy Software Products.
+//   Copyright 1997-2006 by Easy Software Products.
 //
 //   These coded instructions, statements, and computer programs are the
 //   property of Easy Software Products and are protected by Federal
@@ -16,7 +16,7 @@
 //       Attn: CUPS Licensing Information
 //       Easy Software Products
 //       44141 Airport View Drive, Suite 204
-//       Hollywood, Maryland 20636-3111 USA
+//       Hollywood, Maryland 20636 USA
 //
 //       Voice: (301) 373-9600
 //       EMail: cups-info@cups.org
@@ -47,7 +47,7 @@
 #include "Page.h"
 #include "PDFDoc.h"
 #include "PSOutputDev.h"
-#include "Params.h"
+#include "GlobalParams.h"
 #include "Error.h"
 #include "config.h"
 
@@ -72,7 +72,7 @@ main(int  argc,				// I - Number of command-line args
   const char	*val;			// Option value
   ppd_file_t	*ppd;			// Current PPD
   ppd_size_t	*size;			// Current media size
-  FILE		*fp;			// Copy file
+  int		fd;			// Copy file descriptor
   const char	*server_root;		// Location of config files
   char		tempfile[1024];		// Temporary file
   char		buffer[8192];		// Copy buffer
@@ -98,7 +98,7 @@ main(int  argc,				// I - Number of command-line args
 
   // Copy stdin if needed...
   if (argc == 6) {
-    if ((fp = fopen(cupsTempFile(tempfile, sizeof(tempfile)), "w")) == NULL) {
+    if ((fd = cupsTempFd(tempfile, sizeof(tempfile))) < 0) {
       perror("ERROR: Unable to copy PDF file");
       return (1);
     }
@@ -107,8 +107,8 @@ main(int  argc,				// I - Number of command-line args
             tempfile);
 
     while ((bytes = fread(buffer, 1, sizeof(buffer), stdin)) > 0)
-      fwrite(buffer, 1, bytes, fp);
-    fclose(fp);
+      write(fd, buffer, bytes);
+    close(fd);
 
     fileName = new GString(tempfile);
   } else {
@@ -265,8 +265,6 @@ main(int  argc,				// I - Number of command-line args
            ppdIsMarked(ppd, "KD03Duplex", "DuplexTumble"))
     duplex = 1;
 
-  cupsFreeOptions(num_options, options);
-
   if (ppd != NULL)
     ppdClose(ppd);
 
@@ -285,7 +283,8 @@ main(int  argc,				// I - Number of command-line args
   globalParams->setPSPaperHeight(length);
   globalParams->setPSImageableArea(left, bottom, right, top);
   globalParams->setPSDuplex(duplex);
-  globalParams->setPSFit(fit);
+  globalParams->setPSExpandSmaller(fit);
+  globalParams->setPSShrinkLarger(fit);
   globalParams->setPSLevel(level);
   globalParams->setPSASCIIHex(level == psLevel1);
   globalParams->setPSEmbedType1(1);
@@ -309,9 +308,11 @@ main(int  argc,				// I - Number of command-line args
     // write PostScript file
     psOut = new PSOutputDev(psFileName->getCString(), doc->getXRef(),
                             doc->getCatalog(), 1, doc->getNumPages(),
-			    psModePS);
+			    psModePS, 0, 0, 0, 0, gFalse,
+			    cupsGetOption("page-ranges", num_options, options));
     if (psOut->isOk())
-      doc->displayPages(psOut, 1, doc->getNumPages(), 72, 72, 0, gFalse);
+      doc->displayPages(psOut, 1, doc->getNumPages(), 72.0, 72.0, 0,
+                        gTrue, gFalse, gFalse);
     delete psOut;
 
     // clean up
@@ -322,6 +323,8 @@ main(int  argc,				// I - Number of command-line args
     error(-1, "Unable to print this document.");
   }
 
+  cupsFreeOptions(num_options, options);
+
   delete doc;
   delete globalParams;
 
@@ -338,5 +341,5 @@ main(int  argc,				// I - Number of command-line args
 
 
 //
-// End of "$Id: pdftops.cxx,v 1.20 2004/02/06 21:01:51 mike Exp $".
+// End of "$Id: pdftops.cxx 4906 2006-01-10 20:53:28Z mike $".
 //
