@@ -1,9 +1,9 @@
 /*
- * "$Id: log.c,v 1.42 2004/08/23 18:36:50 mike Exp $"
+ * "$Id: log.c 5493 2006-05-05 16:33:57Z mike $"
  *
  *   Log file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -23,11 +23,11 @@
  *
  * Contents:
  *
- *   GetDateTime()    - Returns a pointer to a date/time string.
- *   LogMessage()     - Log a message to the error log file.
- *   LogPage()        - Log a page to the page log file.
- *   LogRequest()     - Log an HTTP request in Common Log Format.
- *   check_log_file() - Open/rotate a log file if it needs it.
+ *   cupsdGetDateTime() - Returns a pointer to a date/time string.
+ *   cupsdLogMessage()  - Log a message to the error log file.
+ *   cupsdLogPage()     - Log a page to the page log file.
+ *   cupsdLogRequest()  - Log an HTTP request in Common Log Format.
+ *   check_log_file()   - Open/rotate a log file if it needs it.
  */
 
 /*
@@ -36,10 +36,7 @@
 
 #include "cupsd.h"
 #include <stdarg.h>
-
-#ifdef HAVE_VSYSLOG
-#  include <syslog.h>
-#endif /* HAVE_VSYSLOG */
+#include <syslog.h>
 
 
 /*
@@ -50,17 +47,17 @@ static int	check_log_file(cups_file_t **, const char *);
 
 
 /*
- * 'GetDateTime()' - Returns a pointer to a date/time string.
+ * 'cupsdGetDateTime()' - Returns a pointer to a date/time string.
  */
 
-char *				/* O - Date/time string */
-GetDateTime(time_t t)		/* I - Time value */
+char *					/* O - Date/time string */
+cupsdGetDateTime(time_t t)		/* I - Time value */
 {
-  struct tm	*date;		/* Date/time value */
-  static time_t	last_time = -1;	/* Last time value */
-  static char	s[1024];	/* Date/time string */
-  static const char * const months[12] =
-		{		/* Months */
+  struct tm	*date;			/* Date/time value */
+  static time_t	last_time = -1;		/* Last time value */
+  static char	s[1024];		/* Date/time string */
+  static const char * const months[12] =/* Months */
+		{
 		  "Jan",
 		  "Feb",
 		  "Mar",
@@ -111,17 +108,17 @@ GetDateTime(time_t t)		/* I - Time value */
 
 
 /*
- * 'LogMessage()' - Log a message to the error log file.
+ * 'cupsdLogMessage()' - Log a message to the error log file.
  */
 
 int					/* O - 1 on success, 0 on error */
-LogMessage(int        level,		/* I - Log level */
-           const char *message,		/* I - printf-style message string */
-	   ...)				/* I - Additional args as needed */
+cupsdLogMessage(int        level,	/* I - Log level */
+                const char *message,	/* I - printf-style message string */
+	        ...)			/* I - Additional args as needed */
 {
-  int		len;			/* Length of message */
-  va_list	ap;			/* Argument pointer */
-  static const char levels[] =		/* Log levels... */
+  int			len;		/* Length of message */
+  va_list		ap;		/* Argument pointer */
+  static const char	levels[] =	/* Log levels... */
 		{
 		  ' ',
 		  'X',
@@ -157,7 +154,7 @@ LogMessage(int        level,		/* I - Log level */
   * See if we want to log this message...
   */
 
-  if (level > LogLevel)
+  if (level > LogLevel || !ErrorLog)
     return (1);
 
 #ifdef HAVE_VSYSLOG
@@ -165,7 +162,7 @@ LogMessage(int        level,		/* I - Log level */
   * See if we are logging errors via syslog...
   */
 
-  if (strcmp(ErrorLog, "syslog") == 0)
+  if (!strcmp(ErrorLog, "syslog"))
   {
     va_start(ap, message);
     vsyslog(syslevels[level], message, ap);
@@ -186,7 +183,7 @@ LogMessage(int        level,		/* I - Log level */
   * Print the log level and date/time...
   */
 
-  cupsFilePrintf(ErrorFile, "%c %s ", levels[level], GetDateTime(time(NULL)));
+  cupsFilePrintf(ErrorFile, "%c %s ", levels[level], cupsdGetDateTime(time(NULL)));
 
  /*
   * Allocate the line buffer as needed...
@@ -275,15 +272,15 @@ LogMessage(int        level,		/* I - Log level */
 
 
 /*
- * 'LogPage()' - Log a page to the page log file.
+ * 'cupsdLogPage()' - Log a page to the page log file.
  */
 
-int				/* O - 1 on success, 0 on error */
-LogPage(job_t       *job,	/* I - Job being printed */
-        const char  *page)	/* I - Page being printed */
+int					/* O - 1 on success, 0 on error */
+cupsdLogPage(cupsd_job_t *job,		/* I - Job being printed */
+             const char  *page)		/* I - Page being printed */
 {
-  ipp_attribute_t *billing,	/* job-billing attribute */
-                  *hostname;	/* job-originating-host-name attribute */
+  ipp_attribute_t *billing,		/* job-billing attribute */
+                  *hostname;		/* job-originating-host-name attribute */
 
 
   billing  = ippFindAttribute(job->attrs, "job-billing", IPP_TAG_ZERO);
@@ -295,7 +292,7 @@ LogPage(job_t       *job,	/* I - Job being printed */
   * See if we are logging pages via syslog...
   */
 
-  if (strcmp(PageLog, "syslog") == 0)
+  if (!strcmp(PageLog, "syslog"))
   {
     syslog(LOG_INFO, "PAGE %s %s %d %s %s %s", job->printer->name,
            job->username ? job->username : "-",
@@ -322,7 +319,7 @@ LogPage(job_t       *job,	/* I - Job being printed */
 
   cupsFilePrintf(PageFile, "%s %s %d %s %s %s %s\n", job->printer->name,
         	 job->username ? job->username : "-",
-        	 job->id, GetDateTime(time(NULL)), page,
+        	 job->id, cupsdGetDateTime(time(NULL)), page,
 		 billing ? billing->values[0].string.text : "-",
         	 hostname->values[0].string.text);
   cupsFileFlush(PageFile);
@@ -332,15 +329,15 @@ LogPage(job_t       *job,	/* I - Job being printed */
 
 
 /*
- * 'LogRequest()' - Log an HTTP request in Common Log Format.
+ * 'cupsdLogRequest()' - Log an HTTP request in Common Log Format.
  */
 
-int				/* O - 1 on success, 0 on error */
-LogRequest(client_t      *con,	/* I - Request to log */
-           http_status_t code)	/* I - Response code */
+int					/* O - 1 on success, 0 on error */
+cupsdLogRequest(cupsd_client_t *con,	/* I - Request to log */
+                http_status_t  code)	/* I - Response code */
 {
-  static const char * const states[] =
-		{		/* HTTP client states... */
+  static const char * const states[] =	/* HTTP client states... */
+		{
 		  "WAITING",
 		  "OPTIONS",
 		  "GET",
@@ -363,13 +360,13 @@ LogRequest(client_t      *con,	/* I - Request to log */
   * See if we are logging accesses via syslog...
   */
 
-  if (strcmp(AccessLog, "syslog") == 0)
+  if (!strcmp(AccessLog, "syslog"))
   {
-    syslog(LOG_INFO, "REQUEST %s - %s \"%s %s HTTP/%d.%d\" %d %d\n",
+    syslog(LOG_INFO, "REQUEST %s - %s \"%s %s HTTP/%d.%d\" %d " CUPS_LLFMT "\n",
            con->http.hostname, con->username[0] != '\0' ? con->username : "-",
 	   states[con->operation], con->uri,
 	   con->http.version / 100, con->http.version % 100,
-	   code, con->bytes);
+	   code, CUPS_LLCAST con->bytes);
 
     return (1);
   }
@@ -386,11 +383,18 @@ LogRequest(client_t      *con,	/* I - Request to log */
   * Write a log of the request in "common log format"...
   */
 
-  cupsFilePrintf(AccessFile, "%s - %s %s \"%s %s HTTP/%d.%d\" %d %d\n",
+  cupsFilePrintf(AccessFile,
+                 "%s - %s %s \"%s %s HTTP/%d.%d\" %d " CUPS_LLFMT " %s %s\n",
         	 con->http.hostname, con->username[0] != '\0' ? con->username : "-",
-		 GetDateTime(con->start), states[con->operation], con->uri,
+		 cupsdGetDateTime(con->start), states[con->operation], con->uri,
 		 con->http.version / 100, con->http.version % 100,
-		 code, con->bytes);
+		 code, CUPS_LLCAST con->bytes,
+		 con->request ?
+		     ippOpString(con->request->request.op.operation_id) : "-",
+		 con->response ?
+		     ippErrorString(con->response->request.status.status_code) :
+		     "-");
+
   cupsFileFlush(AccessFile);
 
   return (1);
@@ -402,7 +406,7 @@ LogRequest(client_t      *con,	/* I - Request to log */
  */
 
 static int				/* O  - 1 if log file open */
-check_log_file(cups_file_t **log,	/* IO - Log file */
+check_log_file(cups_file_t **lf,	/* IO - Log file */
 	       const char  *logname)	/* I  - Log filename */
 {
   char	backname[1024],			/* Backup log filename */
@@ -411,18 +415,19 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
 
 
  /*
-  * See if we have a log file to check or we are handling a signal...
+  * See if we have a log file to check...
   */
 
-  if (log == NULL || logname == NULL || !logname[0] || SignalCount)
+  if (!lf || !logname || !logname[0])
     return (1);
 
  /*
   * Format the filename as needed...
   */
 
-  if (*log == NULL ||
-      (cupsFileTell(*log) > MaxLogSize && MaxLogSize > 0))
+  if (!*lf ||
+      (strncmp(logname, "/dev/", 5) && cupsFileTell(*lf) > MaxLogSize &&
+       MaxLogSize > 0))
   {
    /*
     * Handle format strings...
@@ -476,19 +481,28 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
   * See if the log file is open...
   */
 
-  if (*log == NULL)
+  if (!*lf)
   {
    /*
     * Nope, open the log file...
     */
 
-    if ((*log = cupsFileOpen(filename, "a")) == NULL)
+    if ((*lf = cupsFileOpen(filename, "a")) == NULL)
+    {
+      syslog(LOG_ERR, "Unable to open log file \"%s\" - %s", filename,
+             strerror(errno));
+
       return (0);
+    }
 
     if (strncmp(filename, "/dev/", 5))
     {
-      fchown(cupsFileNumber(*log), RunUser, Group);
-      fchmod(cupsFileNumber(*log), LogFilePerm);
+     /*
+      * Change ownership and permissions of non-device logs...
+      */
+
+      fchown(cupsFileNumber(*lf), RunUser, Group);
+      fchmod(cupsFileNumber(*lf), LogFilePerm);
     }
   }
 
@@ -496,13 +510,14 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
   * Do we need to rotate the log?
   */
 
-  if (cupsFileTell(*log) > MaxLogSize && MaxLogSize > 0)
+  if (strncmp(logname, "/dev/", 5) && cupsFileTell(*lf) > MaxLogSize &&
+      MaxLogSize > 0)
   {
    /*
     * Rotate log file...
     */
 
-    cupsFileClose(*log);
+    cupsFileClose(*lf);
 
     strcpy(backname, filename);
     strlcat(backname, ".O", sizeof(backname));
@@ -510,14 +525,20 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
     unlink(backname);
     rename(filename, backname);
 
-    if ((*log = cupsFileOpen(filename, "a")) == NULL)
-      return (0);
-
-    if (strncmp(filename, "/dev/", 5))
+    if ((*lf = cupsFileOpen(filename, "a")) == NULL)
     {
-      fchown(cupsFileNumber(*log), RunUser, Group);
-      fchmod(cupsFileNumber(*log), LogFilePerm);
+      syslog(LOG_ERR, "Unable to open log file \"%s\" - %s", filename,
+             strerror(errno));
+
+      return (0);
     }
+
+   /*
+    * Change ownership and permissions of non-device logs...
+    */
+
+    fchown(cupsFileNumber(*lf), RunUser, Group);
+    fchmod(cupsFileNumber(*lf), LogFilePerm);
   }
 
   return (1);
@@ -525,5 +546,5 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
 
 
 /*
- * End of "$Id: log.c,v 1.42 2004/08/23 18:36:50 mike Exp $".
+ * End of "$Id: log.c 5493 2006-05-05 16:33:57Z mike $".
  */
