@@ -1,9 +1,9 @@
 /*
- * "$Id: server.c,v 1.19 2005/01/03 19:29:59 mike Exp $"
+ * "$Id: server.c 5493 2006-05-05 16:33:57Z mike $"
  *
  *   Server start/stop routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -23,8 +23,8 @@
  *
  * Contents:
  *
- *   StartServer() - Start the server.
- *   StopServer()  - Stop the server.
+ *   cupsdStartServer() - Start the server.
+ *   cupsdStopServer()  - Stop the server.
  */
 
 /*
@@ -33,21 +33,30 @@
 
 #include <cups/http-private.h>
 #include "cupsd.h"
-
 #include <grp.h>
+#ifdef HAVE_NOTIFY_H
+#  include <notify.h>
+#endif /* HAVE_NOTIFY_H */
 
 
 /*
- * 'StartServer()' - Start the server.
+ * Local globals...
+ */
+
+static int	started = 0;
+
+
+/*
+ * 'cupsdStartServer()' - Start the server.
  */
 
 void
-StartServer(void)
+cupsdStartServer(void)
 {
 #ifdef HAVE_LIBSSL
-  int		i;		/* Looping var */
-  struct timeval curtime;	/* Current time in microseconds */
-  unsigned char	data[1024];	/* Seed data */
+  int			i;		/* Looping var */
+  struct timeval	curtime;	/* Current time in microseconds */
+  unsigned char		data[1024];	/* Seed data */
 #endif /* HAVE_LIBSSL */
 
 
@@ -77,78 +86,89 @@ StartServer(void)
   */
 
   gnutls_global_init();
-#elif defined(HAVE_CDSASSL)
-  ServerCertificatesArray = CDSAGetServerCerts();
 #endif /* HAVE_LIBSSL */
 
  /*
   * Startup all the networking stuff...
   */
 
-  StartListening();
-  StartBrowsing();
-  StartPolling();
+  cupsdStartListening();
+  cupsdStartBrowsing();
+  cupsdStartPolling();
 
  /*
   * Create a pipe for CGI processes...
   */
 
   if (cupsdOpenPipe(CGIPipes))
-    LogMessage(L_ERROR, "StartServer: Unable to create pipes for CGI status!");
+    cupsdLogMessage(CUPSD_LOG_ERROR,
+                    "cupsdStartServer: Unable to create pipes for CGI status!");
   else
   {
-    LogMessage(L_DEBUG2, "StartServer: Adding fd %d to InputSet...", CGIPipes[0]);
+    CGIStatusBuffer = cupsdStatBufNew(CGIPipes[0], "[CGI]");
+
+    cupsdLogMessage(CUPSD_LOG_DEBUG2,
+                    "cupsdStartServer: Adding fd %d to InputSet...",
+		    CGIPipes[0]);
     FD_SET(CGIPipes[0], InputSet);
   }
+
+ /*
+  * Mark that the server has started and printers and jobs may be changed...
+  */
+
+  LastEvent     = CUPSD_EVENT_PRINTER_CHANGED | CUPSD_EVENT_JOB_STATE_CHANGED |
+                  CUPSD_EVENT_SERVER_STARTED;
+
+  started = 1;
 }
 
 
 /*
- * 'StopServer()' - Stop the server.
+ * 'cupsdStopServer()' - Stop the server.
  */
 
 void
-StopServer(void)
+cupsdStopServer(void)
 {
+  if (!started)
+    return;
+
  /*
   * Close all network clients and stop all jobs...
   */
 
-  CloseAllClients();
-  StopListening();
-  StopPolling();
-  StopBrowsing();
+  cupsdCloseAllClients();
+  cupsdStopListening();
+  cupsdStopPolling();
+  cupsdStopBrowsing();
+  cupsdStopAllNotifiers();
+  cupsdSaveRemoteCache();
+  cupsdDeleteAllCerts();
 
-  if (Clients != NULL)
+  if (Clients)
   {
-    free(Clients);
+    cupsArrayDelete(Clients);
     Clients = NULL;
   }
 
-#if defined(HAVE_SSL) && defined(HAVE_CDSASSL)
- /*
-  * Free all of the certificates...
-  */
-
-  if (ServerCertificatesArray)
-  {
-    CFRelease(ServerCertificatesArray);
-    ServerCertificatesArray = NULL;
-  }
-#endif /* HAVE_SSL && HAVE_CDSASSL */
-
  /*
   * Close the pipe for CGI processes...
   */
 
   if (CGIPipes[0] >= 0)
   {
-    LogMessage(L_DEBUG2, "StopServer: Removing fd %d from InputSet...",
-               CGIPipes[0]);
+    cupsdLogMessage(CUPSD_LOG_DEBUG2,
+                    "cupsdStopServer: Removing fd %d from InputSet...",
+                    CGIPipes[0]);
 
     FD_CLR(CGIPipes[0], InputSet);
 
-    cupsdClosePipe(CGIPipes);
+    cupsdStatBufDelete(CGIStatusBuffer);
+    close(CGIPipes[1]);
+
+    CGIPipes[0] = -1;
+    CGIPipes[1] = -1;
   }
 
  /*
@@ -175,9 +195,21 @@ StopServer(void)
 
     PageFile = NULL;
   }
+
+#ifdef HAVE_NOTIFY_POST
+ /*
+  * Send one last notification as the server shuts down.
+  */
+
+  cupsdLogMessage(CUPSD_LOG_DEBUG,
+                  "notify_post(\"com.apple.printerListChange\") last");
+  notify_post("com.apple.printerListChange");
+#endif /* HAVE_NOTIFY_POST */
+
+  started = 0;
 }
 
 
 /*
- * End of "$Id: server.c,v 1.19 2005/01/03 19:29:59 mike Exp $".
+ * End of "$Id: server.c 5493 2006-05-05 16:33:57Z mike $".
  */
