@@ -1,9 +1,9 @@
 /*
- * "$Id: lppasswd.c,v 1.18 2005/01/03 19:29:59 mike Exp $"
+ * "$Id: lppasswd.c 5926 2006-09-05 20:45:47Z mike $"
  *
  *   MD5 password program for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2005 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -40,9 +40,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <cups/cups.h>
-#include <cups/md5.h>
 #include <cups/string.h>
+#include <cups/cups.h>
+#include <cups/i18n.h>
+#include <cups/md5.h>
 
 #ifndef WIN32
 #  include <unistd.h>
@@ -70,37 +71,39 @@ static void	usage(FILE *fp);
  * 'main()' - Add, change, or delete passwords from the MD5 password file.
  */
 
-int				/* O - Exit status */
-main(int  argc,			/* I - Number of command-line arguments */
-     char *argv[])		/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  int		i;		/* Looping var */
-  char		*opt;		/* Option pointer */
-  const char	*username;	/* Pointer to username */
-  const char	*groupname;	/* Pointer to group name */
-  int		op;		/* Operation (add, change, delete) */
-  const char	*passwd;	/* Password string */
-  FILE		*infile,	/* Input file */
-		*outfile;	/* Output file */
-  char		line[256],	/* Line from file */
-		userline[17],	/* User from line */
-		groupline[17],	/* Group from line */
-		md5line[33],	/* MD5-sum from line */
-		md5new[33];	/* New MD5 sum */
-  const char	*root;		/* CUPS server root directory */
-  char		passwdmd5[1024],/* passwd.md5 file */
-		passwdold[1024],/* passwd.old file */
-		passwdnew[1024];/* passwd.tmp file */
-  char		*newpass,	/* new password */
-  		*oldpass;	/* old password */
-  int		flag;		/* Password check flags... */
-  int		fd;		/* Password file descriptor */
-  int		error;		/* Write error */
+  int		i;			/* Looping var */
+  char		*opt;			/* Option pointer */
+  const char	*username;		/* Pointer to username */
+  const char	*groupname;		/* Pointer to group name */
+  int		op;			/* Operation (add, change, delete) */
+  const char	*passwd;		/* Password string */
+  FILE		*infile,		/* Input file */
+		*outfile;		/* Output file */
+  char		line[256],		/* Line from file */
+		userline[17],		/* User from line */
+		groupline[17],		/* Group from line */
+		md5line[33],		/* MD5-sum from line */
+		md5new[33];		/* New MD5 sum */
+  const char	*root;			/* CUPS server root directory */
+  char		passwdmd5[1024],	/* passwd.md5 file */
+		passwdold[1024],	/* passwd.old file */
+		passwdnew[1024];	/* passwd.tmp file */
+  char		*newpass,		/* new password */
+  		*oldpass;		/* old password */
+  int		flag;			/* Password check flags... */
+  int		fd;			/* Password file descriptor */
+  int		error;			/* Write error */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;	/* Signal action */
+  struct sigaction action;		/* Signal action */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET*/
 
 
+  _cupsSetLocale(argv);
+
  /*
   * Check to see if stdin, stdout, and stderr are still open...
   */
@@ -120,33 +123,24 @@ main(int  argc,			/* I - Number of command-line arguments */
  /*
   * Find the server directory...
   *
-  * Don't use CUPS_SERVERROOT unless we're run by the
-  * super user.
+  * We use the CUPS_SERVERROOT environment variable when we are running
+  * as root or when lppasswd is not setuid...
   */
 
-  if (!getuid() && (root = getenv("CUPS_SERVERROOT")) != NULL)
-  {
-    snprintf(passwdmd5, sizeof(passwdmd5), "%s/passwd.md5", root);
-    snprintf(passwdold, sizeof(passwdold), "%s/passwd.old", root);
-    snprintf(passwdnew, sizeof(passwdnew), "%s/passwd.new", root);
-  }
-  else
-  {
-    strcpy(passwdmd5, CUPS_SERVERROOT "/passwd.md5");
-    strcpy(passwdold, CUPS_SERVERROOT "/passwd.old");
-    strcpy(passwdnew, CUPS_SERVERROOT "/passwd.new");
-  }
+  if ((root = getenv("CUPS_SERVERROOT")) == NULL ||
+      (getuid() != geteuid() && getuid()))
+    root = CUPS_SERVERROOT;
+
+  snprintf(passwdmd5, sizeof(passwdmd5), "%s/passwd.md5", root);
+  snprintf(passwdold, sizeof(passwdold), "%s/passwd.old", root);
+  snprintf(passwdnew, sizeof(passwdnew), "%s/passwd.new", root);
 
  /*
-  * Find the default system group: "sys", "system", or "root"...
+  * Find the default system group...
   */
 
-  if (getgrnam("sys"))
-    groupname = "sys";
-  else if (getgrnam("system"))
-    groupname = "system";
-  else if (getgrnam("root"))
-    groupname = "root";
+  if (getgrnam(CUPS_DEFAULT_GROUP))
+    groupname = CUPS_DEFAULT_GROUP;
   else
     groupname = "unknown";
 
@@ -194,9 +188,10 @@ main(int  argc,			/* I - Number of command-line arguments */
   * as root...
   */
 
-  if (getuid() && (op != CHANGE || username))
+  if (getuid() && getuid() != geteuid() && (op != CHANGE || username))
   {
-    fputs("lppasswd: Only root can add or delete passwords!\n", stderr);
+    _cupsLangPuts(stderr,
+                  _("lppasswd: Only root can add or delete passwords!\n"));
     return (1);
   }
 
@@ -216,12 +211,14 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   if (op == CHANGE && getuid())
   {
-    if ((passwd = cupsGetPassword("Enter old password:")) == NULL)
+    if ((passwd = cupsGetPassword(_("Enter old password:"))) == NULL)
       return (1);
 
     if ((oldpass = strdup(passwd)) == NULL)
     {
-      perror("lppasswd: Unable to copy password string");
+      _cupsLangPrintf(stderr,
+                      _("lppasswd: Unable to copy password string: %s\n"),
+		      strerror(errno));
       return (1);
     }
   }
@@ -232,21 +229,24 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   if (op != DELETE)
   {
-    if ((passwd = cupsGetPassword("Enter password:")) == NULL)
+    if ((passwd = cupsGetPassword(_("Enter password:"))) == NULL)
       return (1);
 
     if ((newpass = strdup(passwd)) == NULL)
     {
-      perror("lppasswd: Unable to copy password string!");
+      _cupsLangPrintf(stderr,
+                      _("lppasswd: Unable to copy password string: %s\n"),
+		      strerror(errno));
       return (1);
     }
 
-    if ((passwd = cupsGetPassword("Enter password again:")) == NULL)
+    if ((passwd = cupsGetPassword(_("Enter password again:"))) == NULL)
       return (1);
 
     if (strcmp(passwd, newpass) != 0)
     {
-      fputs("lppasswd: Sorry, passwords don't match!\n", stderr);
+      _cupsLangPuts(stderr,
+                    _("lppasswd: Sorry, passwords don't match!\n"));
       return (1);
     }
 
@@ -269,10 +269,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 
     if (strlen(newpass) < 6 || strstr(newpass, username) != NULL || flag != 3)
     {
-      fputs("lppasswd: Sorry, password rejected.\n"
-	    "Your password must be at least 6 characters long, cannot contain\n"
-	    "your username, and must contain at least one letter and number.\n",
-	    stderr);
+      _cupsLangPuts(stderr,
+                    _("lppasswd: Sorry, password rejected.\n"
+		      "Your password must be at least 6 characters long, "
+		      "cannot contain\n"
+		      "your username, and must contain at least one letter "
+		      "and number.\n"));
       return (1);
     }
   }
@@ -318,16 +320,20 @@ main(int  argc,			/* I - Number of command-line arguments */
   if ((fd = open(passwdnew, O_WRONLY | O_CREAT | O_EXCL, 0400)) < 0)
   {
     if (errno == EEXIST)
-      fputs("lppasswd: Password file busy!\n", stderr);
+      _cupsLangPuts(stderr, _("lppasswd: Password file busy!\n"));
     else
-      perror("lppasswd: Unable to open passwd file");
+      _cupsLangPrintf(stderr,
+                      _("lppasswd: Unable to open password file: %s\n"),
+		      strerror(errno));
 
     return (1);
   }
 
   if ((outfile = fdopen(fd, "w")) == NULL)
   {
-    perror("lppasswd: Unable to open passwd file");
+    _cupsLangPrintf(stderr,
+                    _("lppasswd: Unable to open password file: %s\n"),
+		    strerror(errno));
 
     unlink(passwdnew);
 
@@ -343,7 +349,9 @@ main(int  argc,			/* I - Number of command-line arguments */
   infile = fopen(passwdmd5, "r");
   if (infile == NULL && errno != ENOENT && op != ADD)
   {
-    perror("lppasswd: Unable to open password file");
+    _cupsLangPrintf(stderr,
+                    _("lppasswd: Unable to open password file: %s\n"),
+		    strerror(errno));
 
     fclose(outfile);
 
@@ -376,7 +384,9 @@ main(int  argc,			/* I - Number of command-line arguments */
 
       if (fputs(line, outfile) == EOF)
       {
-        perror("lppasswd: Unable to write to password file");
+	_cupsLangPrintf(stderr,
+                	_("lppasswd: Unable to write to password file: %s\n"),
+			strerror(errno));
         error = 1;
 	break;
       }
@@ -387,7 +397,9 @@ main(int  argc,			/* I - Number of command-line arguments */
       while (fgets(line, sizeof(line), infile) != NULL)
 	if (fputs(line, outfile) == EOF)
 	{
-          perror("lppasswd: Unable to write to password file");
+	  _cupsLangPrintf(stderr,
+                	  _("lppasswd: Unable to write to password file: %s\n"),
+			  strerror(errno));
 	  error = 1;
 	  break;
 	}
@@ -397,8 +409,9 @@ main(int  argc,			/* I - Number of command-line arguments */
   if (op == CHANGE &&
       (strcmp(username, userline) || strcmp(groupname, groupline)))
   {
-    fprintf(stderr, "lppasswd: user \"%s\" and group \"%s\" do not exist.\n",
-            username, groupname);
+    _cupsLangPrintf(stderr,
+                    _("lppasswd: user \"%s\" and group \"%s\" do not exist.\n"),
+        	    username, groupname);
     error = 1;
   }
   else if (op != DELETE)
@@ -406,7 +419,8 @@ main(int  argc,			/* I - Number of command-line arguments */
     if (oldpass &&
         strcmp(httpMD5(username, "CUPS", oldpass, md5new), md5line) != 0)
     {
-      fputs("lppasswd: Sorry, password doesn't match!\n", stderr);
+      _cupsLangPuts(stderr,
+                    _("lppasswd: Sorry, password doesn't match!\n"));
       error = 1;
     }
     else
@@ -415,7 +429,9 @@ main(int  argc,			/* I - Number of command-line arguments */
                httpMD5(username, "CUPS", newpass, md5new));
       if (fputs(line, outfile) == EOF)
       {
-        perror("lppasswd: Unable to write to password file");
+	_cupsLangPrintf(stderr,
+                	_("lppasswd: Unable to write to password file: %s\n"),
+			strerror(errno));
         error = 1;
       }
     }
@@ -437,7 +453,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   if (error)
   {
-    fputs("lppasswd: Password file not updated!\n", stderr);
+    _cupsLangPuts(stderr, _("lppasswd: Password file not updated!\n"));
     
     unlink(passwdnew);
 
@@ -449,9 +465,11 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   unlink(passwdold);
-  if (link(passwdmd5, passwdold))
+  if (link(passwdmd5, passwdold) && errno != ENOENT)
   {
-    perror("lppasswd: failed to backup old password file");
+    _cupsLangPrintf(stderr,
+                    _("lppasswd: failed to backup old password file: %s\n"),
+		    strerror(errno));
     unlink(passwdnew);
     return (1);
   }
@@ -462,7 +480,9 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   if (rename(passwdnew, passwdmd5) < 0)
   {
-    perror("lppasswd: failed to rename password file");
+    _cupsLangPrintf(stderr,
+                    _("lppasswd: failed to rename password file: %s\n"),
+		    strerror(errno));
     unlink(passwdnew);
     return (1);
   }
@@ -479,20 +499,17 @@ static void
 usage(FILE *fp)		/* I - File to send usage to */
 {
   if (getuid())
-  {
-    fputs("Usage: lppasswd [-g groupname]\n", fp);
-  }
+    _cupsLangPuts(fp, _("Usage: lppasswd [-g groupname]\n"));
   else
-  {
-    fputs("Usage: lppasswd [-g groupname] [username]\n", fp);
-    fputs("       lppasswd [-g groupname] -a [username]\n", fp);
-    fputs("       lppasswd [-g groupname] -x [username]\n", fp);
-  }
+    _cupsLangPuts(fp, 
+                  _("Usage: lppasswd [-g groupname] [username]\n"
+		    "       lppasswd [-g groupname] -a [username]\n"
+		    "       lppasswd [-g groupname] -x [username]\n"));
 
   exit(1);
 }
 
 
 /*
- * End of "$Id: lppasswd.c,v 1.18 2005/01/03 19:29:59 mike Exp $".
+ * End of "$Id: lppasswd.c 5926 2006-09-05 20:45:47Z mike $".
  */
